/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	testEth.c	LWIP configuration and test.
				This file purpose is to set up the Ethernet interface of STM32F429ZI card.
				Set up netif to address from testEth.h (IP_ADDRx), e.g. 192.168.0.129
				Pinging 192.168.0.129 allows to check if the connection is up

	When		Who		What
	29/07/19	R.Jacq	Create
	06/10/19	ac		Simplified and Formatted

----------------------------------------------------------------------
*/

/* Includes ------------------------------------------------------------------*/
#include "testEth.h"
#include "lwip/netif.h"
#include "lwip/tcpip.h"
#include "lwip/api.h"
#include "ethernetif.h"

#include "lwip/ip4_addr.h"

#include "aaprintf.h"
//#include "aalogmes.h"

#define TCPECHO_THREAD_PRIO		6u
#define UDPECHO_THREAD_PRIO		6u

#define ECHO_STACK_SIZE			192u

#define	TASK_FLAG				AA_FLAG_STACKCHECK		// To check stack usage


static	struct netif gnetif; 	/* The network interface structure */

static void		Netif_Config			(void) ;
static void		tcpEchoServerTask		(uintptr_t arg) ;
static void		udpEchoServerTask		(uintptr_t arg) ;

//--------------------------------------------------------------------------------
// Initializes the lwIP stack

static void Netif_Config (void)
{
	struct ip4_addr ipaddr ;
	struct ip4_addr netmask ;
	struct ip4_addr gw ;

	/* IP address setting */
	IP4_ADDR (& ipaddr,		IP_ADDR0,		IP_ADDR1,		IP_ADDR2,		IP_ADDR3) ;
	IP4_ADDR (& netmask,	NETMASK_ADDR0,	NETMASK_ADDR1,	NETMASK_ADDR2,	NETMASK_ADDR3) ;
	IP4_ADDR (& gw,			GW_ADDR0,		GW_ADDR1,		GW_ADDR2,		GW_ADDR3) ;

	/* Create tcp_ip stack thread */
	tcpip_init( NULL, NULL );

	/* - netif_add(struct netif *netif, struct ip_addr *ipaddr,
	struct ip_addr *netmask, struct ip_addr *gw,
	void *state, err_t (* init)(struct netif *netif),
	err_t (* input)(struct pbuf *p, struct netif *netif))

	Adds your network interface to the netif_list. Allocate a struct
	netif and pass a pointer to this structure as the first argument.
	Give pointers to cleared ip_addr structures when using DHCP,
	or fill them with sane numbers otherwise. The state pointer may be NULL.

	The init function pointer must point to a initialization function for
	your ethernet netif interface. The following code illustrates it's use.*/
	netif_add (& gnetif, & ipaddr, & netmask, & gw, NULL, & ethernetif_init, & tcpip_input) ;

	/*  Registers the default network interface. */
	netif_set_default (& gnetif) ;

	if (netif_is_link_up (& gnetif))
	{
		/* When the netif is fully configured this function must be called.*/
		netif_set_up (& gnetif) ;
	}
	else
	{
		/* When the netif link is down this function must be called */
		netif_set_down (& gnetif) ;
	}

	/* Set the link callback function, this function is called on change of link status*/
	netif_set_link_callback (& gnetif, ethernetif_update_config) ;

	/* Create the Ethernet link monitor thread */
	{
		uint8_t res;

		res = aaTaskCreate (
				DEFAULT_THREAD_PRIO,		// Task priority
				"tLwipLk",					// Task name
				ethernetif_check_link,		// Entry point,
				(uintptr_t) & gnetif,		// Entry point parameter
				NULL,						// Stack pointer
				100u,						// Stack size
				TASK_FLAG,					// Flags
				NULL) ;						// Created task id
		(void) res ;
/*
		if (res != AA_ENONE)
		{
			aaPrintf ("Error create ethernetif_set_link: %u\n", res);
		}
		else
		{
			aaPrintf ("ethernetif_set_link task OK\n");
		}
*/
	}
}

//--------------------------------------------------------------------------------
// This is the TCP echo server task
// It start a new socket connect to port 7 and listen to incoming connection
// As connection is received, read the recv buffer and send it back (like an echo)

static void tcpEchoServerTask (uintptr_t arg)
{
	struct netconn	* conn, * newconn;
    struct netbuf	* buf;
    void			* data;
    u16_t			len;
    err_t			err, accept_err;

uint32_t packetCount ;

    LWIP_UNUSED_ARG(arg);

	/* Create a new connection identifier. */
	conn = netconn_new (NETCONN_TCP);
	if (conn != NULL)
	{
		/* Bind connection to well known port number 7. */
		err = netconn_bind (conn, NULL, 7);
		if (err == ERR_OK)
		{
			/* Tell connection to go into listening mode. */
			netconn_listen (conn);
			while (1)
			{
				/* Grab new connection. */
				accept_err = netconn_accept (conn, & newconn);
aaPrintf ("TCP accept %08X\n", (uint32_t) newconn) ;
packetCount = 0 ;
				/* Process the new connection. */
				if (accept_err == ERR_OK)
				{
					while (netconn_recv (newconn, & buf) == ERR_OK)
					{
						do
						{
							netbuf_data (buf, & data, & len) ;
							netconn_write (newconn, data, len, NETCONN_COPY) ;
aaPrintf ("TCP %d  %d  \n", ++packetCount, len) ;
						} while (netbuf_next(buf) >= 0) ;

						netbuf_delete(buf);
					}
					/* Close connection and discard connection identifier. */
aaPrintf ("TCP close %08X\n", (uint32_t) newconn) ;
					netconn_close  (newconn) ;
					netconn_delete (newconn) ;
				}
			}
		}
		else
		{
			aaPrintf ("tcpEchoServerTask: netconn_bind error\n") ;
			netconn_delete(newconn);
		}
	}
	else
	{
		aaPrintf ("tcpEchoServerTask: netconn_new error\n") ;
		netconn_delete (conn) ;
	}
}

//--------------------------------------------------------------------------------

static void udpEchoServerTask (uintptr_t arg)
{
	struct netconn		* conn ;
	struct netbuf		* buf ;
	ip_addr_t			* addr ;
	uint16_t			port ;
	err_t				err, recv_err ;

	LWIP_UNUSED_ARG(arg) ;

	conn = netconn_new (NETCONN_UDP) ;
	if (conn != NULL)
	{
		err = netconn_bind (conn, IP_ADDR_ANY, 7) ;
		if (err == ERR_OK)
		{
aaPrintf ("UDP bind OK\n") ;
			while (1)
			{
				recv_err = netconn_recv (conn, & buf) ;
aaPrintf (" recv %d\n", recv_err) ;

				if (recv_err == ERR_OK)
				{
					addr = netbuf_fromaddr (buf) ;
					port = netbuf_fromport (buf) ;
aaPrintf (" P %08X %u\n", * (uint32_t *) addr, port) ;
					netconn_connect (conn, addr, port) ;
					buf->addr.addr = 0 ;
					netconn_send (conn,buf) ;
					netbuf_delete (buf) ;
				}
			}
		}
		else
		{
			aaPrintf ("udpEchoServerTask: netconn_bind error %d\n") ;
			netconn_delete (conn) ;
		}
	}
	else
	{
		aaPrintf ("udpEchoServerTask: netconn_new error\n") ;
		netconn_delete (conn) ;
	}
}

//--------------------------------------------------------------------------------
//	This callback notify the user of Ethernet link change (connect/disconnect the cable)

void ethernetif_notify_conn_changed (struct netif * pNetif)
{
	aaPrintf ("Ethernet link is %s\n", (netif_is_link_up (pNetif) == 0) ? "Down" : "Up") ;
}

//--------------------------------------------------------------------------------
//	The Ethernet setting
//  First initialize lwip tcpip stack and net interface config
//  Then create a task to handle TCP packets (TCP echo server)
//  Then create a task to handle UDP packets (UDP echo server)

void	testEth (void)
{
	aaTaskId_t taskId;
	uint8_t res;

	/* Network interface configuration */
	Netif_Config();

	/* Initialize tcp echo server */
	res = aaTaskCreate (
			TCPECHO_THREAD_PRIO,	// Task priority
			"tcpEcho",				// Task name
			tcpEchoServerTask,		// Entry point,
			0,						// Entry point parameter
			NULL,					// Stack pointer
			ECHO_STACK_SIZE,		// Stack size
			TASK_FLAG,				// Flags
			& taskId);				// Created task id
	if (res != AA_ENONE)
	{
		aaPrintf ("Error create TCP Echo Server task: %u\n", res);
	}
	else
	{
		aaPrintf ("TCP Echo Server task OK\n");
	}

	/* Initialize udp echo server*/
	res = aaTaskCreate (
			UDPECHO_THREAD_PRIO,	// Task priority
			"udpEcho",				// Task name
			udpEchoServerTask,		// Entry point,
			0,						// Entry point parameter
			NULL,					// Stack pointer
			ECHO_STACK_SIZE,		// Stack size
			TASK_FLAG,				// Flags
			& taskId);				// Created task id
	if (res != AA_ENONE)
	{
		aaPrintf ("Error create UDP Echo Server task: %u\n", res);
	}
	else
	{
		aaPrintf ("UDP Echo Server task OK\n");
	}
}

//--------------------------------------------------------------------------------

