/*
----------------------------------------------------------------------
	
	AdAstra - Real Time Kernel

	Alain Chebrou

	userInitTask.c	The first task

	When		Who	What
	25/11/13	ac	Creation
	27/01/22	ac	Add aaSpuriousInterruptHandler() example
					Add HardFault_Handler_C()
	09/06/22	ac	add BSP_ATTR_ALIGN(8) to ledStack

----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aalogmes.h"
#include	"aaprintf.h"
#include	"aautils.h"		// For displaytaskInfo()

#include	"testEth.h"		// For LWIP test
#include	"test.h"		// For basic peripheral libraries tests

//--------------------------------------------------------------------------------
//	Demo : Blink LEDs and test user button
//	User BSP_LED0 and BSP_BUTTON0 because most boards have at least a user LED and a user button

// Define the LEDs task stack, to avoid use of dynamic memory
#define	LED_STACK_SIZE	64u
static	bspStackType_t	ledStack [LED_STACK_SIZE] BSP_ATTR_ALIGN(8) BSP_ATTR_NOINIT ;

void	ledsTask (uintptr_t arg)
{
	(void) arg ;

	while (1)
	{
		// Blink the LED at 1Hz
		bspOutput (BSP_LED0, BSP_LED0_ON) ;
		aaTaskDelay (bspGetTickRate () / 2u) ;

		// Blink the LED as long as the button is not pressed
		if (bspInput (BSP_BUTTON0) != BSP_BUTTON0_PRESSED)
		{
			bspOutput (BSP_LED0, BSP_LED0_OFF) ;
			aaTaskDelay (bspGetTickRate () / 2u) ;
		}
	}
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	The first user task

//	The C++ entry, for demo purpose.
//	You can start your C++ code from there
//	Have a look to: https://isocpp.org/wiki/faq/mixing-c-and-cpp
void	cppEntry (uintptr_t arg) ;

static	aaTaskInfo_t	taskInfo [AA_TASK_MAX] ;

void	userInitTask (uintptr_t arg)
{
	// Remove all the lines that follow, and replace them with your application
	// If your application is entirely C++ then call cppEntry()
	// cppEntry (arg) ;

	(void) arg ;		// Unused arg

	// Create a task to blink the LED
	// With the highest task priority for maximum steadiness
	aaTaskCreate (
			AA_PRIO_MAX,		// Task priority
			"tLeds",			// Task name
			ledsTask,			// Entry point,
			0,					// Entry point parameter
			ledStack,			// Stack pointer
			LED_STACK_SIZE,		// Stack size
			AA_FLAG_STACKCHECK,	// Flags
			NULL) ;				// Created task id

	testEth () ;	// This configure LWIP and start TCP and UDP echo tasks
					// Configure the MAC address in stm32f4xx_hal_conf.h
					// and the static IP and gateway addresses in testEth.h



	// Use console: echo received char, assume AA_WITH_CONSOLE is 1 (console enabled)
	{
		char		cc ;

		while (1)
		{
			aaPrintf ("User task, t to execute the test\n") ;
			cc = aaGetChar () ;
			aaPutChar (cc) ;
			aaPutChar ('\n') ;

			if (cc == 'q')
			{
				break ;	// End of test, kill this task
			}

			if (cc == 'k')
			{
				aaPrintf ("Main stack free: %u\n", bspMainStackCheck ())  ;
			}

			if (cc == '+')
			{
				// To start the C++ application: call cppEntry()
				cppEntry (arg) ;
			}

			if (cc == 't')
			{
				// Go to the test menu defined in test.c
				tests () ;
				continue ;
			}

			#if (AA_WITH_MALLOC_TLSF == 1)
			{
				if (cc == 'm')
				{
					// Check dynamic memory partition
					{
						aaMemPoolCheck (1) ;
					}
					continue ;
				}
			}
			#endif

			// Display amount of free space
			#if (AA_WITH_MALLOC_BLOC == 1)
			{
				if (cc == 'b')
				{
					aaPrintf ("Malloc Block free space %d\n", aaMallocSize ()) ;
					continue ;
				}
			}
			#endif

			if (cc == 'u')
			{
				// Display tasks information
				// To enable stack check, add AA_FLAG_STACKCHECK flag to each task creation
				displaytaskInfo (taskInfo) ;
				continue ;
			}

			if (cc == 'c')
			{
				// Clear task counters
				aaTaskStatClear () ;
				continue ;
			}

			// Use aaLogMes() to see if SWO is enabled (if DEBUG and running with debugger)
			aaLogMes ("log! %c\n", cc, 0, 0, 0, 0) ;
		}
	}

	aaPrintf ("Exit user task\n") ;
	aaTaskDelete (AA_SELFTASKID) ;	// Useless, example of use.
}

//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
//	Manage kernel notifications

void	aaUserNotify (uint32_t	event, uintptr_t arg)
{
	switch (event)
	{
		case AA_NOTIFY_STACKOVFL:	// Stack overflow
			AA_ASSERT (0) ;
			break ;

		case AA_NOTIFY_STACKTHR:	// Stack threshold level is reached
			AA_ASSERT (0) ;
			break ;

		default:
			break ;
	}
	(void) arg ;
}

//--------------------------------------------------------------------------------
//	Display spurious interrupt info on the console
//	The processor jumps here if an unexpected interrupt occurs
//	or a specific handler is not present in the application code

#include "bspcfg.h"			// For bspDisableIrq()
#include "uartbasic.h"

void __attribute__ ((section(".isr_handlers"))) aaSpuriousInterruptHandler (void)
{
	aaDevIo_t	* hDevice ;
	uint32_t	itNum ;

	bspDisableIrqAll () ;

	hDevice = aaGetDefaultConsole () ;
	if (hDevice != NULL)
	{
		// Disable UART interrupt management
		uartSetFlag (hDevice->hDev, UART_FLAG_NOIT, 1u) ;
		// Display the real interrupt number
		itNum = SCB->ICSR & SCB_ICSR_VECTACTIVE_Msk ;
		aaPrintf ("Spurious Interrupt: %u-16=%u\n", itNum, itNum-16u) ;
	}

	BSP_DEBUG_BKPT() ;
}

//--------------------------------------------------------------------------------
//	Display information about hard fault

//	Uncomment this function to replace the default HardFault handler.

// 	For more information:
//	https://github.com/tobermory/faultHandling-cortex-m

typedef struct
{
	uint32_t	r0 ;
	uint32_t	r1 ;
	uint32_t	r2 ;
	uint32_t	r3 ;
	uint32_t	r12 ;
	uint32_t	lr ;
	uint32_t	pc ;
	uint32_t	psr ;
	#if  defined(__ARM_ARCH_7EM__)
		uint32_t s[16];				// Floating point registers
	#endif

} exceptionStackFrame ;

void __attribute__ ((used)) HardFault_Handler_C (exceptionStackFrame * frame, uint32_t lr)
{
	aaDevIo_t	* hDevice ;

	bspDisableIrqAll () ;

	// Check if debug is enabled.
	// If debug is enabled, use bkpt instruction to jump to debugger
	// If debug is not enabled, a bkpt instruction triggers a HardFault exception
	if ((CoreDebug->DHCSR & CoreDebug_DHCSR_C_DEBUGEN_Msk) != 0u)
	{
		// Debug enabled, can use BKPT instruction
		BSP_DEBUG_BKPT () ;
	}

	// If no active debugger displays info on the console

	hDevice = aaGetDefaultConsole () ;
	if (hDevice != NULL)
	{
		// Disable UART interrupt management
		uartSetFlag (hDevice->hDev, UART_FLAG_NOIT, 1u) ;

		uint32_t	mmfar	= SCB->MMFAR ;	// MemManage Fault Address
		uint32_t	bfar 	= SCB->BFAR ;	// Bus Fault Address

		aaPrintf ("HardFault:\n") ;
		aaPrintf ("R0     %08X\n", frame->r0) ;
		aaPrintf ("R1     %08X\n", frame->r1) ;
		aaPrintf ("R2     %08X\n", frame->r2) ;
		aaPrintf ("R3     %08X\n", frame->r3) ;
		aaPrintf ("R12    %08X\n", frame->r12) ;
		aaPrintf ("LR     %08X\n", frame->lr) ;
		aaPrintf ("PC     %08X\n", frame->pc) ;
		aaPrintf ("PSR    %08X\n\n", frame->psr) ;
		aaPrintf ("CFSR   %08X\n", SCB->CFSR) ;		// Configurable Fault Status Registers
		aaPrintf ("HFSR   %08X\n", SCB->HFSR) ; 	// Hard Fault Status register
		aaPrintf ("DFSR   %08X\n", SCB->DFSR) ;		// Debug Fault Status register
		aaPrintf ("AFSR   %08X\n", SCB->AFSR) ; 	// Auxiliary Fault Status register
		aaPrintf ("MMFAR  %08X %c\n", mmfar, (SCB->CFSR & SCB_CFSR_MMARVALID_Msk) != 0u ? '1' : '0') ;
		aaPrintf ("BFAR   %08X %c\n", bfar,  (SCB->CFSR & SCB_CFSR_BFARVALID_Msk) != 0u ? '1' : '0') ;
		aaPrintf ("LR/EXC_RETURN %08X\n", lr) ;
	}

	// Here you can generate an external signal, like turning on a LED
	while (1)
	{
	}
}

//--------------------------------------------------------------------------------
