/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	SystemClock_Config.c		The function to initialize MCU clocks
								Example for:
									STM32F469/479
									STM32F427/437
									STM32F429/439
									STM32F405/415
									STM32F407/417
									STM32F446
									STM32F411/401
								 These functions are examples and should be changed as needed
	When		Who	What
	21/01/20	ac	Created
	19/05/20	ac	Add examples for other MCU than F429
	05/07/21	ac	Remove LSE and RTC enable

----------------------------------------------------------------------
*/

#include	"aa.h"		// For BSP_DEBUG_BKPT

#include	"stm32f4xx_ll_rcc.h"
#include	"stm32f4xx_ll_system.h"
#include	"stm32f4xx_ll_pwr.h"
#include	"stm32f4xx_ll_bus.h"

static	void   Error_Handler	(void) ;

// ----------------------------------------------------------------------------
//	SystemClock_Config() may be generated by STM32CubeMX to configure the clock in accordance with your needs
//	You can use CubeMX to generate the LL code, and copy SystemClock_Config() here.

//	This function must:
//		Not initialize SysTick
//		Initialize peripheral clocks
//		Set TIMPRE if requested
//		LSE and RTC initialization are complex, it is better not to do it here

// ----------------------------------------------------------------------------
#if (defined STM32F427xx  ||  defined STM32F429xx  ||  defined STM32F437xx  ||  defined STM32F439xx)
// ----------------------------------------------------------------------------
//	Use 8 MHz HSE as system clock source, system clock is 168 MHz
//	Requires a solder jumper on NUCLEO board (default is closed) to get the MCO output from the STLINK MCU
//	48MHz for USB FS

void SystemClock_Config (void)
{
	LL_FLASH_SetLatency(LL_FLASH_LATENCY_5);

	if(LL_FLASH_GetLatency() != LL_FLASH_LATENCY_5)
	{
		Error_Handler () ;
	}

	LL_PWR_SetRegulVoltageScaling(LL_PWR_REGU_VOLTAGE_SCALE1);
#if defined(PWR_CR_ODSWEN)
	LL_PWR_DisableOverDriveMode();		// For F42x and F43x only
#endif

	// Start HSI	(MCO test)
	LL_RCC_HSI_SetCalibTrimming(16);
	LL_RCC_HSI_Enable();
	while(LL_RCC_HSI_IsReady() != 1)
	{
	}

	// Start HSE
	LL_RCC_HSE_EnableBypass();
	LL_RCC_HSE_Enable();
	while(LL_RCC_HSE_IsReady() != 1)
	{
	}

	LL_RCC_PLL_ConfigDomain_SYS(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_4, 168, LL_RCC_PLLP_DIV_2);
	LL_RCC_PLL_ConfigDomain_48M(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_4, 168, LL_RCC_PLLQ_DIV_7);
	LL_RCC_PLL_Enable();

	// Wait till PLL is ready
	while(LL_RCC_PLL_IsReady() != 1)
	{
	}

	LL_RCC_SetAHBPrescaler	(LL_RCC_SYSCLK_DIV_1);
	LL_RCC_SetAPB1Prescaler	(LL_RCC_APB1_DIV_4);
	LL_RCC_SetAPB2Prescaler	(LL_RCC_APB2_DIV_2);
	LL_RCC_SetSysClkSource	(LL_RCC_SYS_CLKSOURCE_PLL);

	// Wait till System clock is ready
	while(LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_PLL)
	{
	}
	LL_RCC_SetTIMPrescaler(LL_RCC_TIM_PRESCALER_TWICE);
}

// ----------------------------------------------------------------------------
//	The purpose of this function is to provide a prologue and an epilogue to SystemClock_Config().

void	bspSystemClockInit ()
{
	// These initialization may be required if the application configures LSE, or other
	LL_APB2_GRP1_EnableClock (LL_APB2_GRP1_PERIPH_SYSCFG) ;
	LL_APB1_GRP1_EnableClock (LL_APB1_GRP1_PERIPH_PWR) ;

	SystemClock_Config () ;

	// Stop SysTick if it is started by SystemClock_Config()
	SysTick->CTRL = 0 ;
}

// ----------------------------------------------------------------------------
#elif (defined STM32F405xx  ||  defined STM32F415xx  ||  defined STM32F407xx  ||  defined STM32F417xx)
// ----------------------------------------------------------------------------
//	Use 8 MHz HSE as system clock source, system clock is 168 MHz

void SystemClock_Config(void)
{
  LL_FLASH_SetLatency(LL_FLASH_LATENCY_5);

  if(LL_FLASH_GetLatency() != LL_FLASH_LATENCY_5)
  {
	  Error_Handler();
  }
  LL_PWR_SetRegulVoltageScaling(LL_PWR_REGU_VOLTAGE_SCALE1);
  LL_RCC_HSE_EnableBypass();
  LL_RCC_HSE_Enable();

   /* Wait till HSE is ready */
  while(LL_RCC_HSE_IsReady() != 1)
  {
  }

  LL_RCC_PLL_ConfigDomain_SYS(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_4, 168, LL_RCC_PLLP_DIV_2);
  LL_RCC_PLL_ConfigDomain_48M(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_4, 168, LL_RCC_PLLQ_DIV_7);
  LL_RCC_PLL_Enable();

   /* Wait till PLL is ready */
  while(LL_RCC_PLL_IsReady() != 1)
  {
  }
  LL_RCC_SetAHBPrescaler(LL_RCC_SYSCLK_DIV_1);
  LL_RCC_SetAPB1Prescaler(LL_RCC_APB1_DIV_4);
  LL_RCC_SetAPB2Prescaler(LL_RCC_APB2_DIV_2);
  LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_PLL);

   /* Wait till System clock is ready */
  while(LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_PLL)
  {
  }
}

// ----------------------------------------------------------------------------
//	The purpose of this function is to provide a prologue and an epilogue to SystemClock_Config().

void	bspSystemClockInit ()
{
	// These initialization may be required if the application configures LSE, or other
	LL_APB2_GRP1_EnableClock (LL_APB2_GRP1_PERIPH_SYSCFG) ;
	LL_APB1_GRP1_EnableClock (LL_APB1_GRP1_PERIPH_PWR) ;

	SystemClock_Config () ;

	// Stop SysTick if it is started by SystemClock_Config()
	SysTick->CTRL = 0 ;
}

// ----------------------------------------------------------------------------
#elif (defined STM32F446xx)
// ----------------------------------------------------------------------------
//	Use 8 MHz HSE as system clock source, system clock is 168 MHz
//	Required solder jump on NUCLEO board (default is closed) to get the MCO output from the STLINK MCU
//	USB clock configured to 48 MHz
//	MCO1 and MCO2 configured

void SystemClock_Config(void)
{
  LL_FLASH_SetLatency(LL_FLASH_LATENCY_5);

  if(LL_FLASH_GetLatency() != LL_FLASH_LATENCY_5)
  {
	  Error_Handler();
  }
  LL_PWR_SetRegulVoltageScaling(LL_PWR_REGU_VOLTAGE_SCALE1);
  LL_PWR_DisableOverDriveMode();
  LL_RCC_HSE_EnableBypass();
  LL_RCC_HSE_Enable();

   /* Wait till HSE is ready */
  while(LL_RCC_HSE_IsReady() != 1)
  {
  }

  LL_RCC_PLL_ConfigDomain_SYS(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_4, 168, LL_RCC_PLLP_DIV_2);
  LL_RCC_PLL_ConfigDomain_48M(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_4, 168, LL_RCC_PLLQ_DIV_7);
  LL_RCC_PLL_Enable();

   /* Wait till PLL is ready */
  while(LL_RCC_PLL_IsReady() != 1)
  {
  }
  LL_RCC_SetAHBPrescaler(LL_RCC_SYSCLK_DIV_1);
  LL_RCC_SetAPB1Prescaler(LL_RCC_APB1_DIV_4);
  LL_RCC_SetAPB2Prescaler(LL_RCC_APB2_DIV_2);
  LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_PLL);

   /* Wait till System clock is ready */
  while(LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_PLL)
  {
  }

  LL_RCC_ConfigMCO(LL_RCC_MCO1SOURCE_PLLCLK, LL_RCC_MCO1_DIV_4);
  LL_RCC_ConfigMCO(LL_RCC_MCO2SOURCE_HSE, LL_RCC_MCO2_DIV_1);
  LL_RCC_SetUSBClockSource(LL_RCC_USB_CLKSOURCE_PLL);
  LL_RCC_SetTIMPrescaler(LL_RCC_TIM_PRESCALER_TWICE);
}

// ----------------------------------------------------------------------------
//	The purpose of this function is to provide a prologue and an epilogue to SystemClock_Config().

void	bspSystemClockInit ()
{
	// These initialization may be required if the application configures LSE, or other
	LL_APB2_GRP1_EnableClock (LL_APB2_GRP1_PERIPH_SYSCFG) ;
	LL_APB1_GRP1_EnableClock (LL_APB1_GRP1_PERIPH_PWR) ;

	SystemClock_Config () ;

	// Stop SysTick if it is started by SystemClock_Config()
	SysTick->CTRL = 0 ;
}

// ----------------------------------------------------------------------------
#elif (defined STM32F469xx  ||  defined STM32F479xx)
// ----------------------------------------------------------------------------
//	Use 8 MHz HSE as system clock source, system clock is 168 MHz
//	USB clock configured to 48 MHz
//	MCO1 and MCO2 configured

void SystemClock_Config(void)
{
  LL_FLASH_SetLatency(LL_FLASH_LATENCY_5);

  if(LL_FLASH_GetLatency() != LL_FLASH_LATENCY_5)
  {
	  Error_Handler();
  }
  LL_PWR_SetRegulVoltageScaling(LL_PWR_REGU_VOLTAGE_SCALE1);
  LL_PWR_DisableOverDriveMode();
  LL_RCC_HSE_EnableBypass();
  LL_RCC_HSE_Enable();

   /* Wait till HSE is ready */
  while(LL_RCC_HSE_IsReady() != 1)
  {
  }

  LL_RCC_PLL_ConfigDomain_SYS(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_4, 168, LL_RCC_PLLP_DIV_2);
  LL_RCC_PLL_ConfigDomain_48M(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_4, 168, LL_RCC_PLLQ_DIV_7);
  LL_RCC_PLL_Enable();

   /* Wait till PLL is ready */
  while(LL_RCC_PLL_IsReady() != 1)
  {
  }
  LL_RCC_SetAHBPrescaler(LL_RCC_SYSCLK_DIV_1);
  LL_RCC_SetAPB1Prescaler(LL_RCC_APB1_DIV_4);
  LL_RCC_SetAPB2Prescaler(LL_RCC_APB2_DIV_2);
  LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_PLL);

   /* Wait till System clock is ready */
  while(LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_PLL)
  {
  }
  LL_RCC_ConfigMCO(LL_RCC_MCO1SOURCE_PLLCLK, LL_RCC_MCO1_DIV_4);
  LL_RCC_ConfigMCO(LL_RCC_MCO2SOURCE_HSE, LL_RCC_MCO2_DIV_1);
  LL_RCC_SetSDIOClockSource(LL_RCC_SDIO_CLKSOURCE_PLL48CLK);
  LL_RCC_SetCK48MClockSource(LL_RCC_CK48M_CLKSOURCE_PLL);
  LL_RCC_SetUSBClockSource(LL_RCC_USB_CLKSOURCE_PLL);
  LL_RCC_SetTIMPrescaler(LL_RCC_TIM_PRESCALER_TWICE);
}
// ----------------------------------------------------------------------------
//	The purpose of this function is to provide a prologue and an epilogue to SystemClock_Config().

void	bspSystemClockInit ()
{
	// These initialization may be required if the application configures LSE, or other
	LL_APB2_GRP1_EnableClock (LL_APB2_GRP1_PERIPH_SYSCFG) ;
	LL_APB1_GRP1_EnableClock (LL_APB1_GRP1_PERIPH_PWR) ;

	SystemClock_Config () ;

	// Stop SysTick if it is started by SystemClock_Config()
	SysTick->CTRL = 0 ;
}

// ----------------------------------------------------------------------------
#elif (defined STM32F411xE)
// ----------------------------------------------------------------------------
#if 0
//	Use 25 MHz quartz HSE as system clock source, system clock is 40 MHz

void SystemClock_Config(void)
{
  LL_FLASH_SetLatency(LL_FLASH_LATENCY_1);
  while(LL_FLASH_GetLatency()!= LL_FLASH_LATENCY_1)
  {
  }
  LL_PWR_SetRegulVoltageScaling(LL_PWR_REGU_VOLTAGE_SCALE1);
  LL_RCC_HSE_Enable();

   /* Wait till HSE is ready */
  while(LL_RCC_HSE_IsReady() != 1)
  {

  }
  LL_PWR_EnableBkUpAccess();
  LL_RCC_LSE_Enable();

   /* Wait till LSE is ready */
  while(LL_RCC_LSE_IsReady() != 1)
  {

  }
  LL_RCC_PLL_ConfigDomain_SYS(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_15, 96, LL_RCC_PLLP_DIV_2);
  LL_RCC_PLL_Enable();

   /* Wait till PLL is ready */
  while(LL_RCC_PLL_IsReady() != 1)
  {

  }
  LL_RCC_SetAHBPrescaler(LL_RCC_SYSCLK_DIV_2);
  LL_RCC_SetAPB1Prescaler(LL_RCC_APB1_DIV_2);
  LL_RCC_SetAPB2Prescaler(LL_RCC_APB2_DIV_1);
  LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_PLL);

   /* Wait till System clock is ready */
  while(LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_PLL)
  {
  }

  LL_RCC_ConfigMCO(LL_RCC_MCO1SOURCE_LSE, LL_RCC_MCO1_DIV_1);
  LL_RCC_SetTIMPrescaler(LL_RCC_TIM_PRESCALER_TWICE);
}
#endif	// 0

#if 1
//	Use 25 MHz quartz HSE as system clock source, system clock is 96 MHz
//	HSI is started
//	USB clock configured to 48 MHz
//	I2S clock is 176.4 MHz  (For 44.1 KHz sampling)
//	This is not a low power configuration!

void SystemClock_Config(void)
{
  LL_FLASH_SetLatency(LL_FLASH_LATENCY_3);

  if(LL_FLASH_GetLatency() != LL_FLASH_LATENCY_3)
  {
	  Error_Handler();
  }
  LL_PWR_SetRegulVoltageScaling(LL_PWR_REGU_VOLTAGE_SCALE1);
  LL_RCC_HSE_Enable();

  /* Wait till HSE is ready */
  while(LL_RCC_HSE_IsReady() != 1)
  {
  }

  LL_RCC_HSI_SetCalibTrimming(16);
  LL_RCC_HSI_Enable();

  /* Wait till HSI is ready */
  while(LL_RCC_HSI_IsReady() != 1)
  {
  }

  LL_RCC_PLL_ConfigDomain_SYS(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_25, 192, LL_RCC_PLLP_DIV_2);
  LL_RCC_PLL_ConfigDomain_48M(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_25, 192, LL_RCC_PLLQ_DIV_4);
  LL_RCC_PLLI2S_ConfigDomain_I2S(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLI2SM_DIV_18, 254, LL_RCC_PLLI2SR_DIV_2);
  LL_RCC_PLL_Enable();

  /* Wait till PLL is ready */
  while(LL_RCC_PLL_IsReady() != 1)
  {
  }
  LL_RCC_PLLI2S_Enable();

  /* Wait till PLLI2S is ready */
  while(LL_RCC_PLLI2S_IsReady() != 1)
  {
  }

  LL_RCC_SetAHBPrescaler(LL_RCC_SYSCLK_DIV_1);
  LL_RCC_SetAPB1Prescaler(LL_RCC_APB1_DIV_2);
  LL_RCC_SetAPB2Prescaler(LL_RCC_APB2_DIV_1);
  LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_PLL);

  /* Wait till System clock is ready */
  while(LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_PLL)
  {
  }

  LL_RCC_SetI2SClockSource(LL_RCC_I2S1_CLKSOURCE_PLLI2S);
  LL_RCC_SetTIMPrescaler(LL_RCC_TIM_PRESCALER_TWICE);
}
#endif	// 0

// ----------------------------------------------------------------------------
//	The purpose of this function is to provide a prologue and an epilogue to SystemClock_Config().

void	bspSystemClockInit ()
{
	// These initialization may be required if the application configures LSE, or other
	LL_APB2_GRP1_EnableClock (LL_APB2_GRP1_PERIPH_SYSCFG) ;
	LL_APB1_GRP1_EnableClock (LL_APB1_GRP1_PERIPH_PWR) ;

	SystemClock_Config () ;

	// Stop SysTick if it is started by SystemClock_Config()
	SysTick->CTRL = 0 ;
}

// ----------------------------------------------------------------------------
#elif (defined STM32F401xC  ||  defined STM32F401xE)
// ----------------------------------------------------------------------------
//	Use 8 MHz HSE as system clock source, system clock is 72 MHz
//	HSI is started
//	USB clock configured to 48 MHz
//	MCO configured

void SystemClock_Config(void)
{
  LL_FLASH_SetLatency(LL_FLASH_LATENCY_2);

   if(LL_FLASH_GetLatency() != LL_FLASH_LATENCY_2)
  {
  Error_Handler();
  }
  LL_PWR_SetRegulVoltageScaling(LL_PWR_REGU_VOLTAGE_SCALE2);
  LL_RCC_HSE_Enable();

   /* Wait till HSE is ready */
  while(LL_RCC_HSE_IsReady() != 1)
  {

  }
  LL_RCC_HSI_SetCalibTrimming(16);
  LL_RCC_HSI_Enable();

   /* Wait till HSI is ready */
  while(LL_RCC_HSI_IsReady() != 1)
  {

  }
  LL_RCC_LSI_Enable();

   /* Wait till LSI is ready */
  while(LL_RCC_LSI_IsReady() != 1)
  {

  }
  LL_PWR_EnableBkUpAccess();
  LL_RCC_ForceBackupDomainReset();
  LL_RCC_ReleaseBackupDomainReset();
  LL_RCC_SetRTCClockSource(LL_RCC_RTC_CLKSOURCE_LSI);
  LL_RCC_EnableRTC();
  LL_RCC_PLL_ConfigDomain_SYS(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_4, 72, LL_RCC_PLLP_DIV_2);
  LL_RCC_PLL_ConfigDomain_48M(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_4, 72, LL_RCC_PLLQ_DIV_3);
  LL_RCC_PLL_Enable();

   /* Wait till PLL is ready */
  while(LL_RCC_PLL_IsReady() != 1)
  {

  }
  LL_RCC_SetAHBPrescaler(LL_RCC_SYSCLK_DIV_1);
  LL_RCC_SetAPB1Prescaler(LL_RCC_APB1_DIV_2);
  LL_RCC_SetAPB2Prescaler(LL_RCC_APB2_DIV_1);
  LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_PLL);

   /* Wait till System clock is ready */
  while(LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_PLL)
  {

  }

  LL_RCC_ConfigMCO(LL_RCC_MCO1SOURCE_HSI, LL_RCC_MCO1_DIV_1);
  LL_RCC_SetTIMPrescaler(LL_RCC_TIM_PRESCALER_TWICE);
}
// ----------------------------------------------------------------------------
//	The purpose of this function is to provide a prologue and an epilogue to SystemClock_Config().

void	bspSystemClockInit ()
{
	// These initialization may be required if the application configures LSE, or other
	LL_APB2_GRP1_EnableClock (LL_APB2_GRP1_PERIPH_SYSCFG) ;
	LL_APB1_GRP1_EnableClock (LL_APB1_GRP1_PERIPH_PWR) ;

	SystemClock_Config () ;

	// Stop SysTick if it is started by SystemClock_Config()
	SysTick->CTRL = 0 ;
}

#else
	#error	"MCU not  defined"
#endif
// ----------------------------------------------------------------------------
//	This function is used by CubeMX generated code

BSP_ATTR_USED static	void   Error_Handler	(void)
{
	BSP_DEBUG_BKPT () ;
	while (1)
	{
	}
}

// ----------------------------------------------------------------------------
