/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	itm.c		ARM SWO trace for F1/F4/L4

	When		Who	What
	11/07/17	ac	Creation

	// Switch to SWD without debugger:
	http://infocenter.arm.com/help/index.jsp?topic=/com.arm.doc.ddi0316d/Chdhfbhc.html

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

//	SWO is available only in privileged mode

#include	"aa.h"
#include	"aakernel.h"

#if (defined STM32F4)
	#include	"stm32f4xx.h"
#elif (defined STM32L4)
	#include	"stm32l4xx.h"
#elif (defined STM32F1)
	#include	"stm32f1xx.h"
#endif

#include	"itm.h"

//-------------------------------------------------------------------------------------------------

static	uint32_t	bItmAvailable ;		// 1 if SWO enabled (by debugger), else 0

//-------------------------------------------------------------------------------------------------
/*
	Initialize the SWO trace port for debug message printing
	portMask : Stimulus bit mask to be configured
	cpuCoreFreqHz : CPU core clock frequency in Hz
	baudrate : SWO frequency in Hz
*/

void swoInit (uint32_t portMask, uint32_t cpuCoreFreqHz, uint32_t baudrate)
{
	uint32_t SWOPrescaler = (cpuCoreFreqHz / baudrate) - 1u ; // baudrate in Hz, note that cpuCoreFreqHz is expected to match the CPU core clock

	CoreDebug->DEMCR = CoreDebug_DEMCR_TRCENA_Msk; 		// Debug Exception and Monitor Control Register (DEMCR): enable trace in core debug
	DBGMCU->CR	= 0x00000027u ;							// DBGMCU_CR : TRACE_IOEN DBG_STANDBY DBG_STOP 	DBG_SLEEP
	TPI->SPPR	= 0x00000002u ;							// Selected PIN Protocol Register: Select which protocol to use for trace output (2: SWO)
	TPI->ACPR	= SWOPrescaler ;						// Async Clock Prescaler Register: Scale the baud rate of the asynchronous output
	ITM->LAR	= 0xC5ACCE55u ;							// ITM Lock Access Register: C5ACCE55 enables more write access to Control Register 0xE00 :: 0xFFC
	ITM->TCR	= 0x0001000Du ;							// ITM Trace Control Register
	ITM->TPR	= ITM_TPR_PRIVMASK_Msk ;				// ITM Trace Privilege Register: All stimulus ports
	ITM->TER	= portMask ;							// ITM Trace Enable Register: Enabled tracing on stimulus ports. One bit per stimulus port.
	DWT->CTRL	= 0x400003FEu ;							// Data Watchpoint and Trace Register
	TPI->FFCR	= 0x00000100u ;							// Formatter and Flush Control Register

	// ITM/SWO works only if enabled from debugger.
	// If ITM stimulus 0 is not free, don't try to send data to SWO
	if (ITM->PORT [0].u8 == 1)
	{
		bItmAvailable = 1 ;
	}

// The raw version from SEGGER (http://forum.segger.com/index.php?page=Thread&postID=1749)
#if defined DUMMY
	SWOSpeed = 6000000;
	SWOPrescaler = (72000000 / SWOSpeed) - 1;  // SWOSpeed in Hz
	*((volatile unsigned *)0xE000EDFC) = 0x01000000;   // "Debug Exception and Monitor Control Register (DEMCR)"
	*((volatile unsigned *)0xE0042004) = 0x00000027;
	*((volatile unsigned *)0xE00400F0) = 0x00000002;   // "Selected PIN Protocol Register": Select which protocol to use for trace output (2: SWO)
	*((volatile unsigned *)0xE0040010) = SWOPrescaler; // "Async Clock Prescaler Register". Scale the baud rate of the asynchronous output
	*((volatile unsigned *)0xE0000FB0) = 0xC5ACCE55;   // ITM Lock Access Register, C5ACCE55 enables more write access to Control Register 0xE00 :: 0xFFC
	*((volatile unsigned *)0xE0000E80) = 0x0001000D;   // ITM Trace Control Register
	*((volatile unsigned *)0xE0000E40) = 0x0000000F;   // ITM Trace Privilege Register
	*((volatile unsigned *)0xE0000E00) = 0x00000001;   // ITM Trace Enable Register. Enabled tracing on stimulus ports. One bit per stimulus port.
	*((volatile unsigned *)0xE0001000) = 0x400003FE;   // DWT_CTRL
	*((volatile unsigned *)0xE0040304) = 0x00000100;   // Formatter and Flush Control Register
#endif
}

//-------------------------------------------------------------------------------------------------
//	Returns 1 if SWO is enabled, else 0

uint32_t	swoIsEnabled	(void)
{
	return bItmAvailable ;
}

//-------------------------------------------------------------------------------------------------
//	Sends a character over the SWO stimulus BSP_SWO_STIMULUS defined in bspcfg.h
//	value : Character to be sent

//	Try to aggregate 4 character to send 32 bits words (4 character at the same time)
//	\n character flushes the 32 bits buffer

void swoPut (uintptr_t dev, char value)
{
	static	uint8_t		pos = 0u ;
	static	uint32_t	word = 0u ;

	(void) dev ;		// Unused device

	if (bItmAvailable == 0u)
	{
		return ;	// ITM not available or not initialized
	}

	word |= (uint32_t) value << (pos << 3u) ;
	pos ++ ;
	if (pos == 4u)
	{
		swoSend32 (word, BSP_SWO_STIMULUS) ;
		pos = 0u ;
		word = 0u ;
	}
	else
	{
		if (value == '\n')
		{
			// Flush word
			while (pos != 0u)
			{
				swoSend8 ((uint8_t) word, BSP_SWO_STIMULUS) ;
				word >>= 8u ;
				pos -- ;
			}
			word = 0u ;
		}
	}
}

//-------------------------------------------------------------------------------------------------
//	Send 1 char to stimulus port BSP_SWO_STIMULUS
//	To be used with aaPrintfEx() to print to SWO

void swoPutCharFn (char cc, uintptr_t arg)
{
	(void) arg ;
	swoPut (0u, cc) ;
}

//-------------------------------------------------------------------------------------------------

void swoSend8 (uint8_t value, uint8_t portNo)
{
	aaIrqStatus_t	intState ;

	if (bItmAvailable != 0u)
	{
		while (1)
		{
			intState = bspSaveAndDisableIrq () ;
			if (ITM->PORT [portNo].u8 == 1u)
			{
				ITM->PORT[portNo].u8 = value ;
				bspRestoreIrq (intState) ;
				break ;
			}
			bspRestoreIrq (intState) ;
		}
	}
}

//-------------------------------------------------------------------------------------------------
// Transmitted in little endian (LS byte first)

void swoSend16 (uint16_t value, uint8_t portNo)
{
	aaIrqStatus_t	intState ;

	if (bItmAvailable != 0u)
	{
		while (1)
		{
			intState = bspSaveAndDisableIrq () ;
			if (ITM->PORT [portNo].u16 == 1u)
			{
				ITM->PORT[portNo].u16 = value ;
				bspRestoreIrq (intState) ;
				break ;
			}
			bspRestoreIrq (intState) ;
		}
	}
}

//-------------------------------------------------------------------------------------------------
// Transmitted in little endian (LS byte first)

void swoSend32 (uint32_t value, uint8_t portNo)
{
	aaIrqStatus_t	intState ;

	if (bItmAvailable != 0u)
	{
		while (1)
		{
			intState = bspSaveAndDisableIrq () ;
			if (ITM->PORT [portNo].u32 == 1u)
			{
				ITM->PORT[portNo].u32 = value ;
				bspRestoreIrq (intState) ;
				break ;
			}
			bspRestoreIrq (intState) ;
		}
	}
}

//-------------------------------------------------------------------------------------------------
// Transmit a NUL terminated string (without the NUL) on terminal output

void swoPutStr (char * pStr)
{
	if (bItmAvailable != 0u)
	{
		while (* pStr != 0u)
		{
			swoPut (0u, * pStr++) ;
		}
	}
}

//-------------------------------------------------------------------------------------------------
