/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	Devices vector table for GCC based toolchain:
		STM32F401/411
		STM32F405/415
		STM32F407/417
		STM32F412
		STM32F413/423
		STM32F427/437
		STM32F429/439
		STM32F469/479
		STM32F446
	Exception vectors and default handlers for Cortex-M3, Cortex-M4, Cortex-M7
	Interrupts vector and default handlers

	After Reset the Cortex-M processor is in Thread mode,
	priority is Privileged, and the stack is set to Main.

	When		Who	What
	28/04/17	ac	Creation
	26/01/22	ac	Add STM32F410/F412/F413/F423 support
	27/01/22	ac	Add aaSpuriousInterruptHandler()


----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/
/*
	__ARM_ARCH_6M__		Set to 1 when generating code for Armv6-M (Cortex-M0, Cortex-M0+, Cortex-M1)
	__ARM_ARCH_7M__		Set to 1 when generating code for Armv7-M (Cortex-M3)
	__ARM_ARCH_7EM__	Set to 1 when generating code for Armv7-M (Cortex-M4, Cortex-M7) with FPU
*/

#include	<stm32f4xx.h>

#if (defined __CC_ARM)
	static inline void __DEBUG_BKPT (void)
	{
		if ((CoreDebug->DHCSR & CoreDebug_DHCSR_C_DEBUGEN_Msk) != 0u)
		{
			// Debug enabled, can use BKPT instruction
			__breakpoint (0) ;
		}
		else
		{
			while (1) { }
		}
	}
#else
	__attribute__ ((always_inline)) static inline void __DEBUG_BKPT (void)
	{
		if ((CoreDebug->DHCSR & CoreDebug_DHCSR_C_DEBUGEN_Msk) != 0u)
		{
			// Debug enabled, can use BKPT instruction
			__ASM volatile ("bkpt 0") ;
		}
		else
		{
			while (1) { }
		}
	}
#endif

extern void __attribute__((noreturn)) start_ (void);	// The entry point in startup.c

// ----------------------------------------------------------------------------
// Exception Stack Frame of the Cortex-M0-M0+-M3-M4-M7 processors.

typedef struct
{
	uint32_t	r0 ;
	uint32_t	r1 ;
	uint32_t	r2 ;
	uint32_t	r3 ;
	uint32_t	r12 ;
	uint32_t	lr ;
	uint32_t	pc ;
	uint32_t	psr ;
	#if  defined(__ARM_ARCH_7EM__)
		uint32_t s[16];				// Floating point registers
	#endif

} exceptionStackFrame ;

void	  HardFault_Handler_C (exceptionStackFrame * frame, uint32_t lr) ;

#if defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)

	void	  UsageFault_Handler_C	(exceptionStackFrame * frame, uint32_t lr) ;
	void	  BusFault_Handler_C	(exceptionStackFrame * frame, uint32_t lr) ;

#endif // defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)

// ----------------------------------------------------------------------------
// Forward declarations of exception handlers
// There is specific default handlers for these exception at the end of the file

extern void		Reset_Handler		(void) ;
extern void		NMI_Handler			(void) ;
extern void		HardFault_Handler	(void) ;

#if defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)
	extern void	MemManage_Handler	(void) ;
	extern void	BusFault_Handler	(void) ;
	extern void	UsageFault_Handler	(void) ;
	extern void	DebugMon_Handler	(void) ;
#endif

extern void		SVC_Handler			(void) ;
extern void		PendSV_Handler		(void) ;
extern void		SysTick_Handler		(void) ;

// Generic default handler for interruptions
void __attribute__((weak)) Default_Handler (void) ;

// ----------------------------------------------------------------------------
// Forward declaration of the IRQ handlers with weak definitions.
// The application can defines a handler with the same name and take precedence

void __attribute__ ((weak, alias ("Default_Handler"))) WWDG_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) PVD_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TAMP_STAMP_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) RTC_WKUP_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) FLASH_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) RCC_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI0_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI3_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI4_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Stream0_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Stream1_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Stream2_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Stream3_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Stream4_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Stream5_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Stream6_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) ADC_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN1_TX_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN1_RX0_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN1_RX1_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN1_SCE_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI9_5_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM1_BRK_TIM9_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM1_UP_TIM10_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM1_TRG_COM_TIM11_IRQHandler(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM1_CC_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM3_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM4_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) I2C1_EV_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) I2C1_ER_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) I2C2_EV_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) I2C2_ER_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SPI1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SPI2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) USART1_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) USART2_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) USART3_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI15_10_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) RTC_Alarm_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) OTG_FS_WKUP_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM8_BRK_TIM12_IRQHandler	(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM8_UP_TIM13_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM8_TRG_COM_TIM14_IRQHandler(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM8_CC_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Stream7_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) FMC_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SDIO_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM5_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SPI3_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) UART4_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) UART5_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM6_DAC_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM7_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA2_Stream0_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA2_Stream1_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA2_Stream2_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA2_Stream3_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA2_Stream4_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) ETH_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) ETH_WKUP_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN2_TX_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN2_RX0_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN2_RX1_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN2_SCE_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) OTG_FS_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA2_Stream5_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA2_Stream6_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA2_Stream7_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) USART6_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) I2C3_EV_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) I2C3_ER_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) OTG_HS_EP1_OUT_IRQHandler	(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) OTG_HS_EP1_IN_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) OTG_HS_WKUP_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) OTG_HS_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DCMI_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CRYP_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) AES_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) HASH_RNG_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) FPU_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) UART7_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) UART8_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SPI4_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SPI5_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SPI6_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SAI1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) LTDC_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) LTDC_ER_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA2D_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) QUADSPI_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DSI_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SAI2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CEC_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SPDIF_RX_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) FMPI2C1_EV_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) FMPI2C1_ER_IRQHandler		(void) ;

void __attribute__ ((weak, alias ("Default_Handler"))) UART9_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) UART10_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DFSDM1_FLT0_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DFSDM1_FLT1_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN3_TX_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN3_RX0_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN3_RX1_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN3_SCE_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DFSDM2_FLT0_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DFSDM2_FLT1_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DFSDM2_FLT2_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DFSDM2_FLT3_IRQHandler		(void) ;

// ----------------------------------------------------------------------------

extern unsigned int _estack_ ;				// From linker script: initial stack address

typedef void (* const pHandler) (void) ;	// Type of the interruptions vector array

// ----------------------------------------------------------------------------

// The table of interrupt handlers. It has an explicit section name
// and relies on the linker script to place it at the correct location in memory.

// Can't place integer and function pointer in one array (Forbidden by ISO C)
__attribute__ ((section(".isr_stackaddress"),used)) unsigned int * isrStackAdress =
{
	& _estack_							// The initial Main stack pointer, 1st in VTOR
} ;

__attribute__ ((section(".isr_vectors"),used)) pHandler isrVectors_[] =
{
	// Cortex-M Core Handlers
	//									   The initial Main stack pointer is in the section .isr_stackaddress
	Reset_Handler,						// The reset handler

	NMI_Handler,						// The NMI handler
	HardFault_Handler,					// The hard fault handler

#if defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)
	MemManage_Handler,					// The MPU fault handler
	BusFault_Handler,					// The bus fault handler
	UsageFault_Handler,					// The usage fault handler
#else
	0,									// Reserved
	0,									// Reserved
	0,									// Reserved
#endif
	0,									// Reserved
	0,									// Reserved
	0,									// Reserved
	0,									// Reserved
	SVC_Handler,						// SVCall handler
#if defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)
	DebugMon_Handler,					// Debug monitor handler
#else
	0,									// Reserved
#endif
	0,									// Reserved
	PendSV_Handler,						// The PendSV handler
	SysTick_Handler,					// The SysTick handler

	// ----------------------------------------------------------------------
	// External Interrupts
	WWDG_IRQHandler,					// 0	Window WatchDog
	PVD_IRQHandler,						// 1	PVD through EXTI Line detection
	TAMP_STAMP_IRQHandler,				// 2	Tamper and TimeStamps through the EXTI line
	RTC_WKUP_IRQHandler,				// 3	RTC Wakeup through the EXTI line
	FLASH_IRQHandler,					// 4	FLASH
	RCC_IRQHandler,						// 5	RCC
	EXTI0_IRQHandler,					// 6	EXTI Line0
	EXTI1_IRQHandler,					// 7	EXTI Line1
	EXTI2_IRQHandler,					// 8	EXTI Line2
	EXTI3_IRQHandler,					// 9	EXTI Line3
	EXTI4_IRQHandler,					// 10	EXTI Line4
	DMA1_Stream0_IRQHandler,			// 11	DMA1 Stream 0
	DMA1_Stream1_IRQHandler,			// 12	DMA1 Stream 1
	DMA1_Stream2_IRQHandler,			// 13	DMA1 Stream 2
	DMA1_Stream3_IRQHandler,			// 14	DMA1 Stream 3
	DMA1_Stream4_IRQHandler,			// 15	DMA1 Stream 4
	DMA1_Stream5_IRQHandler,			// 16	DMA1 Stream 5
	DMA1_Stream6_IRQHandler,			// 17	DMA1 Stream 6
	ADC_IRQHandler,						// 18	ADC1, ADC2 and ADC3s
	CAN1_TX_IRQHandler,					// 19	CAN1 TX
	CAN1_RX0_IRQHandler,				// 20	CAN1 RX0
	CAN1_RX1_IRQHandler,				// 21	CAN1 RX1
	CAN1_SCE_IRQHandler,				// 22	CAN1 SCE
	EXTI9_5_IRQHandler,					// 23	External Line[9:5]s
	TIM1_BRK_TIM9_IRQHandler,			// 24	TIM1 Break and TIM9
	TIM1_UP_TIM10_IRQHandler,			// 25	TIM1 Update and TIM10
	TIM1_TRG_COM_TIM11_IRQHandler,		// 26	TIM1 Trigger and commutation and TIM11
	TIM1_CC_IRQHandler,					// 27	TIM1 Capture Compare
	TIM2_IRQHandler,					// 28	TIM2
	TIM3_IRQHandler,					// 29	TIM3
	TIM4_IRQHandler,					// 30	TIM4
	I2C1_EV_IRQHandler,					// 31	I2C1 Event
	I2C1_ER_IRQHandler,					// 32	I2C1 Error
	I2C2_EV_IRQHandler,					// 33	I2C2 Event
	I2C2_ER_IRQHandler,					// 34	I2C2 Error
	SPI1_IRQHandler,					// 35	SPI1
	SPI2_IRQHandler,					// 36	SPI2
	USART1_IRQHandler,					// 37	USART1
	USART2_IRQHandler,					// 38	USART2
	USART3_IRQHandler,					// 39	USART3
	EXTI15_10_IRQHandler,				// 40  External Line[15:10]s
	RTC_Alarm_IRQHandler,				// 41	RTC Alarm (A and B) through EXTI Line
	OTG_FS_WKUP_IRQHandler,             // 42	USB OTG FS Wakeup through EXTI line
	TIM8_BRK_TIM12_IRQHandler,			// 43	TIM8 Break and TIM12
	TIM8_UP_TIM13_IRQHandler,			// 44	TIM8 Update and TIM13
	TIM8_TRG_COM_TIM14_IRQHandler,		// 45	TIM8 Trigger and Commutation and TIM14
	TIM8_CC_IRQHandler,					// 46	TIM8 Capture Compare
	DMA1_Stream7_IRQHandler,			// 47	DMA1 Stream7
	FMC_IRQHandler,						// 48	FMC / FSMC
	SDIO_IRQHandler,					// 49	SDIO
	TIM5_IRQHandler,					// 50	TIM5
	SPI3_IRQHandler,					// 51	SPI3
	UART4_IRQHandler,					// 52	UART4
	UART5_IRQHandler,					// 53	UART5
	TIM6_DAC_IRQHandler,				// 54	TIM6 and DAC1&2 underrun errors
	TIM7_IRQHandler,					// 55	TIM7
	DMA2_Stream0_IRQHandler,			// 56	DMA2 Stream 0
	DMA2_Stream1_IRQHandler,			// 57	DMA2 Stream 1
	DMA2_Stream2_IRQHandler,			// 58	DMA2 Stream 2
	DMA2_Stream3_IRQHandler,			// 59	DMA2 Stream 3
	DMA2_Stream4_IRQHandler,			// 60	DMA2 Stream 4
#if (defined STM32F439xx | defined STM32F437xx | defined STM32F429xx | defined STM32F427xx | defined STM32F417xx | defined STM32F407xx)
	ETH_IRQHandler,						// 61	Ethernet
	ETH_WKUP_IRQHandler,				// 62	Ethernet Wakeup through EXTI line
# else
	DFSDM1_FLT0_IRQHandler,				// 61	Reserved F412 F413
	DFSDM1_FLT1_IRQHandler,				// 62	Reserved F412 F413
#endif
	CAN2_TX_IRQHandler,					// 63	CAN2 TX
	CAN2_RX0_IRQHandler,				// 64	CAN2 RX0
	CAN2_RX1_IRQHandler,				// 65	CAN2 RX1
	CAN2_SCE_IRQHandler,				// 66	CAN2 SCE
	OTG_FS_IRQHandler,					// 67	USB OTG FS
	DMA2_Stream5_IRQHandler,			// 68	DMA2 Stream 5
	DMA2_Stream6_IRQHandler,			// 69	DMA2 Stream 6
	DMA2_Stream7_IRQHandler,			// 70	DMA2 Stream 7
	USART6_IRQHandler,					// 71	USART6
	I2C3_EV_IRQHandler,					// 72	I2C3 event
	I2C3_ER_IRQHandler,					// 73	I2C3 error
#if (defined STM32F413xx || defined STM32F423xx)
	CAN3_TX_IRQHandler,					// 74	CAN3 TX
	CAN3_RX0_IRQHandler,				// 75	CAN3 RX0
	CAN3_RX1_IRQHandler,				// 76	CAN3 RX1
	CAN3_SCE_IRQHandler,				// 77	CAN3 SCE
#else
	OTG_HS_EP1_OUT_IRQHandler,			// 74	USB OTG HS End Point 1 Out
	OTG_HS_EP1_IN_IRQHandler,			// 75	USB OTG HS End Point 1 In
	OTG_HS_WKUP_IRQHandler,				// 76	USB OTG HS Wakeup through EXTI
	OTG_HS_IRQHandler,					// 77	USB OTG HS
#endif
	DCMI_IRQHandler,					// 78	DCMI
#if (defined STM32F423xx)
	AES_IRQHandler,						// 79	AES
#else
	CRYP_IRQHandler,					// 79	Crypto global interrupt
#endif
	HASH_RNG_IRQHandler,				// 80	Hash and Rng
	FPU_IRQHandler,						// 81	FPU
	UART7_IRQHandler,					// 82	UART7
	UART8_IRQHandler,					// 83	UART8
	SPI4_IRQHandler,					// 84	SPI4
	SPI5_IRQHandler,					// 85	SPI5
	SPI6_IRQHandler,					// 86	SPI6
	SAI1_IRQHandler,					// 87	SAI1
#if (defined STM32F413xx || defined STM32F423xx)
	UART9_IRQHandler,					// 88	UART9
	UART10_IRQHandler,					// 89	UART10
#else
	LTDC_IRQHandler,					// 88	LTDC
	LTDC_ER_IRQHandler,					// 89	LTDC error
#endif
	DMA2D_IRQHandler,					// 90	DMA2D
#if (defined STM32F469xx | defined STM32F479xx)
	QUADSPI_IRQHandler,					// 91	QUADSPI global Interrupt
	DSI_IRQHandler,						// 92	DSI global Interrupt
#else
	SAI2_IRQHandler,					// 91	SAI2 global Interrupt
	QUADSPI_IRQHandler,					// 92	QuadSPI global Interrupt
#endif
	CEC_IRQHandler,						// 93	CEC global Interrupt
	SPDIF_RX_IRQHandler,				// 94	SPDIF-RX global Interrupt
	FMPI2C1_EV_IRQHandler,				// 95	FMPI2C1 Event Interrupt
	FMPI2C1_ER_IRQHandler,				// 96	FMPI2C1 Error Interrupt
#if (defined STM32F413xx || defined STM32F423xx)
	DFSDM2_FLT0_IRQHandler,				// 97	DFSDM2
	DFSDM2_FLT1_IRQHandler,				// 97
	DFSDM2_FLT2_IRQHandler,				// 97
	DFSDM2_FLT3_IRQHandler,				// 97
#endif
} ;

//-----------------------------------------------------------------------------
// The processor jumps here if an unexpected interrupt occurs
// or a specific handler is not present in the application code (spurious interrupt).
// When in DEBUG, triggers a debug exception to warn the user

// The user can redefine the aaSpuriousInterruptHandler() function
// The user can't redefine Default_Handler() because it have to be in this source module

void __attribute__ ((weak, section(".isr_handlers"))) aaSpuriousInterruptHandler (void)
{
	__DEBUG_BKPT () ;
}

void __attribute__ ((section(".isr_handlers"))) Default_Handler (void)
{
	aaSpuriousInterruptHandler () ;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Default exception handlers. Override the ones here by defining your own
// handler routines in your application code.
// When in DEBUG, triggers a debug exception to warn the user

void __attribute__ ((section(".isr_handlers"), noreturn, naked)) Reset_Handler (void)
{
	start_ () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak)) NMI_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak)) SVC_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak)) PendSV_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------
// Hard Fault handler wrapper in assembly.
// It extracts the location of stack frame and passes it to handler in C as a pointer.
// We also pass the LR value as second parameter.
// (Based on Joseph Yiu's, The Definitive Guide to ARM Cortex-M3 and
// Cortex-M4 Processors, Third Edition, Chap. 12.8, page 402).

// naked attribute:
// Use this attribute on the ARM ports to indicate that the specified function
// does not need prologue/epilogue sequences generated by the compiler.
// It is up to the programmer to provide these sequences.
// The only statements that can be safely included in naked functions are asm statements that do not have operands.
// All other statements, including declarations of local variables, if statements, and so forth, should be avoided.
// Naked functions should be used to implement the body of an assembly function,
// while allowing the compiler to construct the requisite function declaration for the assembler.

void __attribute__ ((section(".isr_handlers"),weak,naked)) HardFault_Handler (void)
{
	__ASM volatile
	(
		" tst	lr,#4	\n"
		" ite	eq		\n"
		" mrseq	r0,msp	\n"
		" mrsne	r0,psp	\n"
		" mov	r1,lr	\n"
		" ldr	r2,=HardFault_Handler_C \n"
		" bx	r2"

		: // Outputs
		: // Inputs
		: // Clobbers
	) ;
}

// Only referenced by inline assembly, so need "used" attribute
// The user may redefine this function, so "weak" attribute
void __attribute__ ((section(".isr_handlers"),weak,used)) HardFault_Handler_C (
						exceptionStackFrame		* frame,
						uint32_t				lr)
{
	(void) frame ;
	(void) lr ;

	#if defined (DEBUG)
		uint32_t	mmfar	= SCB->MMFAR ;	// MemManage Fault Address
		uint32_t	bfar 	= SCB->BFAR ;	// Bus Fault Address
		uint32_t	cfsr 	= SCB->CFSR ;	// Configurable Fault Status Registers

		(void) mmfar;
		(void) bfar ;
		(void) cfsr  ;
	#endif

	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Exceptions specific to ARM ARCH 7M /  7EM

#if defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)

void __attribute__ ((section(".isr_handlers"),weak)) MemManage_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak)) DebugMon_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak,naked)) BusFault_Handler (void)
{
	__ASM volatile
	(
		" tst		lr,#4	\n"
		" ite		eq		\n"
		" mrseq		r0,msp	\n"
		" mrsne		r0,psp	\n"
		" mov		r1,lr	\n"
		" ldr		r2,=BusFault_Handler_C \n"
		" bx		r2"
		: // Outputs
		: // Inputs
		: // Clobbers
	) ;
}

// Only referenced by inline assembly, so need "used" attribute
// The user may redefine this function, so "weak" attribute
void __attribute__ ((section(".isr_handlers"),weak,used)) BusFault_Handler_C (
						exceptionStackFrame		* frame,
						uint32_t				lr)
{
	(void) frame ;
	(void) lr ;

	#if defined (DEBUG)
		uint32_t	mmfar	= SCB->MMFAR ;	// MemManage Fault Address
		uint32_t	bfar 	= SCB->BFAR ;	// Bus Fault Address
		uint32_t	cfsr 	= SCB->CFSR ;	// Configurable Fault Status Registers

		(void) mmfar;
		(void) bfar ;
		(void) cfsr  ;
	#endif

	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak,naked)) UsageFault_Handler (void)
{
	__ASM volatile
	(
		" tst		lr,#4	\n"
		" ite		eq		\n"
		" mrseq		r0,msp	\n"
		" mrsne		r0,psp	\n"
		" mov		r1,lr	\n"
		" ldr		r2,=UsageFault_Handler_C \n"
		" bx		r2"
		: // Outputs
		: // Inputs
		: // Clobbers
	) ;
}

// Only referenced by inline assembly, so need "used" attribute
// The user may redefine this function, so "weak" attribute
void __attribute__ ((section(".isr_handlers"),weak,used)) UsageFault_Handler_C (
						exceptionStackFrame		* frame,
						uint32_t 				lr)
{
	(void) frame ;
	(void) lr ;

	#if defined (DEBUG)
		uint32_t	mmfar	= SCB->MMFAR ;	// MemManage Fault Address
		uint32_t	bfar 	= SCB->BFAR ;	// Bus Fault Address
		uint32_t	cfsr 	= SCB->CFSR ;	// Configurable Fault Status Registers

		(void) mmfar;
		(void) bfar ;
		(void) cfsr ;
	#endif

	__DEBUG_BKPT () ;
}

#endif	// defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

