/* 
 * The MIT License (MIT)
 *
 * Copyright (c) 2019 Ha Thach (tinyusb.org)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * This file is part of the TinyUSB stack.
 */

#ifndef _TUSB_OS_CUSTOM_H_
#define _TUSB_OS_CUSTOM_H_
//--------------------------------------------------------------------+

// AdAstra Headers
#include "aa.h"
#include "aaqueue.h"

#ifdef __cplusplus
extern "C" {
#endif

// Convert ms to tick count. Add 999 to round up and avoid a result of 0
#define	MS_TO_TICKS(msec)	(((uint32_t)msec * bspGetTickRate () + 999u) / 1000u)

//--------------------------------------------------------------------+
// TASK API
//--------------------------------------------------------------------+

static inline void osal_task_delay(uint32_t msec)
{
  aaTaskDelay (MS_TO_TICKS (msec)) ;
}

//--------------------------------------------------------------------+
// Semaphore API
//--------------------------------------------------------------------+
typedef aaSemId_t	osal_semaphore_def_t ;	// Unused
typedef aaSemId_t	osal_semaphore_t ;

// Create an empty semaphore
static inline osal_semaphore_t osal_semaphore_create (osal_semaphore_def_t * semdef)
{
	aaSemId_t semId ;

	(void) semdef ;
	aaSemCreate (0u, & semId) ;
	AA_ASSERT (semId != AA_INVALID_SEM) ;
	return  semId ;
}

//--------------------------------------------------------------------+

static inline bool osal_semaphore_post (osal_semaphore_t sem_hdl, bool in_isr)
{
	(void) in_isr ;
	return AA_ENONE == aaSemGive (sem_hdl) ;
}

//--------------------------------------------------------------------+

static inline bool osal_semaphore_wait (osal_semaphore_t sem_hdl, uint32_t msec)
{
  uint32_t const ticks = (msec == OSAL_TIMEOUT_WAIT_FOREVER) ? AA_INFINITE : MS_TO_TICKS (msec);
  return AA_ENONE == aaSemTake (sem_hdl, ticks);
}

//--------------------------------------------------------------------+

static inline void osal_semaphore_reset (osal_semaphore_t const sem_hdl)
{
	aaSemReset (sem_hdl, 0u) ;
}

//--------------------------------------------------------------------+
// MUTEX API (priority inheritance)
//--------------------------------------------------------------------+
// The TinyUSB code requires type osal_mutex_t to be a pointer
// Because sometime it uses NULL.
// So the aaMutexId_t is casted to void *

typedef aaMutexId_t		osal_mutex_def_t ;	// Unused
typedef void			* osal_mutex_t ;

// Convert osal_mutex_t to aaMutexId_t
#define	TO_MUTEXID(hdl)	((aaMutexId_t) (uint32_t) hdl)

static inline osal_mutex_t osal_mutex_create (osal_mutex_def_t * mdef)
{
	aaSemId_t mutexId ;

	(void) mdef ;
	aaMutexCreate (& mutexId) ;
	if (mutexId == AA_INVALID_MUTEX)
	{
		AA_ASSERT (0) ;
		mutexId = 0 ;
	}
	return  (osal_mutex_t) (uint32_t) mutexId ;
}

//--------------------------------------------------------------------+

static inline bool osal_mutex_lock (osal_mutex_t mutex_hdl, uint32_t msec)
{
  uint32_t const ticks = (msec == OSAL_TIMEOUT_WAIT_FOREVER) ? AA_INFINITE : MS_TO_TICKS (msec) ;
  return AA_ENONE == aaMutexTake (TO_MUTEXID (mutex_hdl), ticks) ;
}

//--------------------------------------------------------------------+

static inline bool osal_mutex_unlock (osal_mutex_t mutex_hdl)
{
  return AA_ENONE == aaMutexGive (TO_MUTEXID (mutex_hdl)) ;
}

//--------------------------------------------------------------------+
// QUEUE API
//--------------------------------------------------------------------+

// role device/host is used by OS NONE for mutex (disable usb isr) only
#define OSAL_QUEUE_DEF(_int_set, _name, _depth, _type) \
  static _type _name##_##buf[_depth];\
  osal_queue_def_t _name = { .depth = _depth, .item_sz = sizeof(_type), .buf = _name##_##buf }

typedef struct
{
	uint16_t	depth ;
	uint16_t	item_sz ;
	void		* buf ;

} osal_queue_def_t ;

//--------------------------------------------------------------------+

typedef aaQueueId_t	osal_queue_t;

static inline osal_queue_t osal_queue_create (osal_queue_def_t * qdef)
{
	aaQueueId_t	queueId ;

	aaQueueCreate (& queueId, qdef->item_sz, qdef->depth, qdef->buf, 0) ;
	if (queueId == AA_INVALID_QUEUE)
	{
		AA_ASSERT (0) ;
		queueId = 0 ;
	}
	return queueId ;
}

//--------------------------------------------------------------------+
//	No timeout

static inline bool osal_queue_receive (osal_queue_t qhdl, void * data)
{
	return AA_ENONE == aaQueueTake (qhdl, data, 0, AA_INFINITE) ;
}

//--------------------------------------------------------------------+
// Used in ISR => no timeout

static inline bool osal_queue_send (osal_queue_t qhdl, void const * data, bool in_isr)
{
	(void) in_isr ;
	return AA_ENONE == aaQueueGive (qhdl, (void *) data, 0, 0) ;
}

//--------------------------------------------------------------------+
// Return true if the queue is empty

static inline bool osal_queue_empty (osal_queue_t qhdl)
{
	uint32_t	count ;

	aaQueueGetCount (qhdl, & count) ;
	return count == 0u ;
}

//--------------------------------------------------------------------+

#ifdef __cplusplus
 }
#endif

//--------------------------------------------------------------------+
#endif	// _TUSB_OS_CUSTOM_H_
