/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	adcbasic.h	Some routine to manage basic ADC

	When		Who	What
	25/05/18	ac	Creation

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined ADCBASIC_H_
#define	ADCBASIC_H_
//----------------------------------------------------------------------
#include	"stm32f4xx_ll_adc.h"
#include	"dmabasic.h"

// adcInitAdc() initializes all channels sampling time to this value
// The user can later set the channels sampling time with adcInitSamplingTime()
#define	ADC_DEF_SAMPLINGTIME	LL_ADC_SAMPLINGTIME_480CYCLES

// Theses flags are memorized in adcCommonConfig
#define	ADC_ADC1			0x00000001u
#if defined (ADC2)
	#define	ADC_ADC2			0x00000002u
	#define	ADC_DUAL			0x00000008u
	#define	ADC_INTERLEAVED		0x00000020u
#endif	// ADC2
#if defined (ADC3)
	#define	ADC_ADC3		0x00000004u
	#define	ADC_TRIPLE		0x00000010u
#endif	// ADC3

// Common prescaler for adcInit()
#define	ADC_PRES2			0x00u
#define	ADC_PRES4			0x01u
#define	ADC_PRES6			0x10u
#define	ADC_PRES8			0x11u

// For adcInitAdc()
#define	ADC_ALIGNRIGHT		0x00000000u
#define	ADC_ALIGNLEFT		0x00000100u
#define	ADC_RESO12BITS		0x00000000u
#define	ADC_RESO8BITS		0x00000200u
#define	ADC_INT_EOC			0x00001000u		// interrupt on end of sequence conversions
#define	ADC_INT_EOCS		0x00002000u		// interrupt on end of unitary conversion
#define	ADC_INT_OVR			0x00004000u		// interrupt on overrun

// For adcInitSequencer()
#define	ADC_DISCONT			0x00010000u
#define	ADC_ONESHOT			0x00020000u

// For adcDmaInit()
#define	ADC_DMA_SINGLE		0x00010000u		// No new DMA request after the last transfer
#define	ADC_DMA_CIRCULAR	0x00020000u		// DMA circular
#define	ADC_DMA_DOUBLE		0x00040000u		// DMA double buffer
#define	ADC_DMA_NOIT		0x00080000u		// No DMA interruptions

// For adcSetSensor
#define	ADC_SENSOR_NONE		0x00000000u		// Disable sensors
#define	ADC_SENSOR_TEMP		0x00000001u		// Temperature sensor and VREFINT
#define	ADC_SENSOR_VBAT		0x00000002u		// VBAT sensor
#define	ADC_SENSOR_VREF		0x000000043		// VREFINT sensor


typedef enum
{
	ADC_ENONE		= 0,
	ADC_EFAIL		= 1,
	ADC_ENOTINIT	= 2,
	ADC_EARG		= 3,
	ADC_ESTATE		= 4

} adcStatus_t ;

#define	ADC_INT_PRIORITY			(BSP_MAX_INT_PRIO+2u)		//	ADCs interruptions priority

#define	ADC1_DMA_INT_PRIORITY		(BSP_MAX_INT_PRIO+2u)		//	ADC1 DMA interruptions priority
#define	ADC2_DMA_INT_PRIORITY		(BSP_MAX_INT_PRIO+2u)		//	ADC2 DMA interruptions priority
#if defined (ADC3)
	#define	ADC3_DMA_INT_PRIORITY	(BSP_MAX_INT_PRIO+2u)		//	ADC3 DMA interruptions priority
#endif	// ADC3

// Forward declaration for opaque type
typedef	struct adcDesc_t adcDesc_t;

// Callback type for ADC interruptions handler
typedef	void (* adcCallback_t) (adcDesc_t * pAdcDesc, uint32_t event, uintptr_t arg) ;

#define	ADC_EVENT_EOC		1u		// End of conversion
#define	ADC_EVENT_OVR		2u		// DMA overrun

// A structure to describe channels sampling time for adcInitSamplingTime()
typedef struct
{
	uint8_t	channel ;	// Channel number: 0..15
	uint8_t	time; 		// Sampling time: LL_ADC_SAMPLINGTIME_xCYCLES

} adcSamplingTime_t ;

//----------------------------------------------------------------------
#ifdef __cplusplus
extern "C" {
#endif

adcStatus_t		adcInit				(uint32_t flags, uint32_t prescaler, uint32_t interDelay) ;
void			adcInitChannels		(uint32_t flags, const uint8_t * pChannel, uint32_t count) ;
adcStatus_t		adcInitAdc			(uint32_t flags, adcDesc_t ** ppAdcDesc) ;
adcStatus_t		adcInitSequencer	(adcDesc_t * pAdcDesc, uint32_t flags, const uint8_t * pChannels, uint32_t count) ;
adcStatus_t		adcInitSamplingTime	(adcDesc_t * pAdcDesc, const adcSamplingTime_t * pSamplingTime, uint32_t count) ;
adcStatus_t		adcSensorEnable		(adcDesc_t * pAdcDesc, uint32_t mode) ;
int32_t			adcToTemp			(uint32_t tsValue) ;
adcStatus_t		adcDmaInit			(adcDesc_t * pAdcDesc, uint32_t flags, uint32_t length) ;
adcStatus_t 	adcDmaBuffer		(adcDesc_t * pAdcDesc, const void * buffer1, const void * buffer2) ;
adcStatus_t 	adcDmaGetDesc		(adcDesc_t * pAdcDesc, dmaDesc_t ** ppDmaDesc) ;
adcStatus_t		adcSetTrigger		(adcDesc_t * pAdcDesc, uint32_t triggerSource, uint32_t edge) ;
adcStatus_t 	adcStop				(adcDesc_t * pAdcDesc) ;
adcStatus_t 	adcStart			(adcDesc_t * pAdcDesc) ;
void 			adcSwTrigger		(adcDesc_t * pAdcDesc) ;
adcStatus_t		adcSetCallback		(adcDesc_t * pAdcDesc, adcCallback_t cb, uintptr_t cbArg) ;

adcStatus_t		adcWaitForEoc		(adcDesc_t * pAdcDesc) ;
void			adcReadUnitary		(adcDesc_t * pAdcDesc, uint16_t * pData) ;

#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// ADCBASIC_H_
