/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	dacbasic.h	DAC management with DMA

	When		Who	What
	05/05/18	ac	Creation

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined DACBASIC_H_
#define DACBASIC_H_
//----------------------------------------------------------------------

#include	"dmabasic.h"
#include	"gpiobasic.h"

// Some devices doesn't have DAC
#if (defined DAC  ||  defined DAC1)

#include	"stm32f4xx_ll_dac.h"	// For LL_DAC_TRIG_xx definitions

// To define the supported DAC devices

#define	WITH_DAC_1		1	// DAC1 with 2 channels
#define	WITH_DAC_2		0	// WARNING: DAC2 is not supported by this implementation

// If one WITH_DAC_x is 1 then dacDescArray in dacbasic.c is not empty and must be defined
#if (WITH_DAC_1 == 1  ||  WITH_DAC_2 == 1)
	#define	WITH_DAC_X		1
#endif

//----------------------------------------------------------------------
// Initialization and setting flags

#define		DAC_CHAN1			0x00000001		// Channel 1 id
#define		DAC_CHAN2			0x00000002		// Channel 2 id

// For dacInit()
#define		DAC_MONO1			0x00000010		// Use independent channel 1
#define		DAC_MONO2			0x00000020		// Use independent channel 2
#define		DAC_DUAL			0x00000040		// Use dual channel 1 and 2

// For dacSetChannel()
#define		DAC_OUTBUFFDIS		0x00000100		// Disable output buffer
#define		DAC_OUTBUFFEN		0x00000000		// Enable output buffer (default)

#define		DAC_BITS8			0x00000200		// 8 bits words
#define		DAC_BITS12			0x00000000		// 12 bits words (default)

#define		DAC_ALIGNLEFT		0x00000400		// Left aligned data
#define		DAC_ALIGNRIGHT		0x00000000		// Right aligned data (default)

// For dacDmaInit()
#define		DAC_DMA_CIRCULAR	0x00001000		// Circular DMA buffer
#define		DAC_DMA_DOUBLE		0x00002000		// Double DMA buffer
#define		DAC_DMA_SINGLE		0x00004000		// One DMA buffer, not circular
#define		DAC_DMA_MASK		0x00007000

#define		DAC_DMA_NOIT		0x00010000		// No DMA interrupt

//----------------------------------------------------------------------

typedef struct dacDesc_s
{
	uint16_t		id ;			// Descriptor id
	uint8_t			periphRstPos ;	// DAC periph reset bit pos
	uint8_t			periphEnPos ;	// DAC periph enable bit pos
	DAC_TypeDef		* pDac ;		// Pointer to DAC registers
	gpioPinDesc_t	dacOut1 ;		// DAC channel 1 output pin descriptor
	gpioPinDesc_t	dacOut2 ;		// DAC channel 1 output pin descriptor
	dmaDesc_t		dma1 ;
	dmaDesc_t		dma2 ;

} dacDesc_t ;

//----------------------------------------------------------------------

//	Default priority for DAC interrupt
#define		DAC_INT_PRIORITY	BSP_IRQPRIOMAX_MINUS(2)

#define		DAC_EV_UNDR			0x10000		// Underrun event for DAC interrupt callback

typedef enum
{
	DAC_ENONE		= 0,
	DAC_EFAIL		= 1,
	DAC_ENOTINIT	= 2,
	DAC_ETIMEOUT	= 3,
	DAC_EARG		= 4,
	DAC_ESTATE		= 5

} dacStatus_t ;

#ifdef __cplusplus
extern "C" {
#endif

dacStatus_t		dacGetDesc 			(uint32_t id, const dacDesc_t ** ppDacDesc) ;
dacStatus_t 	dacInit				(const dacDesc_t * pDacDesc, uint32_t flags) ;
dacStatus_t 	dacSetChannel		(uint32_t channel, uint32_t flags, uint32_t trigSrc) ;
dacStatus_t 	dacSetCallback		(uint32_t channel, dmaCallback_t cb, uintptr_t arg) ;
dacStatus_t 	dacDmaInit			(uint32_t channel, uint32_t flags, uint32_t length) ;
dacStatus_t 	dacDmaBuffer		(uint32_t channel, const void * buffer1, const void * buffer2) ;
dacStatus_t 	dacStart			(uint32_t channel) ;
dacStatus_t 	dacStop				(uint32_t channel) ;
dacStatus_t 	dacSwTrigger		(uint32_t channel) ;
dacStatus_t 	dacOuput			(uint32_t channel, uint32_t value)  ;

#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// defined DAC  ||  defined DAC1

#endif	// DACBASIC_H_
