/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	dmabasic.c	DMA management for F4 (doesn't have data cache)

	When		Who	What
	23/04/18	ac	Creation


----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aakernel.h"

#include	"gpiobasic.h"
#include	"rccbasic.h"
#include	"dmabasic.h"

//----------------------------------------------------------------------

//	Array to get interrupt number of DMA stream
static	const uint8_t	dmaStreamIrqIndex [16] =
{
	DMA1_Stream0_IRQn,
	DMA1_Stream1_IRQn,
	DMA1_Stream2_IRQn,
	DMA1_Stream3_IRQn,
	DMA1_Stream4_IRQn,
	DMA1_Stream5_IRQn,
	DMA1_Stream6_IRQn,
	DMA1_Stream7_IRQn,
	
	DMA2_Stream0_IRQn,
	DMA2_Stream1_IRQn,
	DMA2_Stream2_IRQn,
	DMA2_Stream3_IRQn,
	DMA2_Stream4_IRQn,
	DMA2_Stream5_IRQn,
	DMA2_Stream6_IRQn,
	DMA2_Stream7_IRQn,
} ;

// Position of streams flags in LISR/HISR/LIFCR/HIFCR
static const uint8_t	dmaStreamFlagOffset [4] =
{
	DMA_LISR_FEIF0_Pos,
	DMA_LISR_FEIF1_Pos,
	DMA_LISR_FEIF2_Pos,
	DMA_LISR_FEIF3_Pos,
} ;

//----------------------------------------------------------------------

// DMA callback descriptor
typedef struct
{
	dmaCallback_t	fn ;	// Function to call, NULL if no function
	uintptr_t		arg ;	// User argument

} dmaCallbackDesc_t;

static	void dmaDefCallback (uint32_t event, uint32_t dir, uintptr_t arg) ;

static	dmaCallbackDesc_t	dmaCallbackArray [16] =
{
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },

	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
} ;

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	Default DMA callback, when not set by the application.

static	void dmaDefCallback (uint32_t event, uint32_t source, uintptr_t arg)
{
	(void) event ;
	(void) source ;
	(void) arg ;
}

//----------------------------------------------------------------------
//	Enable DMA peripheral clock

void		dmaEnableClock	(dma_t * pDma)
{
	rccEnableClock (pDma, (pDma == (dma_t *) DMA1) ? RCC_AHB1ENR_DMA1EN : RCC_AHB1ENR_DMA2EN) ;
}

//----------------------------------------------------------------------
//	Return DMA_ISR flags for streamNum right aligned
//	Return a combination of: DMA_FLAG_FEIF, DMA_FLAG_DMEIF, DMA_FLAG_TEIF, DMA_FLAG_HTIF, DMA_FLAG_TCIF

uint32_t	dmaGetFlags		(const dmaDesc_t * pDmaDesc)
{
	dma_t		* pDma  = DMA_GETDMA (pDmaDesc->dmaNum) ;
	uint32_t 	streamNum = DMA_GETSTREAM (pDmaDesc) ;
	uint32_t	isr ;

	if (streamNum < 4u)
	{
		isr = pDma->LISR ;
	}
	else
	{
		isr = pDma->HISR ;
		streamNum -= 4u ;
	}
	return ((isr >> dmaStreamFlagOffset [streamNum]) & DMA_FLAG_ALLIF) ;
}

uint32_t	dmaGetFlagsRaw		(dma_t * pDma, uint32_t streamNum)
{
	uint32_t	num, isr ;

	num = streamNum ;
	if (num < 4u)
	{
		isr = pDma->LISR ;
	}
	else
	{
		isr = pDma->HISR ;
		num -= 4u ;
	}
	return ((isr >> dmaStreamFlagOffset [num]) & DMA_FLAG_ALLIF) ;
}

//----------------------------------------------------------------------
//	Allows to clear DMA_ISR flags for streamNum. Bits are right aligned
//	Use a combination of: DMA_FLAG_FEIF, DMA_FLAG_DMEIF, DMA_FLAG_TEIF, DMA_FLAG_HTIF, DMA_FLAG_TCIF

void		dmaClearFlags		(const dmaDesc_t * pDmaDesc, uint32_t flags)
{
	dma_t			* pDma  = DMA_GETDMA (pDmaDesc->dmaNum) ;
	uint32_t 		streamNum = DMA_GETSTREAM (pDmaDesc) ;
	uint32_t		reg ;
	__IO uint32_t	* pReg ;

	if (streamNum < 4)
	{
		pReg = & pDma->LIFCR ;
	}
	else
	{
		pReg = & pDma->HIFCR ;
		streamNum -= 4 ;
	}
	reg = (flags & DMA_FLAG_ALLIF) << dmaStreamFlagOffset [streamNum] ;
	* pReg = reg ;
}

void		dmaClearFlagsRaw	(dma_t * pDma, uint32_t streamNum, uint32_t flags)
{
	uint32_t		num, reg ;
	__IO uint32_t	* pReg ;

	num = streamNum ;
	if (num < 4u)
	{
		pReg = & pDma->LIFCR ;
	}
	else
	{
		pReg = & pDma->HIFCR ;
		num -= 4u ;
	}
	reg = (flags & DMA_FLAG_ALLIF) << dmaStreamFlagOffset [num] ;
	* pReg = reg ;
}

//----------------------------------------------------------------------
// Configure and enable DMA interrupt

void	dmaSetIrq (const dmaDesc_t * pDmaDesc)
{
	IRQn_Type	irqNum = (IRQn_Type) dmaStreamIrqIndex [((pDmaDesc->dmaNum - 1u) << 3u) + DMA_GETSTREAM (pDmaDesc)] ;
	
	NVIC_SetPriority	(irqNum, pDmaDesc->irqPrio);
	NVIC_EnableIRQ		(irqNum) ;
}

//----------------------------------------------------------------------
// Disable DMA interrupt

void	dmaDisableIrq (const dmaDesc_t * pDmaDesc)
{
	IRQn_Type	irqNum = (IRQn_Type) dmaStreamIrqIndex [((pDmaDesc->dmaNum - 1u) << 3u) + DMA_GETSTREAM (pDmaDesc)] ;
	
	NVIC_DisableIRQ (irqNum) ;
}

//--------------------------------------------------------------------------------
//	Set the DMA mode
//	Mode is: LL_DMA_MODE_NORMAL or LL_DMA_MODE_CIRCULAR

void	dmaSetMode (const dmaDesc_t * pDmaDesc, uint32_t mode)
{
	dma_t		* pDma ;
	dmaStream_t	* pStream ;
	uint32_t	reg ;

	AA_ASSERT (pDmaDesc->dmaNum != DMA_NONE) ;
	if (pDmaDesc->dmaNum == DMA_NONE)
	{
		return ;
	}
	pDma    = DMA_GETDMA (pDmaDesc->dmaNum) ;
	pStream = & pDma->stream [DMA_GETSTREAM (pDmaDesc)]  ;

	reg = pStream->CR ;
	reg &= ~LL_DMA_MODE_CIRCULAR ;
	pStream->CR = reg | mode ;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------

void	dmaSetCallback (const dmaDesc_t * pDmaDesc, dmaCallback_t fn, uintptr_t arg)
{
	if (fn == NULL)
	{
		// No user callback, set default callback
		dmaCallbackArray [((pDmaDesc->dmaNum - 1u) << 3u) + DMA_GETSTREAM (pDmaDesc)].fn  = dmaDefCallback ;
	}
	else
	{
		dmaCallbackArray [((pDmaDesc->dmaNum - 1u) << 3u) + DMA_GETSTREAM (pDmaDesc)].fn  = fn ;
		dmaCallbackArray [((pDmaDesc->dmaNum - 1u) << 3u) + DMA_GETSTREAM (pDmaDesc)].arg = arg ;
	}
}

//----------------------------------------------------------------------
//	dmaNum: 1..2
//	streamNum: 0..7

static	void	DMA_Stream_IRQHandler (uint32_t dmaNum, uint32_t streamNum)
{
	dma_t				* pDma = DMA_GETDMA (dmaNum) ;
	dmaCallbackDesc_t	* pCb = & dmaCallbackArray [((dmaNum - 1u) << 3u) + streamNum] ;
	dmaStream_t			* pStream ;
	volatile uint32_t	* pClear ;
	uint32_t			reg ;
	uint32_t			maskBase, mask ;
	
	aaIntEnter () ;

	// Read interrupt status register
	if (streamNum < 4u)
	{
		reg = pDma->LISR ;
		pClear = & pDma->LIFCR ;
	}
	else
	{
		reg = pDma->HISR ;
		pClear = & pDma->HIFCR ;
	}

	pStream	= dmaStreamAddressRaw (DMA_GETDMA (dmaNum), streamNum) ;

	// maskBase: mask of LSB interrupt flag of the stream
	// To get other flags mask: left shift maskBase using stream0 pos values
	maskBase = 1u << dmaStreamFlagOffset [streamNum & 0x03u] ;
	
	// Check transfer complete
	mask = maskBase << DMA_LISR_TCIF0_Pos ;
	if ((reg & mask) != 0u)
	{
		// Clear flag
		* pClear = mask ;

		if ((pStream->CR & DMA_SxCR_TCIE) != 0u)
		{
			// TC interrupt enabled, call the callback
			(* pCb->fn) (DMA_EV_COMPLETE, dmaNum << 16 | streamNum, pCb->arg) ;
		}
	}

	// Check half transfer complete
	mask =  maskBase << DMA_LISR_HTIF0_Pos ;
	if ((reg & mask) != 0u)
	{
		// Clear flag
		* pClear = mask ;

		if ((pStream->CR & DMA_SxCR_HTIE) != 0u)
		{
			// HT interrupt enabled, call the callback
			(* pCb->fn) (DMA_EV_HCOMPLETE, dmaNum << 16 | streamNum, pCb->arg) ;
		}
	}

	// Check transfer error
	mask = maskBase <<  DMA_LISR_TEIF0_Pos ;
	if ((reg & mask) != 0u)
	{
		// Clear flag
		* pClear = mask ;

		if ((pStream->CR & DMA_SxCR_TEIE) != 0u)
		{
			// TE interrupt enabled, call the callback
			(* pCb->fn) (DMA_EV_ERROR, dmaNum << 16 | streamNum, pCb->arg) ;
		}
	}

	aaIntExit () ;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	DMA1 IRQ handlers

void DMA1_Stream0_IRQHandler (void) ;
void DMA1_Stream0_IRQHandler (void)
{
	DMA_Stream_IRQHandler (1u, 0u) ;
}

void DMA1_Stream1_IRQHandler (void) ;
void DMA1_Stream1_IRQHandler (void)
{
	DMA_Stream_IRQHandler (1u, 1u) ;
}

void DMA1_Stream2_IRQHandler (void) ;
void DMA1_Stream2_IRQHandler (void)
{
	DMA_Stream_IRQHandler (1u, 2u) ;
}

void DMA1_Stream3_IRQHandler (void) ;
void DMA1_Stream3_IRQHandler (void)
{
	DMA_Stream_IRQHandler (1u, 3u) ;
}

void DMA1_Stream4_IRQHandler (void) ;
void DMA1_Stream4_IRQHandler (void)
{
	DMA_Stream_IRQHandler (1u, 4u) ;
}

void DMA1_Stream5_IRQHandler (void) ;
void DMA1_Stream5_IRQHandler (void)
{
	DMA_Stream_IRQHandler (1u, 5u) ;
}

void DMA1_Stream6_IRQHandler (void) ;
void DMA1_Stream6_IRQHandler (void)
{
	DMA_Stream_IRQHandler (1u, 6u) ;
}

void DMA1_Stream7_IRQHandler (void) ;
void DMA1_Stream7_IRQHandler (void)
{
	DMA_Stream_IRQHandler (1u, 7u) ;
}

//----------------------------------------------------------------------
//	DMA2 IRQ handlers

void DMA2_Stream0_IRQHandler (void) ;
void DMA2_Stream0_IRQHandler (void)
{
	DMA_Stream_IRQHandler (2u, 0u) ;
}

void DMA2_Stream1_IRQHandler (void) ;
void DMA2_Stream1_IRQHandler (void)
{
	DMA_Stream_IRQHandler (2u, 1u) ;
}

void DMA2_Stream2_IRQHandler (void) ;
void DMA2_Stream2_IRQHandler (void)
{
	DMA_Stream_IRQHandler (2u, 2u) ;
}

void DMA2_Stream3_IRQHandler (void) ;
void DMA2_Stream3_IRQHandler (void)
{
	DMA_Stream_IRQHandler (2u, 3u) ;
}

void DMA2_Stream4_IRQHandler (void) ;
void DMA2_Stream4_IRQHandler (void)
{
	DMA_Stream_IRQHandler (2u, 4u) ;
}

void DMA2_Stream5_IRQHandler (void) ;
void DMA2_Stream5_IRQHandler (void)
{
	DMA_Stream_IRQHandler (2u, 5u) ;
}

void DMA2_Stream6_IRQHandler (void) ;
void DMA2_Stream6_IRQHandler (void)
{
	DMA_Stream_IRQHandler (2u, 6u) ;
}

void DMA2_Stream7_IRQHandler (void) ;
void DMA2_Stream7_IRQHandler (void)
{
	DMA_Stream_IRQHandler (2u, 7u) ;
}

//----------------------------------------------------------------------
