/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	dmabasic.c	DMA management for F4 (doesn't have data cache)

	When		Who	What
	23/04/18	ac	Creation

	29/07/21	ac	Modify dmaDesc_t: add config
					Add dmaGetFlagsRaw(), dmaClearFlagsRaw()

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined	DMABASIC_H_
#define DMABASIC_H_
//----------------------------------------------------------------------

#include	"stm32f4xx_ll_dma.h"

//----------------------------------------------------------------------

#define	DMA_GETDMA(dmaNum)				((dma_t *)(((dmaNum) == 1u) ? DMA1 : DMA2))	// Get pointer to DMa device from DMA number
#define	DMA_GETSTREAM(pDesc)			((pDesc)->streamChan >> 4)
#define	DMA_GETCHANNEL(pDesc)			((pDesc)->streamChan & 0x07u)
#define	DMA_TO_STREAMCHAN(stream,chan)	((uint8_t)(((stream) << 4) | (chan)))


#define	DMA_NONE	0
#define	DMA_DMA1	1
#define	DMA_DMA2	2

typedef struct dmaDesc_s
{
	uint8_t		dmaNum ;		//	DMA number 1 or 2, DMA_NONE => unused descriptor
	uint8_t		streamChan ;	//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
	uint8_t		irqPrio ;		//	Interrupt priority
	uint32_t	config ;		//	Direction e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY, mode, priority...

} dmaDesc_t ;

typedef DMA_Stream_TypeDef dmaStream_t ;	// For consistency with other MCU families


// Event of DMA IRQ callback function
#define	DMA_EV_COMPLETE		1u		// Transfer complete
#define	DMA_EV_HCOMPLETE	2u		// Transfer half complete
#define	DMA_EV_ERROR		3u		// Transfer error

// Callback parameter source is 'dmaNum << 16 | streamNum' :
//		High word : DMA number (1 or 2)
//		Low word  : stream number (0..n)

// Prototype of DMA IRQ callback function
typedef void (* dmaCallback_t) (uint32_t event, uint32_t source, uintptr_t arg) ;

//----------------------------------------------------------------------
//	Some definitions of DMA and BDMA structures easier to use than those of LL

typedef struct dma_s
{
	__IO uint32_t		LISR ;			// DMA low interrupt status register
	__IO uint32_t		HISR ;			// DMA high interrupt status register
	__IO uint32_t		LIFCR ;			// DMA low interrupt flag clear register
	__IO uint32_t		HIFCR ;			// DMA high interrupt flag clear register
	dmaStream_t			stream [8] ;

} dma_t ;

//----------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

void		dmaEnableClock		(dma_t * pDma) ;
void		dmaSetIrq			(const dmaDesc_t * pDmaDesc) ;
void		dmaDisableIrq		(const dmaDesc_t * pDmaDesc) ;
void		dmaSetMode 			(const dmaDesc_t * pDmaDesc, uint32_t mode) ;
void		dmaSetCallback		(const dmaDesc_t * pDmaDesc, dmaCallback_t fn, uintptr_t arg) ;
void		dmaSetCallbackRaw	(dma_t * pDma, uint32_t streamNum, dmaCallback_t fn, uintptr_t arg) ;

// Flags to use with dmaGetFlags(), dmaClearFlags(), dmaGetFlagsRaw(), dmaClearFlagsRaw()
#define DMA_FLAG_FEIF		((uint32_t) 0x00000001u)
#define DMA_FLAG_DMEIF		((uint32_t) 0x00000004u)
#define DMA_FLAG_TEIF		((uint32_t) 0x00000008u)	// Transfer error
#define DMA_FLAG_HTIF		((uint32_t) 0x00000010u)	// Transfer half complete
#define DMA_FLAG_TCIF		((uint32_t) 0x00000020u)	// Transfer complete
#define DMA_FLAG_ALLIF		((uint32_t) 0x0000003Du)

uint32_t	dmaGetFlags			(const dmaDesc_t * pDmaDesc) ;
void		dmaClearFlags		(const dmaDesc_t * pDmaDesc, uint32_t flags) ;
uint32_t	dmaGetFlagsRaw		(dma_t * pDma, uint32_t streamNum) ;
void		dmaClearFlagsRaw	(dma_t * pDma, uint32_t streamNum, uint32_t flags) ;

// Get a pointer to a stream registers
__STATIC_INLINE dmaStream_t * dmaStreamAddressRaw (dma_t * pDma, uint32_t streamNum)
{
	return & pDma->stream [streamNum] ;
}

__STATIC_INLINE dmaStream_t * dmaStreamAddress (const dmaDesc_t * pDmaDesc)
{
	return dmaStreamAddressRaw (DMA_GETDMA (pDmaDesc->dmaNum), DMA_GETSTREAM (pDmaDesc)) ;
}

// Set stream to reset state
__STATIC_INLINE void	dmaStreamResetRaw	(dma_t * pDma, uint32_t streamNum)
{
	dmaStream_t * pStream = dmaStreamAddressRaw (pDma, streamNum) ;

	pStream->CR		= 0u ;
	pStream->NDTR	= 0u ;
	pStream->PAR	= 0u ;
	pStream->M0AR	= 0u ;
	pStream->M1AR	= 0u ;
	pStream->FCR	= (uint32_t) 0x00000021u ;
	dmaClearFlagsRaw (pDma, streamNum, DMA_FLAG_ALLIF) ;
}

__STATIC_INLINE void	dmaStreamReset (const dmaDesc_t * pDmaDesc)
{
	dmaStreamResetRaw (DMA_GETDMA (pDmaDesc->dmaNum), DMA_GETSTREAM (pDmaDesc)) ;
}

// Stop a stream and reset flags
__STATIC_INLINE void	dmaStreamStopRaw	(dma_t *  pDma, uint32_t streamNum)
{
	dmaStream_t * pStream = dmaStreamAddressRaw (pDma, streamNum) ;

	pStream->CR	&= ~DMA_SxCR_EN ;
	dmaClearFlagsRaw (pDma, streamNum, DMA_FLAG_ALLIF) ;
}

__STATIC_INLINE void	dmaStreamStop	(const dmaDesc_t * pDmaDesc)
{
	dmaStreamStopRaw (DMA_GETDMA (pDmaDesc->dmaNum), DMA_GETSTREAM (pDmaDesc)) ;
}

#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// DMABASIC_H_
