/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	extibasic.c		GPIO and software interrupts management

	When		Who	What
	02/09/18	ac	Creation
	22/01/20	ac	Some cosmetic changes
				ac	extiResetInterrupt() changed to extiResetLine()
	30/04/21	ac	Add individual lines callback

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aakernel.h"

#include	"extibasic.h"

//----------------------------------------------------------------------
//	Get interrupt request number from line number 0..22

#define	EXTI_LINE_COUNT	23

static const uint8_t	extiIrqn [EXTI_LINE_COUNT] =
{
		EXTI0_IRQn,			// 0
		EXTI1_IRQn,			// 1
		EXTI2_IRQn,			// 2
		EXTI3_IRQn,			// 3
		EXTI4_IRQn,			// 4
		EXTI9_5_IRQn,		// 5
		EXTI9_5_IRQn,		// 6
		EXTI9_5_IRQn,		// 7
		EXTI9_5_IRQn,		// 8
		EXTI9_5_IRQn,		// 9
		EXTI15_10_IRQn,		// 10
		EXTI15_10_IRQn,		// 11
		EXTI15_10_IRQn,		// 12
		EXTI15_10_IRQn,		// 13
		EXTI15_10_IRQn,		// 14
		EXTI15_10_IRQn,		// 15
		PVD_IRQn,			// 16
		RTC_Alarm_IRQn,		// 17
		OTG_FS_WKUP_IRQn,	// 18
#if (defined STM32F446xx  ||  defined STM32F411xE	||  defined STM32F401xC	||  defined STM32F401xE   ||	\
	 defined STM32F412Cx  ||  defined STM32F412Rx	|| defined STM32F412Vx	|| defined STM32F412Zx	  ||	\
	 defined STM32F410Tx  ||  defined STM32F410Cx   || defined STM32F410Rx  || \
	 defined STM32F413xx  ||  defined STM32F423xx)
		0,					// 19 No Ethernet wake up on F446 / F411 / ...
#else
		ETH_WKUP_IRQn,		// 19
#endif

#if (defined STM32F401xC  ||  defined STM32F401xE   || defined STM32F411xE  ||  defined STM32F413xx  ||  defined STM32F423xx  ||	\
	 defined STM32F412Cx  ||  defined STM32F412Rx	|| defined STM32F412Vx  ||  defined STM32F412Zx  ||	\
	 defined STM32F410Tx  || defined STM32F410Cx    || defined STM32F410Rx)
		0,					// 20 No USB HS on these MCU
#else
		OTG_HS_WKUP_IRQn,	// 20
#endif
		TAMP_STAMP_IRQn,	// 21
		RTC_WKUP_IRQn,		// 22

} ;

static	IRQn_Type	lineToIrqn_ (uint32_t lineNumber)
{
	IRQn_Type num ;

	if (lineNumber < EXTI_LINE_COUNT)
	{
		num = (IRQn_Type) extiIrqn [lineNumber] ;
		if (num != 0)
		{
			return num ;
		}
	}

	AA_ASSERT (0) ;
	return (IRQn_Type) 0 ;
}

//----------------------------------------------------------------------
//	Configure registers for interrupt or event

static extiError_t	extiSetRegisters_ (uint32_t lineNumber, uint32_t priority, uint32_t flags)
{
	uint32_t	pinMask = 1u << lineNumber ;

	EXTI->EMR &= ~pinMask ;
	EXTI->IMR &= ~pinMask ;

	if ((flags & EXTI_FLAG_EVENT) != 0u)
	{
		EXTI->EMR |= pinMask ;
	}

	// Configure trigger active edge
	EXTI->FTSR &= ~pinMask ;
	if ((flags & EXTI_FLAG_FALLING) != 0u)
	{
		EXTI->FTSR |= pinMask ;		// Falling
	}

	EXTI->RTSR &= ~pinMask;
	if ((flags & EXTI_FLAG_RISING) != 0u)
	{
		EXTI->RTSR |= pinMask ;		// Rising
	}

	// Add to NVIC if interrupt is requested
	if ((flags & EXTI_FLAG_INTR) != 0u)
	{
		IRQn_Type irqNum = lineToIrqn_ (lineNumber) ;

		if (irqNum != 0u)
		{
			EXTI->IMR |= pinMask ;
			NVIC_EnableIRQ   (irqNum) ;
			NVIC_SetPriority (irqNum, priority) ;
		}
	}
	return EXTI_ENONE ;
}

//----------------------------------------------------------------------
//	Initialize an interrupt or an event on a GPIO pin
//	Pin: 0..15
//	If the flag EXTI_FLAG_GPIO_NOCONF is not present, the pin is configured using pPinDesc
//	Allowed flags: EXTI_FLAG_INTR, EXTI_FLAG_EVENT, EXTI_FLAG_RISING, EXTI_FLAG_FALLING

extiError_t	extiInitGpio	(const gpioPinDesc_t * pPinDesc, uint32_t priority, uint32_t flags)
{
	uint32_t		reg ;

	if (pPinDesc->pin > 15u)
	{
		AA_ASSERT (0) ;
		return EXTI_EFAIL ;
	}

	// Configure pin
	if ((flags & EXTI_FLAG_GPIO_NOCONF) == 0)
	{
		gpioConfigurePin (pPinDesc) ;
	}

	// Enable SYSCFG clock
	RCC->APB2ENR |= RCC_APB2ENR_SYSCFGEN ;

	// Set proper GPIO port in SYSCFG->EXTICRn
	reg = SYSCFG->EXTICR [pPinDesc->pin / 4u] ;
	reg &= ~(0x0Fu << (4u * (pPinDesc->pin & 0x03u))) ;
	reg |= (uint32_t) (((pPinDesc->port - (uint8_t) 'A') << (4u * (pPinDesc->pin & 0x03u)))) ;
	SYSCFG->EXTICR [pPinDesc->pin / 4u] = reg ;

	return extiSetRegisters_ (pPinDesc->pin, priority, flags) ;
}

//----------------------------------------------------------------------
//	Initialize an interrupt or an event on a line 16..EXTI_LINE_COUNT-1
//	Allowed flags: EXTI_FLAG_INTR, EXTI_FLAG_EVENT, EXTI_FLAG_RISING, EXTI_FLAG_FALLING

extiError_t	extiInitLine		(uint32_t lineNumber, uint32_t priority, uint32_t flags)
{
	if (lineNumber < 16u  ||  lineNumber >= EXTI_LINE_COUNT)
	{
		AA_ASSERT (0) ;
		return EXTI_EFAIL ;
	}

	return extiSetRegisters_ (lineNumber, priority, flags) ;
}

//----------------------------------------------------------------------
//	Initialize an exti line 0..EXTI_LINE_COUNT-1 for software interrupt
//	This doesn't allows to configure a software event

extiError_t	extiInitSwi		(uint32_t lineNumber, uint32_t priority)
{
	if (lineNumber >= EXTI_LINE_COUNT)
	{
		AA_ASSERT (0) ;
		return EXTI_EFAIL ;
	}

	// Force trigger on rising edge
	return extiSetRegisters_ (lineNumber, priority, EXTI_FLAG_INTR | EXTI_FLAG_RISING) ;
}

//----------------------------------------------------------------------

void		extiResetLine	(uint32_t lineNumber)
{
	if (lineNumber < EXTI_LINE_COUNT)
	{
		EXTI->IMR	&= ~(1u << lineNumber) ;
		EXTI->EMR	&= ~(1u << lineNumber) ;
		EXTI->FTSR	&= ~(1u << lineNumber) ;
		EXTI->RTSR	&= ~(1u << lineNumber) ;
	}

	if (lineNumber < 16)
	{
		// Clear port in EXTICRn. In reality this selects the port A
		SYSCFG->EXTICR [lineNumber >> 2u] &= ~(0x0Fu << (4u * (lineNumber & 0x03u))) ;
	}
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// Interrupt handlers

#if (WITH_EXTI_0 == 1)
void EXTI0_IRQHandler (void) ;
void EXTI0_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR & EXTI_PR_PR0) != 0u)
	{
		// Clear bit
		EXTI->PR = EXTI_PR_PR0 ;
		// Call user callback
		extiCallback0 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_1 == 1)
void EXTI1_IRQHandler (void) ;
void EXTI1_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR & EXTI_PR_PR1) != 0u)
	{
		// Clear bit
		EXTI->PR = EXTI_PR_PR1 ;
		// Call user callback
		extiCallback1 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_2 == 1)
void EXTI2_IRQHandler (void) ;
void EXTI2_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR & EXTI_PR_PR2) != 0u)
	{
		// Clear bit
		EXTI->PR = EXTI_PR_PR2 ;
		// Call user callback
		extiCallback2 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_3 == 1)
void EXTI3_IRQHandler (void) ;
void EXTI3_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR & EXTI_PR_PR3) != 0u)
	{
		// Clear bit
		EXTI->PR = EXTI_PR_PR3 ;
		// Call user callback
		extiCallback3 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_4 == 1)
void EXTI4_IRQHandler (void) ;
void EXTI4_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR & EXTI_PR_PR4) != 0u)
	{
		// Clear bit
		EXTI->PR = EXTI_PR_PR4 ;
		// Call user callback
		extiCallback4 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_9_5 == 1)
void EXTI9_5_IRQHandler (void) ;
void EXTI9_5_IRQHandler (void)
{
	uint32_t	reg ;

	aaIntEnter () ;

	reg = EXTI->PR >> 5 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR = EXTI_PR_PR5 ;
		extiCallback5 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR = EXTI_PR_PR6 ;
		extiCallback6 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR = EXTI_PR_PR7 ;
		extiCallback7 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR = EXTI_PR_PR8 ;
		extiCallback8 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR = EXTI_PR_PR9 ;
		extiCallback9 (EXTI_FLAG_RISING) ;
	}

	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_15_10 == 1)
void EXTI15_10_IRQHandler (void) ;
void EXTI15_10_IRQHandler (void)
{
	uint32_t	reg ;

	aaIntEnter () ;

	reg = EXTI->PR >> 10 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR = EXTI_PR_PR10 ;
		extiCallback10 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR = EXTI_PR_PR11 ;
		extiCallback11 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR = EXTI_PR_PR12 ;
		extiCallback12 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;
	if ((reg & 1) != 0)
	{
		EXTI->PR = EXTI_PR_PR13 ;
		extiCallback13 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR = EXTI_PR_PR14 ;
		extiCallback14 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR = EXTI_PR_PR15 ;
		extiCallback15 (EXTI_FLAG_RISING) ;
	}

	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_PVD == 1)
void PVD_IRQHandler (void) ;
void PVD_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR & EXTI_PR_PR16) != 0u)
	{
		// Clear bit
		EXTI->PR = EXTI_PR_PR16 ;
		// Call user callback
		extiCallback16 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_RTC_ALRM == 1)
void RTC_Alarm_IRQHandler (void) ;
void RTC_Alarm_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR & EXTI_PR_PR17) != 0u)
	{
		// Clear bit
		EXTI->PR = EXTI_PR_PR17 ;
		// Call user callback
		extiCallback17 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_OTG_FS_WKUP == 1)
void OTG_FS_WKUP_IRQHandler (void) ;
void OTG_FS_WKUP_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR & EXTI_PR_PR18) != 0u)
	{
		// Clear bit
		EXTI->PR = EXTI_PR_PR18 ;
		// Call user callback
		extiCallback18 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_ETH_WKUP == 1)
void ETH_WKUP_IRQHandler (void) ;
void ETH_WKUP_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR & EXTI_PR_PR19) != 0u)
	{
		// Clear bit
		EXTI->PR = EXTI_PR_PR19 ;
		// Call user callback
		extiCallback19 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_OTG_HS_WKUP == 1)
void OTG_HS_WKUP_IRQHandler (void) ;
void OTG_HS_WKUP_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR & EXTI_PR_PR20) != 0u)
	{
		// Clear bit
		EXTI->PR = EXTI_PR_PR20 ;
		// Call user callback
		extiCallback20 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_TAMP_STAMP == 1)
void TAMP_STAMP_IRQHandler (void) ;
void TAMP_STAMP_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR & EXTI_PR_PR21) != 0u)
	{
		// Clear bit
		EXTI->PR = EXTI_PR_PR21 ;
		// Call user callback
		extiCallback21 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_RTC_WKUP == 1)
void RTC_WKUP_IRQHandler (void) ;
void RTC_WKUP_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR & EXTI_PR_PR22) != 0u)
	{
		// Clear bit
		EXTI->PR = EXTI_PR_PR22 ;
		// Call user callback
		extiCallback22 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------
//	Default callback for all lines

void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback0		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback1		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback2		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback3		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback4		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback5		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback6		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback7		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback8		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback9		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback10		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback11		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback12		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback13		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback14		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback15		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback16		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback17		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback18		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback19		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback20		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback21		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback22		(uint32_t flags) ;

static	void extiDefaultCallback (uint32_t flags)
{
	(void) flags ;
}

//----------------------------------------------------------------------
