/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	extibasic.c	EXTI and software interrupt

	When		Who	What
	02/09/18	ac	Creation
	22/01/20	ac	extiResetInterrupt() changed to extiResetLine()
	30/04/21	ac	Add individual lines callback

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined EXTIBASIC_H_
#define	EXTIBASIC_H_
//----------------------------------------------------------------------

// Define which lines interrupt handlers should be managed by extibasic
// Those who are at 0 can have a user-managed interrupt handlers
// Specific to F4
#define	WITH_EXTI_0				1	// For exti test (BlackPill button)
#define	WITH_EXTI_1				0
#define	WITH_EXTI_2				0
#define	WITH_EXTI_3				0
#define	WITH_EXTI_4				1	// For swi test
#define	WITH_EXTI_9_5			1	// 6  ZLI test
#define	WITH_EXTI_15_10			1	// 12 slave SPI test, 13 Nucleo button
#define	WITH_EXTI_PVD			0
#define	WITH_EXTI_RTC_ALRM		0
#define	WITH_EXTI_OTG_FS_WKUP	0
#define	WITH_EXTI_ETH_WKUP		0
#define	WITH_EXTI_OTG_HS_WKUP	0
#define	WITH_EXTI_TAMP_STAMP	0
#define	WITH_EXTI_RTC_WKUP		0

#include	"gpiobasic.h"
#include	"stm32f4xx_ll_exti.h"
#include	"stm32f4xx_ll_bus.h"

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Definition of returned values

typedef enum
{
	EXTI_ENONE		= 0u,
	EXTI_EFAIL		= 1u

} extiError_t ;

// Bits definition for flag argument
// They are not exclusive
#define	EXTI_FLAG_INTR			0x0001u
#define	EXTI_FLAG_EVENT			0x0002u

#define	EXTI_FLAG_RISING		0x0004u
#define	EXTI_FLAG_FALLING		0x0008u

#define	EXTI_FLAG_GPIO_NOCONF	0x8000u		// extiInitGpio() doesn't configure the GPIO pin (the user does it)

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

#ifdef __cplusplus
extern "C" {
#endif

extiError_t	extiInitGpio		(const gpioPinDesc_t * pPinDesc, uint32_t priority, uint32_t flags) ;
extiError_t	extiInitLine		(uint32_t lineNumber, uint32_t priority, uint32_t flags) ;
extiError_t	extiInitSwi			(uint32_t lineNumber, uint32_t priority) ;
void		extiResetLine		(uint32_t lineNumber) ;

// This function generate a software interrupt on the specified line
__STATIC_INLINE void extiSwiTrigger	(uint32_t lineNumber)
{
	EXTI->SWIER = 1u << lineNumber ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// The EXTI callback function type
// These functions are called from library interrupt handler each time an interrupt occurs on EXTI line.
// flags: combination of EXTI_FLAG_RISING and EXTI_FLAG_FALLING
// On F4/L4/H7 families there is only one event so EXTI_FLAG_RISING is used for every event
typedef void (* extiCallaback_t) (uint32_t flags) ;

/*
	When you use an EXTI line interrupt,
	you must have a callback function whose name is build from the word "extiCallback" concatenated to the line number.
	Example: for the line 12 the callback name is extiCallback12.
	The macro EXTI_CALLBACK help to build the callback name.

	#define	MY_EXTI_LINE			12

	Then use your define to write the callback function:

	void EXTI_CALLBACK(MY_EXTI_LINE) (uint32_t flags)
	{
	}

	Using the EXTI_CALLBACK macro, when you change the EXTI line number,
	you don't have to change the callback name.
*/
#define	EXTI_CB_BUILDER(x)		extiCallback ## x
#define	EXTI_CALLBACK(x)		EXTI_CB_BUILDER(x)


// The EXTI callbacks for configurable EXTI lines
// The user should define its own functions with these names
void	extiCallback0		(uint32_t flags) ;
void	extiCallback1		(uint32_t flags) ;
void	extiCallback2		(uint32_t flags) ;
void	extiCallback3		(uint32_t flags) ;
void	extiCallback4		(uint32_t flags) ;
void	extiCallback5		(uint32_t flags) ;
void	extiCallback6		(uint32_t flags) ;
void	extiCallback7		(uint32_t flags) ;
void	extiCallback8		(uint32_t flags) ;
void	extiCallback9		(uint32_t flags) ;
void	extiCallback10		(uint32_t flags) ;
void	extiCallback11		(uint32_t flags) ;
void	extiCallback12		(uint32_t flags) ;
void	extiCallback13		(uint32_t flags) ;
void	extiCallback14		(uint32_t flags) ;
void	extiCallback15		(uint32_t flags) ;
void	extiCallback16		(uint32_t flags) ;
void	extiCallback17		(uint32_t flags) ;
void	extiCallback18		(uint32_t flags) ;
void	extiCallback19		(uint32_t flags) ;
void	extiCallback20		(uint32_t flags) ;
void	extiCallback21		(uint32_t flags) ;
void	extiCallback22		(uint32_t flags) ;

#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// EXTIBASIC_H_
