/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	i2cbasic.h	Master and Slave basic routines
				Header for applications
				For STM32 F1 F2 F4 L1

	When		Who	What
	01/03/18	ac	Creation

	This code is a rework of STMicroelectronics application note AN2824

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined I2CBASIC_H_
#define I2CBASIC_H_
//----------------------------------------------------------------------
#include	"gpiobasic.h"

//----------------------------------------------------------------------
//	I2C API configuration

// To define the supported I2C devices

#define	WITH_I2C_1		1
#define	WITH_I2C_2		1
#define	WITH_I2C_3		0
#define	WITH_I2C_4		0	// Not in all F4xx devices

// If one WITH_I2C_x is 1 then i2cArray in i2cbasic.c is not empty and must be defined
#if (WITH_I2C_1 == 1  ||  WITH_I2C_2 == 1  ||  WITH_I2C_3 == 1  ||  WITH_I2C_4 == 1)
	#define	WITH_I2C_X		1
#endif

// Check if selected I2C are defined for this MCU
#if (WITH_I2C_1 == 1  &&  ! defined I2C1)	||	\
	(WITH_I2C_2 == 1  &&  ! defined I2C2)	||	\
	(WITH_I2C_3 == 1  &&  ! defined I2C3)	||	\
	(WITH_I2C_4 == 1  &&  ! defined I2C4)
	#error	"I2C does not exist in this MCU"
#endif

// I2C interrupt priority
#define	I2C_INT_PRIORITY		BSP_IRQPRIOMAX_MINUS(2)

//----------------------------------------------------------------------

typedef enum
{
	// Flags to use with i2cMasterStart()
	I2C_TX			= 0x01u,	// Direction flag of i2cMasterStart(): Master transmission is requested
	I2C_RX			= 0x02u,	// Direction flag of i2cMasterStart(): Master reception is requested

	// Flags to use with i2cMasterXfer()/i2cSlaveXfer()
	I2C_NEXT		= 0x08u,	// Not last block to exchange
	I2C_LAST		= 0x10u,	// Last block to exchange
	I2C_NOBLOCK		= 0x20u		// Use with i2cMasterStart()/i2cMasterXfer()/i2cSlaveXfer() for non blocking functions

} i2cMode_t ;

// The values returned by the I2C API
typedef enum
{
	// Return values for all functions
	I2C_ENONE		= 0u,		// No error
	I2C_EFAIL		= 1u,		// Some error
	I2C_ESTATE		= 2u,		// Not allowed for this I2C state

	// More return values for i2cResult()
	I2C_EOK			= 3u,		// Ended without error
	I2C_EBUSY		= 4u,		// Interface is busy
	I2C_EAF			= 5u,		// Acknowledge fail
	I2C_EARLO		= 6u,		// Arbitration lost
	I2C_ETMO		= 7u,		// Timeout
	I2C_EBERR		= 8u,		// Bus error
	I2C_EOVR		= 9u,		// I2C Overflow
	I2C_ESIZE		= 10u,		// Slave / master buffer size non-coherent:
								// Master stopped receive before slave buffer size
								// Master sent more data than the slave buffer size
	I2C_ESLAVERX	= 11u,		// Slave addressed as receiver
	I2C_ESLAVETX	= 12u		// Slave addressed as transmitter
} i2cStatus_t ;


//----------------------------------------------------------------------
//	I2C device descriptor

typedef struct i2cDesc_s
{
	uint16_t		id ;			// Unique descriptor identifier
	uint8_t			periphRstPos ;	// e.g. RCC_APB1RSTR_I2C1RST_Pos
	uint8_t			periphEnPos ;	// e.g. RCC_APB1ENR_I2C1EN_Pos
	gpioPinDesc_t	sda ;			// SDA GPIO descriptor
	gpioPinDesc_t	scl ;			// SCL GPIO descriptor
	I2C_TypeDef		* pI2c ;		// e.g. I2C1

	IRQn_Type		evIrqn ;		// Event interrupt number
	IRQn_Type		erIrqn ;		// Error interrupt number
	uint32_t		irqPriority ;	// Same interrupt priority for both interrupts

}  i2cDesc_t ;

//----------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

// The I2C handle returned by i2cConfigure()
typedef void * i2cHandle_t ;

// Configuration API
const i2cDesc_t *	i2cGetDesc				(uint32_t i2cId) ;
i2cStatus_t			i2cConfigure 			(const i2cDesc_t * pI2cDesc, uint32_t speed, aaSignal_t signal, i2cHandle_t * pHdl) ;
void 				i2cDeactivate 			(i2cHandle_t hdl) ;
i2cStatus_t			i2cResult				(i2cHandle_t hdl, uint32_t * pSize) ;

// Master API
i2cStatus_t 		i2cMasterStart			(i2cHandle_t hdl, uint32_t slaveAddress, uint32_t direction) ;
void				i2cMasterStop			(i2cHandle_t hdl) ;
i2cStatus_t			i2cMasterXfer			(i2cHandle_t hdl, uint8_t * pBuffer, uint32_t numByte, uint32_t nextOrLast) ;
i2cStatus_t			i2cMasterWrite			(i2cHandle_t hdl, uint32_t slaveAddress, uint8_t * pBuffer, uint32_t numByteToWrite) ;
i2cStatus_t			i2cMasterRead			(i2cHandle_t hdl, uint32_t slaveAddress, uint8_t * pBuffer, uint32_t numByteToRead) ;

i2cStatus_t			i2cMasterProbeDevice	(i2cHandle_t hdl, uint32_t slaveAddress) ;

// Slave API
void				i2cSlaveInit			(i2cHandle_t hdl, uint32_t ownAddress1) ;
i2cStatus_t 		i2cSlaveListen			(i2cHandle_t hdl, uint32_t flags) ;
i2cStatus_t			i2cSlaveXfer			(i2cHandle_t hdl, uint8_t * pBuffer, uint32_t numByte, uint32_t nextOrLast) ;

#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// I2CBASIC_H_
