/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	uart.c		Basic UART management for STM32F4xx

	When		Who	What
	10/07/17	ac	Creation
	16/01/18	ac	Port to STM32F429
	02/09/18	ac	modification of pin desc to add flags
					Add flag UART_FLAG_NOIT : don't use interrupts with this uart
	24/01/20	ac	Add 'flags' parameter to uartInit(). Improved 'no interrupt' handling
	12/04/23	ac	Add conditional GPIO configuration (the GPIO is ignored if the port is 15 or 'P')
					This allows RX only or TX only UART


	Use LL driver for initialization (It's easier)
	Use register access for IO, it is more efficient
----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

/*
-----------------------------------------------------------------------------
	UART can be used before the kernel is started and interrupts are enabled.
	If aaIsRunning==0, then the UART functions use polling to send and receive characters
	Interrupts should not be enabled before the kernel is started by aaStart_() function.

	The uartPutChar() and uartGetChar() functions can not be called from an ISR
-----------------------------------------------------------------------------
*/
#include	"aa.h"
#include	"aakernel.h"
#include	"uartbasic.h"
#include	"gpiobasic.h"
#include	"rccbasic.h"

#include	"stm32f4xx_ll_usart.h"
#include	"stm32f4xx_ll_bus.h"

//-----------------------------------------------------------------------------
// Define TX and RX buffers sizes for each UART

#define UART1_TX_SIZE	1024
#define	UART1_RX_SIZE	1024

#define	UART2_TX_SIZE	64
#define	UART2_RX_SIZE	32

#define	UART3_TX_SIZE	1024
#define	UART3_RX_SIZE	32

#define	UART4_TX_SIZE	64
#define	UART4_RX_SIZE	32

#define	UART5_TX_SIZE	64
#define	UART5_RX_SIZE	32

#define	UART6_TX_SIZE	64
#define	UART6_RX_SIZE	32

#if (defined UART7)
#define	UART7_TX_SIZE	64
#define	UART7_RX_SIZE	32
#endif

#if (defined UART8)
#define	UART8_TX_SIZE	64
#define	UART8_RX_SIZE	32
#endif

typedef uint16_t		uartTx_t ;
typedef uint16_t		uartRx_t ;

//-----------------------------------------------------------------------------
//	UART descriptor

//	The pin descriptor is a 32 bits word: xxxx xxxx xxxx xxxx xxxx xxxx xxxxxxxx
//					                      rxAf txAf txPt txPt rxPn txPn    flags

#define	pin2desc(txPin, txPort, rxPin, rxPort, txAf, rxAf, flags)	        \
				(	(((txPin)  & 0x0Fu) << 8u)  | (((txPort) & 0x0Fu) << 16u)  |\
					(((rxPin)  & 0x0Fu) << 12u) | (((rxPort) & 0x0Fu) << 20u)  |\
					(((txAf)   & 0x0Fu) << 24u) | (((rxAf)   & 0x0Fu) << 28u)  |\
					((flags) & UART_FLAG_MASK) \
				)

#define	desc2TxPin(desc)		(((desc) >> 8u)  & 0x0Fu)
#define	desc2TxPort(desc)		(((desc) >> 16u) & 0x0Fu)
#define	desc2RxPin(desc)		(((desc) >> 12u) & 0x0Fu)
#define	desc2RxPort(desc)		(((desc) >> 20u) & 0x0Fu)
#define	desc2TxAf(desc)			(((desc) >> 24u) & 0x0Fu)
#define	desc2RxAf(desc)			(((desc) >> 28u) & 0x0Fu)
#define	desc2Flags(desc)		((desc) & UART_FLAG_MASK)

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// The UART descriptor structure

typedef struct
{
	uint32_t		desc ;			// Configuration of tx and rx pins
	USART_TypeDef	* uartBase ;	// UARTx from stm32f4xx.h (pointer to registers)

					// Buffers management
	uartTx_t		txSize ;
	uartTx_t		iTxRead, iTxWrite, txCount ;
	uartRx_t		rxSize ;
	uartRx_t		iRxRead, iRxWrite, rxCount ;
	uint8_t			* pTxBuffer ;
	uint8_t			* pRxBuffer ;

	aaDriverDesc_t	txList ;		// The list of tasks waiting for Tx
	aaDriverDesc_t	rxList ;		// The list of tasks waiting for Rx

} uartDesc_t ;

static	uartDesc_t	* uartSelect (uint32_t	uart) ;

//-----------------------------------------------------------------------------

#if (WITH_UART1 == 1)

STATIC	uint8_t		uart1Tx [UART1_TX_SIZE] ;
STATIC	uint8_t		uart1Rx [UART1_RX_SIZE] ;

STATIC	uartDesc_t		uart1 =
{
		pin2desc (9u,  0u, 10u, 0u, AA_GPIO_AF_7, AA_GPIO_AF_7, 0u),	// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
//		pin2desc (6u,  1u,  7u, 1u, AA_GPIO_AF_7, AA_GPIO_AF_7, 0u),	// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
//		pin2desc (10u, 2u, 11u, 2u, AA_GPIO_AF_7, AA_GPIO_AF_7, 0u),	// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
		USART1,				// UARTx

		UART1_TX_SIZE,
		0u, 0u, 0u,
		UART1_RX_SIZE,
		0u, 0u, 0u,
		uart1Tx,
		uart1Rx,

		{ { NULL, NULL } },	// txList
		{ { NULL, NULL } },	// rxList
} ;

#endif

//-----------------------------------------------------------------------------

#if (WITH_UART2 == 1)

STATIC	uint8_t		uart2Tx [UART2_TX_SIZE] ;
STATIC	uint8_t		uart2Rx [UART2_RX_SIZE] ;

STATIC	uartDesc_t		uart2 =
{
		pin2desc (2u,  0u,  3u, 0u, AA_GPIO_AF_7, AA_GPIO_AF_7, 0u),	// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
		USART2,				// UARTx

		UART2_TX_SIZE,
		0u, 0u, 0u,
		UART2_RX_SIZE,
		0u, 0u, 0u,
		uart2Tx,
		uart2Rx,

		{ { NULL, NULL } },	// txList
		{ { NULL, NULL } },	// rxList
} ;

#endif

//-----------------------------------------------------------------------------

#if (WITH_UART3 == 1)

// Debug port on ST-LINK connector
STATIC	uint8_t		uart3Tx [UART3_TX_SIZE] ;
STATIC	uint8_t		uart3Rx [UART3_RX_SIZE] ;

STATIC	uartDesc_t		uart3 =
{
//		pin2desc (10u, 1u, 11u, 1u, AA_GPIO_AF_7, AA_GPIO_AF_7, 0u),		// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
//		pin2desc (10u, 2u, 11u, 2u, AA_GPIO_AF_7, AA_GPIO_AF_7, 0u),		// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
		pin2desc ( 8u, 3u,  9u, 3u, AA_GPIO_AF_7, AA_GPIO_AF_7, 0u),		// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
		USART3,				// UARTx

		UART3_TX_SIZE,
		0u, 0u, 0u,
		UART3_RX_SIZE,
		0u, 0u, 0u,
		uart3Tx,
		uart3Rx,

		{ { NULL, NULL } },	// txList
		{ { NULL, NULL } },	// rxList
} ;

#endif

//-----------------------------------------------------------------------------

#if (WITH_UART4 == 1)

STATIC	uint8_t		uart4Tx [UART4_TX_SIZE] ;
STATIC	uint8_t		uart4Rx [UART4_RX_SIZE] ;

STATIC	uartDesc_t		uart4 =
{
		pin2desc ( 0u, 0u,  1u, 0u, AA_GPIO_AF_7, AA_GPIO_AF_7, 0u),		// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
		UART4,				// UARTx

		UART4_TX_SIZE,
		0u, 0u, 0u,
		UART4_RX_SIZE,
		0u, 0u, 0u,
		uart4Tx,
		uart4Rx,

		{ { NULL, NULL } },	// txList
		{ { NULL, NULL } },	// rxList
} ;

#endif

//-----------------------------------------------------------------------------

#if (WITH_UART5 == 1)

STATIC	uint8_t		uart5Tx [UART5_TX_SIZE] ;
STATIC	uint8_t		uart5Rx [UART5_RX_SIZE] ;

STATIC	uartDesc_t		uart5 =
{
		pin2desc (12u, 2u,  2u, 3u, AA_GPIO_AF_7, AA_GPIO_AF_7, 0u),		// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
		UART5,				// UARTx

		UART5_TX_SIZE,
		0u, 0u, 0u,
		UART5_RX_SIZE,
		0u, 0u, 0u,
		uart5Tx,
		uart5Rx,

		{ { NULL, NULL } },	// txList
		{ { NULL, NULL } },	// rxList
} ;

#endif

//-----------------------------------------------------------------------------

#if (WITH_UART6 == 1)

STATIC	uint8_t		uart6Tx [UART6_TX_SIZE] ;
STATIC	uint8_t		uart6Rx [UART6_RX_SIZE] ;

STATIC	uartDesc_t		uart6 =
{
		pin2desc ( 6u, 2u,  7u, 2u, AA_GPIO_AF_8, AA_GPIO_AF_8, 0u),		// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
//		pin2desc (14u, 6u, 15u, 6u, AA_GPIO_AF_8, AA_GPIO_AF_8, 0u),		// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
		USART6,				// UARTx

		UART6_TX_SIZE,
		0u, 0u, 0u,
		UART6_RX_SIZE,
		0u, 0u, 0u,
		uart6Tx,
		uart6Rx,

		{ { NULL, NULL } },	// txList
		{ { NULL, NULL } },	// rxList
} ;

#endif

//-----------------------------------------------------------------------------

#if (WITH_UART7 == 1  &&  defined UART7)

STATIC	uint8_t		uart7Tx [UART7_TX_SIZE] ;
STATIC	uint8_t		uart7Rx [UART7_RX_SIZE] ;

STATIC	uartDesc_t		uart7 =
{
//		pin2desc ( 8u, 4u,  7u, 4u, AA_GPIO_AF_8, AA_GPIO_AF_8, 0u),		// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
		pin2desc ( 7u, 5u,  6u, 5u, AA_GPIO_AF_8, AA_GPIO_AF_8, 0u),		// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
		UART7,				// UARTx

		UART7_TX_SIZE,
		0u, 0u, 0u,
		UART7_RX_SIZE,
		0u, 0u, 0u,
		uart7Tx,
		uart7Rx,

		{ { NULL, NULL } },	// txList
		{ { NULL, NULL } },	// rxList
} ;

#endif

//-----------------------------------------------------------------------------

#if (WITH_UART8 == 1  &&  defined UART8)

STATIC	uint8_t		uart8Tx [UART8_TX_SIZE] ;
STATIC	uint8_t		uart8Rx [UART8_RX_SIZE] ;

STATIC	uartDesc_t		uart8 =
{
		pin2desc ( 1u, 4u,  0u, 4u, AA_GPIO_AF_8, AA_GPIO_AF_8, 0u),		// txPin, txPort, rxPin, rxPort, txAf, rxAf, flags
		UART8,				// UARTx

		UART8_TX_SIZE,
		0u, 0u, 0u,
		UART8_RX_SIZE,
		0u, 0u, 0u,
		uart8Tx,
		uart8Rx,

		{ { NULL, NULL } },	// txList
		{ { NULL, NULL } },	// rxList
} ;

#endif

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// To check a flag in a descriptor flags field

__STATIC_INLINE	uint32_t checkDescFlag (uint32_t desc, uint32_t flag)
{
	return ((desc & flag) == 0u) ? 0u : 1u ;
}

//-----------------------------------------------------------------------------
//	Set a flag to UART descriptor
//	BEWARE:
//	If interrupts are disabled while the pTxBuffer is not empty, these characters will not be sent
//	If interrupts are disabled while tasks are waiting,
//	these tasks will not be awakened until interrupts are enabled again

aaError_t	uartSetFlag		(uartHandle_t hUsart, uint32_t flag, uint32_t bSet)
{
	uartDesc_t				* pUart = (uartDesc_t *) hUsart ;
	volatile USART_TypeDef	* uartBase ;	// UARTx

	if (pUart == NULL  ||  (flag & UART_FLAG_MASK) == 0u)
	{
		return AA_EARG ;
	}
	uartBase = pUart->uartBase ;

	if (bSet != 0u)
	{
		// Set flag
		// If UART_FLAG_NOIT is not set in desc, and set in flag, then disable interrupts
		if (checkDescFlag (pUart->desc, UART_FLAG_NOIT) == 0u  &&   (flag & UART_FLAG_NOIT) != 0u)
		{
			uartBase->CR1 &= ~(USART_CR1_RXNEIE | USART_CR1_TXEIE) ;	// Disable RX and TX interrupt

			// Wait until the current character in the UART is sent
			while ((uartBase->SR & USART_SR_TXE) == 0u)
			{
			}
		}
		pUart->desc |= flag & UART_FLAG_MASK ;
	}
	else
	{
		// Clear flag
		// If UART_FLAG_NOIT is set in desc, and set in flag, then enable interrupts
		if (checkDescFlag (pUart->desc, UART_FLAG_NOIT) != 0u  &&  (flag & UART_FLAG_NOIT) != 0u)
		{
			uartBase->CR1 |= USART_CR1_RXNEIE ;		// Enable RX interrupt

			// If pTxBuffer is not empty then enable TX interrupt to send these characters
			if (pUart->txCount != 0u)
			{
				uartBase->CR1 |= USART_CR1_TXEIE ;	// Enable TX interrupt
			}
		}
		pUart->desc &= (~flag & UART_FLAG_MASK) ;
	}
	return AA_ENONE ;
}

//-----------------------------------------------------------------------------
//	Get a flag from UART descriptor

aaError_t	uartGetFlag		(uartHandle_t hUsart, uint32_t * pFlag)
{
	uartDesc_t	* pUart = (uartDesc_t *) hUsart ;

	if (pUart == NULL)
	{
		return AA_EARG ;
	}
	* pFlag = desc2Flags (pUart->desc) ;
	return AA_ENONE ;
}

//-----------------------------------------------------------------------------
//	Returns pointer to UART descriptor structure from UART number
//	UART number 1 to 8

static	uartDesc_t	* uartSelect (uint32_t	uart)
{
	#if (WITH_UART1 == 1)
		if (uart == 1)
		{
			return & uart1 ;
		}
	#endif

	#if (WITH_UART2 == 1)
		if (uart == 2)
		{
			return & uart2 ;
		}
	#endif

	#if (WITH_UART3 == 1)
		if (uart == 3)
		{
			return & uart3 ;
		}
	#endif

	#if (WITH_UART4 == 1)
		if (uart == 4)
		{
			return & uart4 ;
		}
	#endif

	#if (WITH_UART5 == 1)
		if (uart == 5)
		{
			return & uart5 ;
		}
	#endif

	#if (WITH_UART6 == 1)
		if (uart == 6)
		{
			return & uart6 ;
		}
	#endif

	#if (WITH_UART7 == 1  &&  defined UART7)
		if (uart == 7)
		{
			return & uart7 ;
		}
	#endif

	#if (WITH_UART8 == 1  &&  defined UART8)
		if (uart == 8)
		{
			return & uart8 ;
		}
	#endif

	(void) uart ;
	AA_ASSERT (0) ;
	return NULL ;
}

//-----------------------------------------------------------------------------
//	Returns count of available characters in TX buffer

uint32_t	uartCheckPutChar (uartHandle_t hUsart)
{
	uartDesc_t				* pUart = (uartDesc_t *) hUsart ;
	volatile USART_TypeDef	* uartBase ;	// UARTx

	if (pUart == NULL)
	{
		return 0u ;
	}
	uartBase = pUart->uartBase ;

	// If the kernel is not started, don't use RX interrupt/buffer
	if (aaIsRunning == 0u  ||  checkDescFlag (pUart->desc, UART_FLAG_NOIT) != 0u)
	{
		return ((uartBase->SR & USART_SR_TXE) == 0u) ? 0u : 1u ;
	}

	return (uint32_t) (pUart->txSize - pUart->txCount) ;
}

//-----------------------------------------------------------------------------
//	Send one char to UART. Blocking call if TX buffer is full.

void uartPutChar (uartHandle_t hUsart, char cc)
{
	uartDesc_t				* pUart = (uartDesc_t *) hUsart ;
	volatile USART_TypeDef	* uartBase ;	// UARTx
	uint32_t				flagLF = 0u ;

	if (pUart == NULL)
	{
		return ;
	}
	uartBase = pUart->uartBase ;

	// If cc is LF, then maybe send CR-LF
	if (checkDescFlag (pUart->desc, UART_FLAG_LFCRLF) != 0u  &&  cc == '\n')
	{
		flagLF = 1u ;
		cc = '\r' ;
	}

	// If the kernel is not started, don't use TX interrupt/buffer
	if (aaIsRunning == 0u  ||  checkDescFlag (pUart->desc, UART_FLAG_NOIT) != 0u)
	{
		uartBase->DR = cc ;
		// TXE must be 1 when the first char will be send after the kernel start
		// So wait until the end of the transmission of this character
		while ((uartBase->SR & USART_SR_TXE) == 0u)
		{
		}

		if (flagLF != 0u)
		{
			uartBase->DR = '\n' ;
			while ((uartBase->SR & USART_SR_TXE) == 0u)
			{
			}
		}
		return ;
	}

	// The kernel is started and interrupts are enabled, use TX buffer and TX interrupt
	while (1)
	{
		while (1)
		{
			aaCriticalEnter () ;
			if (pUart->txCount == pUart->txSize)
			{
				// Buffer is full, wait. The critical section will be released by aaIoWait().
				(void) aaIoWait (& pUart->txList, 0u, 0u) ;	// Not ordered, no timeout
			}
			else
			{
				break ;		// Not full. Note: exit while inside the critical section
			}
		}

		if (pUart->txCount != 0u  ||  (uartBase->SR & USART_SR_TXE) == 0u)
		{
			pUart->pTxBuffer [pUart->iTxWrite++] = (uint8_t) cc ;
			if (pUart->iTxWrite == pUart->txSize)
			{
				pUart->iTxWrite = 0u ;
			}
			pUart->txCount++ ;
		}
		else
		{
			uartBase->DR = cc ;
			uartBase->CR1 |= USART_CR1_TXEIE ;	// Enable TX interrupt
		}
		aaCriticalExit () ;

		if (flagLF != 0u)
		{
			flagLF = 0u ;
			cc = '\n'  ;
			continue ;
		}
		break ;
	}
}

//-----------------------------------------------------------------------------
//	Returns count of available characters in RX buffer

uint32_t	uartCheckGetChar (uartHandle_t hUsart)
{
	uartDesc_t				* pUart = (uartDesc_t *) hUsart ;
	volatile USART_TypeDef	* uartBase ;	// UARTx

	if (pUart == NULL)
	{
		return 0u ;
	}
	uartBase = pUart->uartBase ;

	// If the kernel is not started, don't use RX interrupt/buffer
	if (aaIsRunning == 0u  ||  checkDescFlag (pUart->desc, UART_FLAG_NOIT) != 0u)
	{
		return ((uartBase->SR & USART_SR_RXNE) == 0u) ? 0u : 1u ;
	}

	return pUart->rxCount  ;
}

//-----------------------------------------------------------------------------
//	Get one char from UART. Blocking call if RX buffer is empty.

char uartGetChar (uartHandle_t hUsart)
{
	uartDesc_t	* pUart = (uartDesc_t *) hUsart ;
	char		cc ;

	if (pUart == NULL)
	{
		return (char) 0 ;
	}

	// If the kernel is not started, don't use RX interrupt/buffer
	if (aaIsRunning == 0u  ||  checkDescFlag (pUart->desc, UART_FLAG_NOIT) != 0u)
	{
		volatile USART_TypeDef	* uartBase = pUart->uartBase ;	// UARTx

		while ((uartBase->SR & USART_SR_RXNE) == 0u)
		{
		}
		return (char) uartBase->DR  ;
	}


	while (1)
	{
		aaCriticalEnter () ;
		if (pUart->rxCount == 0u)
		{
			// Buffer is empty. aaIoWait() will release the critical section
			(void) aaIoWait (& pUart->rxList, 0u, 0u) ;	// Not ordered, no timeout
		}
		else
		{
			break ;	// Not empty. Note: exit while inside critical section
		}
	}

	cc = (char) pUart->pRxBuffer [pUart->iRxRead++] ;
	if (pUart->iRxRead == pUart->rxSize)
	{
		pUart->iRxRead = 0u ;
	}
	pUart->rxCount-- ;
	aaCriticalExit () ;

	return cc ;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//	USART interrupt handlers

void UART_IRQHandler	(uartDesc_t * pUart) ;

void UART_IRQHandler	(uartDesc_t * pUart)
{
	volatile USART_TypeDef	* uartBase = pUart->uartBase ;	// UARTx
	aaTcb_t					* pTcb ;
	uint16_t				cc ;

	aaIntEnter () ;

	// -------------- RXNE handler
	if ((uartBase->CR1 & USART_CR1_RXNEIE) != 0u  &&  (uartBase->SR & USART_SR_RXNE) != 0u)
	{
		cc = uartBase->DR & 0x0FFu ;
		if (pUart->rxCount < pUart->rxSize)
		{
			pUart->pRxBuffer [pUart->iRxWrite++] = (uint8_t) cc ;
			if (pUart->iRxWrite == pUart->rxSize)
			{
				pUart->iRxWrite = 0u ;
			}
			pUart->rxCount++ ;
		}

		aaCriticalEnter () ;
		if (aaIsIoWaitingTask (& pUart->rxList) != 0u)
		{
			// A thread is waiting for RX char, awake it
			pTcb = aaIoResume (& pUart->rxList) ;
			(void) pTcb ;
		}
		aaCriticalExit () ;
	}

	// -------------- TXE handler
	if ((uartBase->CR1 & USART_CR1_TXEIE) != 0u  &&  (uartBase->SR & USART_SR_TXE) != 0u)
	{
		if (pUart->txCount != 0u)
		{
			// Character available
			uartBase->DR = pUart->pTxBuffer [pUart->iTxRead++] ;
			if (pUart->iTxRead == pUart->txSize)
			{
				pUart->iTxRead = 0u ;
			}
			pUart->txCount-- ;

			aaCriticalEnter () ;
			if (aaIsIoWaitingTask (& pUart->txList) != 0u)
			{
				// A thread is waiting for TX space, awake it
				pTcb = aaIoResume (& pUart->txList) ;
				(void) pTcb ;
			}
			aaCriticalExit () ;
		}
		else
		{
			// Nothing to send, disable TX interrupt
			uartBase->CR1 &= ~USART_CR1_TXEIE ;
		}
	}

	// ------------------------------------------------------------
	// Other USART interrupts handler can go here ...

	aaIntExit () ;
}

//-----------------------------------------------------------------------------

#if (WITH_UART1 == 1)
	void USART1_IRQHandler	(void) ;
	void USART1_IRQHandler	(void)
	{
		UART_IRQHandler (& uart1) ;
	}
#endif

//-----------------------------------------------------------------------------

#if (WITH_UART2 == 1)
	void USART2_IRQHandler	(void) ;
	void USART2_IRQHandler	(void)
	{
		UART_IRQHandler (& uart2) ;
	}
#endif

//-----------------------------------------------------------------------------

#if (WITH_UART3 == 1)
	void USART3_IRQHandler	(void) ;
	void USART3_IRQHandler	(void)
	{
		UART_IRQHandler (& uart3) ;
	}
#endif

//-----------------------------------------------------------------------------

#if (WITH_UART4 == 1)
	void UART4_IRQHandler	(void) ;
	void UART4_IRQHandler	(void)
	{
		UART_IRQHandler (& uart4) ;
	}
#endif

//-----------------------------------------------------------------------------

#if (WITH_UART5 == 1)
	void USART5_IRQHandler	(void) ;
	void USART5_IRQHandler	(void)
	{
		UART_IRQHandler (& uart5) ;
	}
#endif

//-----------------------------------------------------------------------------

#if (WITH_UART6 == 1)
	void USART6_IRQHandler	(void) ;
	void USART6_IRQHandler	(void)
	{
		UART_IRQHandler (& uart6) ;
	}
#endif

//-----------------------------------------------------------------------------

#if (WITH_UART7 == 1  &&  defined UART7)
	void UART7_IRQHandler	(void) ;
	void UART7_IRQHandler	(void)
	{
		UART_IRQHandler (& uart7) ;
	}
#endif

//-----------------------------------------------------------------------------

#if (WITH_UART8 == 1  &&  defined UART8)
	void UART8_IRQHandler	(void) ;
	void UART8_IRQHandler	(void)
	{
		UART_IRQHandler (& uart8) ;
	}
#endif

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//	uart parameter is the UART number from 1 to 8

aaError_t	uartInit (uint32_t	uart, uint32_t baudrate, uint32_t flags, uartHandle_t * pHandle)
{
	uartDesc_t	* pUart ;
	IRQn_Type	irqNum = (IRQn_Type) 0 ;	// To avoid warning when no UART is selected

	pUart = uartSelect (uart) ;
	if (pUart == NULL)
	{
		return AA_EFAIL ;
	}
	aaIoDriverDescInit (& pUart->txList) ;
	aaIoDriverDescInit (& pUart->rxList) ;

	// Add flags
	pUart->desc = (pUart->desc & ~UART_FLAG_MASK) | (flags & UART_FLAG_MASK) ;

#if (WITH_UART1 == 1)
{
	if (uart == 1)
	{
		irqNum = USART1_IRQn ;

		// Enable clock for USART
		LL_APB2_GRP1_EnableClock	(LL_APB2_GRP1_PERIPH_USART1) ;
		LL_APB2_GRP1_ForceReset		(LL_APB2_GRP1_PERIPH_USART1) ;
		LL_APB2_GRP1_ReleaseReset	(LL_APB2_GRP1_PERIPH_USART1) ;
	}
}
#endif

#if (WITH_UART2 == 1)
{
	if (uart == 2)
	{
		irqNum = USART2_IRQn ;

		// Enable clock for USART
		LL_APB1_GRP1_EnableClock	(LL_APB1_GRP1_PERIPH_USART2) ;
		LL_APB1_GRP1_ForceReset		(LL_APB1_GRP1_PERIPH_USART2) ;
		LL_APB1_GRP1_ReleaseReset	(LL_APB1_GRP1_PERIPH_USART2) ;
	}
}
#endif

#if (WITH_UART3 == 1)
{
	if (uart == 3)
	{
		irqNum = USART3_IRQn ;

		// Enable clock for USART
		LL_APB1_GRP1_EnableClock	(LL_APB1_GRP1_PERIPH_USART3) ;
		LL_APB1_GRP1_ForceReset		(LL_APB1_GRP1_PERIPH_USART3) ;
		LL_APB1_GRP1_ReleaseReset	(LL_APB1_GRP1_PERIPH_USART3) ;
	}
}
#endif

#if (WITH_UART4 == 1)
{
	if (uart == 4)
	{
		irqNum = UART4_IRQn ;

		// Enable clock for USART
		LL_APB1_GRP1_EnableClock	(LL_APB1_GRP1_PERIPH_UART4) ;
		LL_APB1_GRP1_ForceReset		(LL_APB1_GRP1_PERIPH_UART4) ;
		LL_APB1_GRP1_ReleaseReset	(LL_APB1_GRP1_PERIPH_UART4) ;
	}
}
#endif

#if (WITH_UART5 == 1)
{
	if (uart == 5)
	{
		irqNum = UART5_IRQn ;

		// Enable clock for USART
		LL_APB1_GRP1_EnableClock	(LL_APB1_GRP1_PERIPH_UART5) ;
		LL_APB1_GRP1_ForceReset		(LL_APB1_GRP1_PERIPH_UART5) ;
		LL_APB1_GRP1_ReleaseReset	(LL_APB1_GRP1_PERIPH_UART5) ;
	}
}
#endif

#if (WITH_UART6 == 1)
{
	if (uart == 6)
	{
		irqNum = USART6_IRQn ;

		// Enable clock for USART
		LL_APB2_GRP1_EnableClock	(LL_APB2_GRP1_PERIPH_USART6) ;
		LL_APB2_GRP1_ForceReset		(LL_APB2_GRP1_PERIPH_USART6) ;
		LL_APB2_GRP1_ReleaseReset	(LL_APB2_GRP1_PERIPH_USART6) ;
	}
}
#endif

#if (WITH_UART7 == 1 && defined UART7)
{
	if (uart == 7)
	{
		irqNum = UART7_IRQn ;

		// Enable clock for USART
		LL_APB1_GRP1_EnableClock	(LL_APB1_GRP1_PERIPH_UART7) ;
		LL_APB1_GRP1_ForceReset		(LL_APB1_GRP1_PERIPH_UART7) ;
		LL_APB1_GRP1_ReleaseReset	(LL_APB1_GRP1_PERIPH_UART7) ;
	}
}
#endif

#if (WITH_UART8 == 1 && defined UART8)
{
	if (uart == 8)
	{
		irqNum = UART8_IRQn ;

		// Enable clock for USART
		LL_APB1_GRP1_EnableClock	(LL_APB1_GRP1_PERIPH_UART8) ;
		LL_APB1_GRP1_ForceReset		(LL_APB1_GRP1_PERIPH_UART8) ;
		LL_APB1_GRP1_ReleaseReset	(LL_APB1_GRP1_PERIPH_UART8) ;
	}
}
#endif

	pUart->iTxRead  = 0u ;
	pUart->iTxWrite = 0u ;
	pUart->txCount  = 0u ;
	pUart->iRxRead  = 0u ;
	pUart->iRxWrite = 0u ;
	pUart->rxCount  = 0u ;

	// Configure TX and RX pins
	{
		gpioPinDesc_t	gpioDesc ;

		gpioDesc.flags	= AA_GPIO_MODE_ALTERNATE | AA_GPIO_PUSH_PULL | AA_GPIO_PULL_UP | AA_GPIO_SPEED_LOW ;
		if (desc2TxPort (pUart->desc) != 15)
		{
			gpioDesc.port	= (uint8_t) ('A' + desc2TxPort (pUart->desc)) ;
			gpioDesc.pin	= desc2TxPin (pUart->desc) ;
			gpioDesc.af		= desc2TxAf (pUart->desc) ;
			gpioConfigurePin (& gpioDesc) ;
		}

		if (desc2RxPort (pUart->desc) != 15)
		{
			gpioDesc.port	= (uint8_t) ('A' + desc2RxPort (pUart->desc)) ;
			gpioDesc.pin	= desc2RxPin (pUart->desc) ;
			gpioDesc.af		= (uint8_t) desc2RxAf (pUart->desc) ;
			gpioConfigurePin (& gpioDesc) ;
		}
	}

	// Initialize and Enable USART
	{
		LL_USART_InitTypeDef	hUart;

		LL_USART_Disable (pUart->uartBase) ;
		LL_USART_StructInit (& hUart) ;
		hUart.BaudRate = baudrate;
		LL_USART_Init (pUart->uartBase, & hUart) ;

		LL_USART_ConfigAsyncMode (pUart->uartBase);
		LL_USART_Enable (pUart->uartBase) ;
	}

	// Configure interrupt priority
	NVIC_SetPriority (irqNum, UART_IRQ_PRIORITY) ;

	if (checkDescFlag (pUart->desc, UART_FLAG_NOIT) == 0u)
	{
		// Enable RX interrupt, don't enable TX interrupt
		pUart->uartBase->CR1 |= USART_CR1_RXNEIE ;
	}

	// Enable USARTx global interrupt
	NVIC_EnableIRQ (irqNum) ;

	* pHandle = pUart ;
	return AA_ENONE ;
}

//-----------------------------------------------------------------------------
//	Warning: it is not checked that waiting lists are empty, or last character transmitted

aaError_t	uartClose			(uartHandle_t hUsart)
{
	uartDesc_t	* pUart = (uartDesc_t *) hUsart ;

	// Disable interrupts
	pUart->uartBase->CR1 &= ~USART_CR1_RXNEIE ;
	pUart->uartBase->CR1 &= ~USART_CR1_TXEIE ;

	// Disable UART
	pUart->uartBase->CR1 &= ~USART_CR1_UE ;

	return AA_ENONE ;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//	To register a UART as standard input/output

void	uartInitStdDev (aaDevIo_t * pDev, uartHandle_t hUsart)
{
	pDev->hDev				= hUsart ;
	pDev->getCharPtr		= uartGetChar ;
	pDev->putCharPtr		= uartPutChar ;
	pDev->checkGetCharPtr	= uartCheckGetChar ;
	pDev->checkPutCharPtr	= uartCheckPutChar ;
}

//-----------------------------------------------------------------------------
/* Very simple example from http://www.micromouseonline.com/2009/12/31/stm32-usart-basics/

void usartSetup (void)
{
  // make sure the relevant pins are appropriately set up.
  RCC_APB2ENR |= RCC_APB2ENR_IOPAEN;              // enable clock for GPIOA
  GPIOA_CRH   |= (0x0BUL  << 4);                  // Tx (PA9) alt. out push-pull
  GPIOA_CRH   |= (0x04UL  << 8);                  // Rx (PA10) in floating
  RCC_APB2ENR |= RCC_APB2ENR_USART1EN;            // enable clock for USART1
  USART1_BRR  = 64000000L/115200L;                // set baudrate
  USART1_CR1 |= (USART1_CR1_RE | USART1_CR1_TE);  // RX, TX enable
  USART1_CR1 |= USART1_CR1_UE;                    // USART enable
}

int SendChar (int ch)
{
  while (!(USART1_SR & USART1_SR_TXE))
  {
  }
  USART1_DR = (ch & 0xFF);
  return (ch);
}

int GetChar (void)
{
  while (!(USART1_SR & USART1_SR_RXNE))
  {
  }
  return ((int)(USART1_DR & 0xFF));
}

*/
//-----------------------------------------------------------------------------
