/*
 * Copyright (c) 2001-2003 Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 *
 * Author: Adam Dunkels <adam@sics.se>
 *
 */

/* lwIP includes. */
#include "lwip/debug.h"
#include "lwip/def.h"
#include "lwip/sys.h"
#include "lwip/mem.h"
#include "lwip/stats.h"

#if !NO_SYS

#include "aa.h"
#include "aakernel.h"

#if defined(LWIP_SOCKET_SET_ERRNO) && defined(LWIP_PROVIDE_ERRNO)
	int errno;
#endif

//-----------------------------------------------------------------------------------
//  Creates an empty mailbox.

err_t sys_mbox_new (sys_mbox_t * mbox, int size)
{
	if (AA_ENONE != aaQueueCreate (mbox, sizeof (void *), (uint32_t) size, NULL, AA_QUEUE_POINTER | AA_QUEUE_FIFO))
	{
		AA_ASSERT (0) ;
	}

	#if SYS_STATS
		++lwip_stats.sys.mbox.used;
		if (lwip_stats.sys.mbox.max < lwip_stats.sys.mbox.used)
		{
			lwip_stats.sys.mbox.max = lwip_stats.sys.mbox.used;
		}
	#endif // SYS_STATS

	if ((* mbox) == AA_INVALID_QUEUE)
	{
		 return ERR_MEM;
	}

	return ERR_OK;
}

//-----------------------------------------------------------------------------------
/*
  Deallocates a mailbox. If there are messages still present in the
  mailbox when the mailbox is deallocated, it is an indication of a
  programming error in lwIP and the developer should be notified.
*/
void sys_mbox_free (sys_mbox_t * mbox)
{
	uint32_t	count ;

	aaQueueGetCount (* mbox, & count) ;
	if (count != 0)
	{
		// Line for breakpoint.  Should never break here!
		#if SYS_STATS
				lwip_stats.sys.mbox.err++;
		#endif // SYS_STATS
		AA_ASSERT (0) ;
	}

	// If there is waiting tasks, they will be flushed
	if (AA_ENONE != aaQueueDelete (* mbox))
	{
		AA_ASSERT (0) ;
	}
	* mbox = AA_INVALID_QUEUE ;

	#if SYS_STATS
		--lwip_stats.sys.mbox.used;
	#endif /* SYS_STATS */
}

//-----------------------------------------------------------------------------------
//   Posts the "msg" to the mailbox.

void sys_mbox_post (sys_mbox_t * mbox, void * data)
{
	while (aaQueueGive (* mbox, data, 0, AA_INFINITE) != AA_ENONE);
}


//-----------------------------------------------------------------------------------
//   Try to post the "msg" to the mailbox.

err_t sys_mbox_trypost (sys_mbox_t * mbox, void * msg)
{
	err_t	result ;

	if (aaQueueGive (* mbox, msg, 0, 0) == AA_ENONE)
	{
		result = ERR_OK;
	}
	else
	{
		// could not post, queue must be full
		result = ERR_MEM;

		#if SYS_STATS
				lwip_stats.sys.mbox.err++;
		#endif // SYS_STATS
	}

   return result;
}

//-----------------------------------------------------------------------------------
//   Try to post the "msg" to the mailbox. New for LWIP 2.1.0

err_t sys_mbox_trypost_fromisr (sys_mbox_t * mbox, void * msg)
{
  return sys_mbox_trypost (mbox, msg) ;
}

//-----------------------------------------------------------------------------------
/*
  Blocks the thread until a message arrives in the mailbox, but does
  not block the thread longer than "timeout" milliseconds (similar to
  the sys_arch_sem_wait() function). The "msg" argument is a result
  parameter that is set by the function (i.e., by doing "*msg =
  ptr"). The "msg" parameter maybe NULL to indicate that the message
  should be dropped.

  The return values are the same as for the sys_arch_sem_wait() function:
  Number of milliseconds spent waiting or SYS_ARCH_TIMEOUT if there was a
  timeout.

  Note that a function with a similar name, sys_mbox_fetch(), is
  implemented by lwIP.
*/
u32_t sys_arch_mbox_fetch (sys_mbox_t *mbox, void **msg, u32_t timeout)
{
	uint32_t	startTime = aaGetTickCount () ;
	uint8_t		* ptr ;

	if (timeout == 0)
	{
		// Wait forever
		aaQueueTake (* mbox, & ptr, 0, AA_INFINITE) ;
		if (msg != NULL)
		{
			* msg = ptr ;
		}
		return (aaGetTickCount () - startTime) ;
	}
	else
	{
		// Not infinite timeout
		if (aaQueueTake (* mbox, & ptr, 0, timeout) == AA_ENONE)
		{
			if (msg != NULL)
			{
				* msg = ptr ;
			}
			return (aaGetTickCount () - startTime) ;
		}
		else
		{
			return SYS_ARCH_TIMEOUT ;
		}
	}
}

//-----------------------------------------------------------------------------------
//	Similar to sys_arch_mbox_fetch, but if message is not ready immediately, we'll
//	return with SYS_MBOX_EMPTY.  On success, 0 is returned.

u32_t sys_arch_mbox_tryfetch (sys_mbox_t *mbox, void **msg)
{
	uint8_t		* ptr ;

	if (aaQueueTake (* mbox, & ptr, 0, 0) == AA_ENONE)
	{
		if (msg != NULL)
		{
			* msg = ptr ;
		}
		return ERR_OK ;
	}
	else
	{
		return SYS_MBOX_EMPTY ;
	}
}

//-----------------------------------------------------------------------------------

int sys_mbox_valid (sys_mbox_t * mbox)
{
	return (* mbox == SYS_MBOX_NULL) ? 0 : 1 ;
}

//-----------------------------------------------------------------------------------
//	Set a mbox invalid so that sys_mbox_valid returns 0

void sys_mbox_set_invalid (sys_mbox_t * mbox)
{                                             
	* mbox = SYS_MBOX_NULL ;
}                                             

//-----------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------
//  Creates a new semaphore. The "count" argument specifies
//  the initial state of the semaphore.

err_t sys_sem_new (sys_sem_t * sem, u8_t count)
{
	if (aaSemCreate (count, sem) != AA_ENONE)
	{
		#if SYS_STATS
			++lwip_stats.sys.sem.err;
		#endif // SYS_STATS
		return ERR_MEM;
	}

	#if SYS_STATS
		++lwip_stats.sys.sem.used;
		if (lwip_stats.sys.sem.max < lwip_stats.sys.sem.used)
		{
			lwip_stats.sys.sem.max = lwip_stats.sys.sem.used;
		}
	#endif /* SYS_STATS */
		
	return ERR_OK;
}

//-----------------------------------------------------------------------------------
/*
  Blocks the thread while waiting for the semaphore to be
  signaled. If the "timeout" argument is non-zero, the thread should
  only be blocked for the specified time (measured in
  milliseconds).

  If the timeout argument is non-zero, the return value is the number of
  milliseconds spent waiting for the semaphore to be signaled. If the
  semaphore wasn't signaled within the specified time, the return value is
  SYS_ARCH_TIMEOUT. If the thread didn't have to wait for the semaphore
  (i.e., it was already signaled), the function may return zero.

  Notice that lwIP implements a function with a similar name,
  sys_sem_wait(), that uses the sys_arch_sem_wait() function.
*/
u32_t sys_arch_sem_wait (sys_sem_t * sem, u32_t timeout)
{
	uint32_t	startTime = aaGetTickCount () ;

	if (timeout == 0)
	{
		// Infinite timeout
		aaSemTake (* sem, AA_INFINITE) ;
	}
	else
	{
		// Non infinite timeout
		if (aaSemTake (* sem, timeout) != AA_ENONE)
		{
			return SYS_ARCH_TIMEOUT ;
		}
	}
	return (aaGetTickCount () - startTime);
}

//-----------------------------------------------------------------------------------
// Signals a semaphore

void sys_sem_signal (sys_sem_t * sem)
{
	aaSemGive (* sem);
}

//-----------------------------------------------------------------------------------
// Deallocates a semaphore

void sys_sem_free (sys_sem_t * sem)
{
	#if SYS_STATS
	  --lwip_stats.sys.sem.used;
	#endif /* SYS_STATS */
  
	aaSemDelete (* sem) ;
	* sem = SYS_SEM_NULL;
}

//-----------------------------------------------------------------------------------
//	Return 1 if the sem ID is valid

int sys_sem_valid (sys_sem_t * sem)
{
  return (* sem == SYS_SEM_NULL) ? 0 : 1 ;
}

//-----------------------------------------------------------------------------------
// Set a sem invalid so that sys_sem_valid returns 0

void sys_sem_set_invalid (sys_sem_t * sem)
{                                                                               
	* sem = SYS_SEM_NULL;
} 

//-----------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------
                                      /* Mutexes*/
//-----------------------------------------------------------------------------------

// For old ports that don't have mutexes LWIP_COMPAT_MUTEX is 1

#if LWIP_COMPAT_MUTEX == 0

//-----------------------------------------------------------------------------------
//	Create a new mutex

err_t sys_mutex_new (sys_mutex_t * mutex)
{
	if (aaMutexCreate (mutex) != AA_ENONE)
	{
		#if SYS_STATS
			++lwip_stats.sys.mutex.err;
		#endif // SYS_STATS
		return ERR_MEM;
	}

	#if SYS_STATS
		++lwip_stats.sys.mutex.used;
		if (lwip_stats.sys.mutex.max < lwip_stats.sys.mutex.used)
		{
			lwip_stats.sys.mutex.max = lwip_stats.sys.mutex.used;
	 	}
	#endif // SYS_STATS
	return ERR_OK;
}

//-----------------------------------------------------------------------------------
//	Deallocate a mutex

void sys_mutex_free (sys_mutex_t * mutex)
{
	#if SYS_STATS
		--lwip_stats.sys.mutex.used;
	#endif /* SYS_STATS */

	aaMutexDelete (* mutex) ;
	* mutex = AA_INVALID_MUTEX ;
}

//-----------------------------------------------------------------------------------
//	Check if a mutex is valid/allocated: return 1 for valid, 0 for invalid

int sys_mutex_valid (sys_mutex_t * mutex)
{
	return (* mutex == AA_INVALID_MUTEX) ? 0 : 1 ;
}

//-----------------------------------------------------------------------------------
//	Set a mutex invalid so that sys_mutex_valid returns 0

void sys_mutex_set_invalid (sys_mutex_t * mutex)
{
	  * mutex = AA_INVALID_MUTEX ;
}

//-----------------------------------------------------------------------------------
//	Lock a mutex

void sys_mutex_lock (sys_mutex_t * mutex)
{
	aaMutexTake (* mutex, AA_INFINITE) ;
}

//-----------------------------------------------------------------------------------
//	Unlock a mutex

void sys_mutex_unlock(sys_mutex_t *mutex)
{
	aaMutexGive (* mutex) ;
}

#endif /*LWIP_COMPAT_MUTEX*/

//-----------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------
/*
  Starts a new thread with priority "prio" that will begin its execution in the
  function "thread()". The "arg" argument will be passed as an argument to the
  thread() function. The id of the new thread is returned. Both the id and
  the priority are system dependent.
*/
sys_thread_t sys_thread_new (const char * name, lwip_thread_fn thread , void * arg, int stacksize, int prio)
{
	aaTaskId_t	taskId ;
	uint32_t	flags = AA_FLAG_STACKCHECK ;

// AA task function and LWIP task function doesn't have the same prototype, so ignore warning
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wcast-function-type"
	if (AA_ENONE != aaTaskCreate ((uint32_t) prio, name, (aaTaskFunction_t) thread, (uintptr_t) arg, NULL, (uint32_t) stacksize, flags, & taskId))
	{
		AA_ASSERT (0) ;
	}
#pragma GCC diagnostic pop
	return taskId ;
}

//-----------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------
//osMutexId lwip_sys_mutex;
//osMutexDef(lwip_sys_mutex);

// Initialize sys arch

void sys_init(void)
{
//	lwip_sys_mutex = osMutexCreate(osMutex(lwip_sys_mutex));
}

//-----------------------------------------------------------------------------------
/*
  This optional function does a "fast" critical region protection and returns
  the previous protection level. This function is only called during very short
  critical regions. An embedded system which supports ISR-based drivers might
  want to implement this function by disabling interrupts. Task-based systems
  might want to implement this by using a mutex or disabling tasking. This
  function should support recursive calls from the same task or interrupt. In
  other words, sys_arch_protect() could be called while already protected. In
  that case the return value indicates that it is already protected.

  sys_arch_protect() is only required if your port is supporting an operating
  system.

  Note: This function is based on FreeRTOS API, because no equivalent CMSIS-RTOS
        API is available
*/

// ??? Mutex is not good here: a mutex can't be called from interrupt. Use aaCriticalEnter() / aaCriticalExit() ;
sys_prot_t sys_arch_protect(void)
{
	aaCriticalEnter () ;
	return (sys_prot_t)1;
}

//-----------------------------------------------------------------------------------
/*
  This optional function does a "fast" set of critical region protection to the
  value specified by pval. See the documentation for sys_arch_protect() for
  more information. This function is only required if your port is supporting
  an operating system.
*/
void sys_arch_unprotect(sys_prot_t pval)
{
	( void ) pval;
	aaCriticalExit ()  ;
}

//-----------------------------------------------------------------------------------
#endif /* !NO_SYS */
