/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	testi2c.c	I2C test

	When		Who	What
	26/01/20	ac	Creation

	BEWARE: to get good aaPrinf traces the console TX buffer must be large (>= 256 bytes)

----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aalogmes.h"
#include 	"aaprintf.h"

#include	"test.h"

#include 	"i2cbasic.h"

//----------------------------------------------------------------------
//	 For debug purpose

#define	DBG_I2C					1		// 1 to display data on SWO

#if (DBG_I2C != 0)
	#include	"itm.h"
	#pragma GCC diagnostic push
	#pragma GCC diagnostic ignored "-Wmultichar"
#endif

//----------------------------------------------------------------------
/*	Cabling for the test:

	MiniF411 (BlackPill)

	SDA ---- 5 -------- PB7 ---------- PB9  ------------ resistor ---- |3.3V
	SCL ---- 6 -------- PB6 ---------- PB10 ------------ resistor ---- |
             24Cxx      I2C1(Slave)    I2C2(Master)
             EEPROM

	NUCLEO-STM32F429ZI

	SDA ---- 5 -------- PB9 CN7-2----- PB11 CN7-34 ----- resistor ---- |3.3V
	SCL ---- 6 -------- PB8 CN7-4----- PB10 CN7-32------ resistor ---- |
             24Cxx      I2C1(Slave)    I2C2(Master)
             EEPROM

	Pull up resistors: 2.2k to 3.3k Ohms
*/

//----------------------------------------------------------------------
//	I2C Test

const	char * i2cErrorText [] =
{
		"I2C_ENONE",		// No error, idle state
		"I2C_EFAIL",		// Some error
		"I2C_ESTATE",		// Not allowed for this I2C state

		"I2C_EOK",			// Ended without error
		"I2C_EBUSY",		// Interface is busy
		"I2C_EAF",			// Acknowledge fail
		"I2C_EARLO",		// Arbitration lost
		"I2C_ETMO",			// Timeout
		"I2C_EBERR",		// Bus error
		"I2C_EOVR",			// I2C Overflow
		"I2C_ESIZE",		// Slave / master buffer size non-coherent. On slave side:
							// Master stopped receive before slave buffer size
							// Master sent more data than the slave buffer size

		"I2C_ESLAVERX",		// Slave addressed as receiver
		"I2C_ESLAVETX",		// Slave addressed as transmitter
} ;

//----------------------------------------------------------------------

#define		I2CSPEED		88000		// Default I2C baud rate
#define		TASK_SIGNAL		0x8000		// The I2C library use this signal to signal the task
#define		MAX_MSG			10u			// Max size of messages (buffer size)

static	void	ee24cTest (void) ;

//----------------------------------------------------------------------

#define		MST_DESCID		2	// I2C description Id for master I2C
#define		SLV_DESCID		1u	// I2C description Id for slave I2C

#define		SLV_ADDRS		0x50u	// I2c Slave address (left shifted)
#define		SLV_STACKSIZE	128u	// Slave management stack size

static	uint8_t		slvBufferRx [MAX_MSG] ;
static	uint8_t		slvBufferTx [MAX_MSG] = { 0x88u, 0xC8u, 0xE8u, 0xF8u, 0x88u, 0xC8u, 0xE8u, 0xF8u, 0x88u, 0xC8u } ;

static	uint32_t	slvBlock		= 1u ;		// 1 or 2 blocks of slvMsgSize bytes
static	uint32_t	slvMsgSize		= 2u ; 		// Slave message size, to send or receive

static	uint32_t	masterBlock		= 1u ; 		// 1 or 2 blocks of msgSize bytes
static	uint32_t	masterRestart	= 0u ;		// With or without restart

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// I2C Slave
// The task to manage slave transfer
// Use function blocking mode

static	void	slaveTask (uintptr_t arg)
{
	const 		i2cDesc_t		* pI2cDesc ;
				i2cHandle_t		hdl ;
	volatile 	i2cStatus_t		res ;		// To avoid optimize out
				uint32_t		size ;

	(void) arg ;

	aaPrintf ("I2C slave task created\n") ;

	pI2cDesc =	i2cGetDesc	(SLV_DESCID) ;

	// ---------------------------- Configure I2C
	res = i2cConfigure (pI2cDesc, I2CSPEED, TASK_SIGNAL, & hdl) ;
	if (res != I2C_ENONE)
	{
		aaPrintf ("i2cConfigure error %d %s\n", res, i2cErrorText[res]) ;
		AA_ASSERT (0) ;
		return ;
	}

	i2cSlaveInit (hdl, SLV_ADDRS) ;
	aaPrintf ("I2C slave initialized\n") ;

	while (1)
	{
		// ---------------------------- Set slave in addressable state
		res = i2cSlaveListen (hdl, 0u) ;	// Flag 0 => blocking mode

		// The slave musts always be ready to receive when started

		if (res == I2C_ESLAVETX)
		{
			// The master requests the slave to send data

			if (slvBlock == 2)
			{
				// For the test to succeed, the size of the master message must be twice as large as the slave message
				(void) i2cSlaveXfer (hdl, slvBufferTx, slvMsgSize, I2C_NEXT) ;
				res = i2cResult (hdl, & size) ;
				aaPrintf ("  %s Slave TX, size %d / %d\n", i2cErrorText[res], size, slvMsgSize) ;
				aaPrintf ("  Data: %02X %02X %02X %02X\n", slvBufferTx[0], slvBufferTx[1], slvBufferTx[2], slvBufferTx[3]) ;
			}

			(void) i2cSlaveXfer (hdl, slvBufferTx, slvMsgSize, I2C_LAST) ;
			res = i2cResult (hdl, & size) ;
			aaPrintf ("  %s Slave TX, size %d / %d\n", i2cErrorText[res], size, slvMsgSize) ;
			aaPrintf ("  Data: %02X %02X %02X %02X\n", slvBufferTx[0], slvBufferTx[1], slvBufferTx[2], slvBufferTx[3]) ;
		}
		else if (res == I2C_ESLAVERX)
		{
			// The master requests the slave to receive data
			// When the master probe the slave (addressed as Tx with 0 data),
			// the slave will get I2C_ESLAVERX then 0 data

			if (slvBlock == 2)
			{
				// For the test to succeed, the size of the master message must be twice as large as the slave message
				aaMEMSET (slvBufferRx, 0, sizeof (slvBufferRx)) ;
				(void) i2cSlaveXfer (hdl, slvBufferRx, slvMsgSize, I2C_NEXT) ;
				res = i2cResult (hdl, & size) ;
				aaPrintf ("  %s Slave RX, size %d / %d\n", i2cErrorText[res], size, slvMsgSize) ;
				aaPrintf ("  Data: %02X %02X %02X %02X\n", slvBufferRx[0], slvBufferRx[1], slvBufferRx[2], slvBufferRx[3]) ;
			}

			aaMEMSET (slvBufferRx, 0, sizeof (slvBufferRx)) ;
			(void) i2cSlaveXfer (hdl, slvBufferRx, slvMsgSize, I2C_LAST) ;
			res = i2cResult (hdl, & size) ;
			aaPrintf ("  %s Slave RX, size %d / %d\n", i2cErrorText[res], size, slvMsgSize) ;
			aaPrintf ("  Data: %02X %02X %02X %02X\n", slvBufferRx[0], slvBufferRx[1], slvBufferRx[2], slvBufferRx[3]) ;
		}
		else
		{
			aaPrintf ("  Invalid result: %d %s, size %d\n", res, i2cErrorText[res], size) ;
		}
	}
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// i2cMasterWrite() test
// Write 1 blocs of bytes to the slave

static	void	i2cTestMasterWrite (uint32_t speed, uint32_t msgSize)
{
	const 		i2cDesc_t		* pI2cDesc ;
				i2cHandle_t		hdl ;
	volatile 	i2cStatus_t		res ;
				uint32_t		size ;
				uint8_t			data [MAX_MSG] ;

	pI2cDesc =	i2cGetDesc	(MST_DESCID) ;

	if (speed == 0u)
	{
		speed = I2CSPEED ;
	}

	for (size = 0u ; size < msgSize ; size++)
	{
		data [size] = 0xAA + 0x11 * size ;
	}


	// ---------------------------- Configure I2C
	res = i2cConfigure (pI2cDesc, speed, TASK_SIGNAL, & hdl) ;
	if (res != I2C_ENONE)
	{
		aaPrintf ("i2cConfigure error %d %s\n", res, i2cErrorText[res]) ;
		AA_ASSERT (0) ;
		return ;
	}

	// ---------------------------- Send message
	res = i2cMasterWrite (hdl, SLV_ADDRS, data, msgSize) ;
	res = i2cResult (hdl, & size) ;
	if (res == I2C_EOK)
	{
		aaPrintf ("%s Transmitted %d bytes\n", i2cErrorText[res], size) ;
		aaPuts ("Data: ") ;
		for (size = 0u ; size < msgSize ; size++)
		{
			aaPrintf ("%02X ", data[size]) ;
		}
		aaPuts ("\n") ;
	}
	else
	{
		aaPrintf ("%s Error %s\n", i2cErrorText[res]) ;
		i2cDeactivate (hdl) ;
	}
}

//----------------------------------------------------------------------
// i2cMasterRead() test
// Read 1 bloc of bytes from the slave

static	void	i2cTestMasterRead (uint32_t speed, uint32_t msgSize)
{
	const 		i2cDesc_t		* pI2cDesc ;
				i2cHandle_t		hdl ;
	volatile 	i2cStatus_t		res ;
				uint8_t			data [MAX_MSG] = { 0u } ;
				uint32_t		size ;

	pI2cDesc =	i2cGetDesc	(MST_DESCID) ;

	if (speed == 0u)
	{
		speed = I2CSPEED ;
	}

	// ---------------------------- Configure I2C
	res = i2cConfigure (pI2cDesc, speed, TASK_SIGNAL, & hdl) ;
	if (res != I2C_ENONE)
	{
		aaPrintf ("i2cConfigure error %d %s\n", res, i2cErrorText[res]) ;
		AA_ASSERT (0) ;
		return ;
	}

	// ---------------------------- Receive message
	res = i2cMasterRead (hdl, SLV_ADDRS, data, msgSize) ;
	res = i2cResult (hdl, & size) ;
	if (res == I2C_EOK)
	{
		aaPrintf ("%s Received %d bytes\n", i2cErrorText[res], size) ;
		aaPuts ("Data: ") ;
		for (size = 0u ; size < msgSize ; size++)
		{
			aaPrintf ("%02X ", data[size]) ;
		}
		aaPuts ("\n") ;
	}
	else
	{
		aaPrintf ("%s Error\n", i2cErrorText[res]) ;
		i2cDeactivate (hdl) ;
	}
}

//----------------------------------------------------------------------
// Master read blocs of msgSize bytes.
// If masterBlock is 2 :         startRX - xfer - xfer - stop
// Else if masterRestart is 1 :  startRX - xfer - startRX - xfer - stop
// Else :                        startRX - xfer - stop

static	void	i2cMasterRxSlaveTx (uint32_t msgSize)
{
	const 		i2cDesc_t		* pI2cDesc ;
				i2cHandle_t		hdl ;
	volatile 	i2cStatus_t		res ;
				uint8_t			data [MAX_MSG] = { 0u } ;
				uint32_t		sentSize ;

	pI2cDesc =	i2cGetDesc	(MST_DESCID) ;

	// ---------------------------- Configure I2C
	res = i2cConfigure (pI2cDesc, I2CSPEED, TASK_SIGNAL, & hdl) ;
	if (res != I2C_ENONE)
	{
		aaPrintf ("i2cConfigure error %d %s\n", res, i2cErrorText[res]) ;
		AA_ASSERT (0) ;
		return ;
	}

	// ---------------------------- Send Start and slave address
	res = i2cMasterStart (hdl, SLV_ADDRS, I2C_RX) ;
	if (res == I2C_EOK)
	{
		// Slave responding
		aaPrintf ("%s Slave ACK\n", i2cErrorText[res]) ;
	}
	else
	{
		// Slave not responding or error
		aaPrintf ("%s Error, deactivate I2C\n", i2cErrorText[res]) ;
		i2cDeactivate (hdl) ;
		return ;
	}

	if (masterBlock == 2)
	{
		//	Receive 1st block
		// ---------------------------- Receive data bytes
		aaMEMSET (data, 0, sizeof (data)) ;
		(void) i2cMasterXfer (hdl, data, msgSize, I2C_NEXT) ;
		res = i2cResult (hdl, & sentSize) ;
		aaPrintf ("%s, Master Received %d / %d bytes\n", i2cErrorText[res], sentSize, msgSize) ;
		aaPrintf ("Data: %02X %02X %02X %02X\n", data[0], data[1], data[2], data[3]) ;
	}
	else
	{
		if (masterRestart == 1u)
		{
			//	Receive 1st block and ReStart
			// ---------------------------- Receive data bytes
			aaMEMSET (data, 0, sizeof (data)) ;
			(void) i2cMasterXfer (hdl, data, msgSize, I2C_LAST) ;
			res = i2cResult (hdl, & sentSize) ;
			aaPrintf ("%s, Master Received %d / %d bytes\n", i2cErrorText[res], sentSize, msgSize) ;
			aaPrintf ("Data: %02X %02X %02X %02X\n", data[0], data[1], data[2], data[3]) ;

			// ---------------------------- Send Start and slave address
			res = i2cMasterStart (hdl, SLV_ADDRS, I2C_RX) ;
			aaPrintf ("%s Master ReSTART\n", i2cErrorText[res]) ;

			if (res == I2C_EOK)
			{
				// Slave responding
				aaPrintf ("%s Slave ACK\n", i2cErrorText[res]) ;
			}
			else
			{
				// Slave not responding or error
				aaPrintf ("%s Error, deactivate I2C\n", i2cErrorText[res]) ;
				i2cDeactivate (hdl) ;
				return ;
			}
		}
	}

	// Receive last block
	// ---------------------------- Receive data bytes
	aaMEMSET (data, 0, sizeof (data)) ;
	(void) i2cMasterXfer (hdl, data, msgSize, I2C_LAST) ;
	res = i2cResult (hdl, & sentSize) ;

	// ---------------------------- Send Stop
	(void) i2cMasterStop (hdl) ;
	aaPrintf ("%s, Master Received %d / %d bytes\n", i2cErrorText[res], sentSize, msgSize) ;
	aaPrintf ("Data: %02X %02X %02X %02X\n", data[0], data[1], data[2], data[3]) ;
}

//----------------------------------------------------------------------
// Master write blocs of msgSize bytes

// If masterBlock is 2 :         startTX - xfer - xfer - stop
// Else if masterRestart is 1 :  startTX - xfer - startTX - xfer - stop
// Else (masterBlock is 1):      startTX - xfer - stop

static	void	i2cMasterTxSlaveRx (uint32_t msgSize)
{
	const 		i2cDesc_t		* pI2cDesc ;
				i2cHandle_t		hdl ;
	volatile 	i2cStatus_t		res ;
				uint8_t			data [MAX_MSG] = { 'M', 'a', 's', 't' } ;
				uint32_t		sentSize ;

	pI2cDesc =	i2cGetDesc	(MST_DESCID) ;

	// ---------------------------- Configure I2C
	res = i2cConfigure (pI2cDesc, I2CSPEED, TASK_SIGNAL, & hdl) ;
	if (res != I2C_ENONE)
	{
		aaPrintf ("i2cConfigure error %d %s\n", res, i2cErrorText[res]) ;
		AA_ASSERT (0) ;
		return ;
	}

	// ---------------------------- Send Start and slave address
	res = i2cMasterStart (hdl, SLV_ADDRS, I2C_TX) ;

	if (res == I2C_EOK)
	{
		// Slave responding
		aaPrintf ("%s Slave ACK\n", i2cErrorText[res]) ;
	}
	else
	{
		// Slave not responding or error
		aaPrintf ("%s Error, deactivate I2C\n", i2cErrorText[res]) ;
		i2cDeactivate (hdl) ;
		return ;
	}

	if (masterBlock == 2)
	{
		(void) i2cMasterXfer (hdl, data, msgSize, I2C_NEXT) ;
		res = i2cResult (hdl, & sentSize) ;
		aaPrintf ("%s, Master Sent %d / %d bytes\n", i2cErrorText[res], sentSize, msgSize) ;
		if (res == I2C_EAF)
		{
			i2cMasterStop (hdl) ;
			return ;
		}
	}
	else
	{
		if (masterRestart == 1u)
		{
			//	Transmit 1st block and ReStart
			// ---------------------------- Transmit data bytes
			(void) i2cMasterXfer (hdl, data, msgSize, I2C_LAST) ;
			res = i2cResult (hdl, & sentSize) ;
			aaPrintf ("%s, Master Sent %d / %d bytes\n", i2cErrorText[res], sentSize, msgSize) ;

			// ---------------------------- Send ReStart and slave address
			res = i2cMasterStart (hdl, SLV_ADDRS, I2C_TX) ;
			aaPrintf ("%s Master ReSTART\n", i2cErrorText[res]) ;

			if (res == I2C_EOK)
			{
				// Slave responding
				aaPrintf ("%s Slave ACK\n", i2cErrorText[res]) ;
			}
			else
			{
				// Slave not responding or error
				aaPrintf ("%s Error, deactivate I2C\n", i2cErrorText[res]) ;
				i2cDeactivate (hdl) ;
				return ;
			}
		}
	}

	// ---------------------------- Transmit data bytes
	(void) i2cMasterXfer (hdl, data, msgSize, I2C_LAST) ;
	res = i2cResult (hdl, & sentSize) ;

	// ---------------------------- Send Stop
if (DBG_I2C != 0) swoSend16 ('w-', 0) ;
	(void) i2cMasterStop (hdl) ;
	aaPrintf ("%s, Master Sent %d / %d bytes\n", i2cErrorText[res], sentSize, msgSize) ;
	aaPrintf ("Data: %02X %02X %02X %02X\n", data[0], data[1], data[2], data[3]) ;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// test of i2cMasterProbeDevice()

static	void	i2cTestProbe (uint32_t bExist)
{
	const i2cDesc_t			* pI2cDesc ;
	i2cHandle_t				hdl ;
	volatile i2cStatus_t	res ;
	uint32_t				addrs ;

	pI2cDesc =	i2cGetDesc	(MST_DESCID) ;

	// Configure I2C
	res = i2cConfigure (pI2cDesc, I2CSPEED, TASK_SIGNAL, & hdl) ;
	if (res != I2C_ENONE)
	{
		aaPrintf ("i2cConfigure error %d %s\n", res, i2cErrorText[res]) ;
		AA_ASSERT (0) ;
		return ;
	}

	addrs = (bExist == 1u) ? SLV_ADDRS : SLV_ADDRS+2u ;	// On board slave
//	addrs = (bExist == 1u) ? PFC_ADDR : PFC_ADDR+2u ;	// PCF8574A slave
	res = i2cMasterProbeDevice (hdl, addrs) ;
	aaPrintf ("i2cMasterCheckDevice %02X : %d %s\n", addrs, res, i2cErrorText[res]) ;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	I2C interactive tests
//	Use one master I2C and one slave I2C on the same board
//	The master have the descriptor ID MST_DESCID, and the slave have the descriptor ID SLV_DESCID

void	i2cTest ()
{
	char		cc ;
	aaTaskId_t	taskId = AA_INVALID_TASK ;
	uint32_t	msgSize = 4u ;

	aaPrintf ("h for test menu\n") ;
	aaPrintf ("t to start the slave task\n") ;
	while (1)
	{
		cc = aaGetChar () ;
		aaPutChar (cc) ;
		aaPutChar ('\n') ;

		if (cc == 'q')
		{
			break ;	// End of test
		}

		if (cc == ' ')		// Line space on message log terminal
		{
			aaLogMes ("\n", 0, 0, 0, 0, 0) ;
		}

		if (cc == 'h')
		{
			aaPrintf ("?    Print test configuration\n") ;
			aaPrintf ("r    Master read  PCF8574A\n") ;
			aaPrintf ("w    Master Write PCF8574A\n") ;
			aaPrintf ("D    Device Probe OK\n") ;
			aaPrintf ("d    Device Probe fail\n") ;
			aaPrintf ("p    (put) Master TX\n") ;
			aaPrintf ("g    (get) Master RX\n") ;

			aaPrintf ("t    Initialize slave task\n") ;
			aaPrintf ("+-   Inc / Dec master message size\n") ;
			aaPrintf ("s v  Slave message size (1..9)\n") ;
			aaPrintf ("B    Toggle master message block (1..2)\n") ;
			aaPrintf ("b    Toggle slave  message block (1..2)\n") ;
			aaPrintf ("e    Toggle master restart on/off\n") ;

			aaPrintf ("m    EEPROM 24Cxx test\n") ;
			aaPrintf ("q    Quit test\n") ;
		}

		if (cc == 'r')
		{
if (DBG_I2C != 0) swoSend16 ('\r\n', 0) ;
			i2cTestMasterRead (0u, msgSize) ;	// Master read
		}

		if (cc == 'w')
		{
if (DBG_I2C != 0) swoSend16 ('\r\n', 0) ;
			i2cTestMasterWrite (0u, msgSize) ;	// Master Write
		}

		if (cc == 'd')
		{
			i2cTestProbe (0u) ;		// Non existing device
		}

		if (cc == 'D')
		{
			i2cTestProbe (1u) ;		// Existing device
		}

		if (cc == 't')				// Start I2C slave task
		{
			if (taskId == AA_INVALID_TASK)
			{
				uint8_t prio ;

				aaTaskGetBasePriority (AA_SELFTASKID, & prio) ;
				aaTaskCreate (	prio - 1u,		// Task priority
								"tI2cSlv",		// Task name
								slaveTask,		// Entry point,
								0u,				// Entry point parameter
								NULL,			// Stack pointer
								SLV_STACKSIZE,	// Stack size
								0u,				// Flags
								& taskId) ;		// Created task id
			}
			else
			{
				aaPrintf ("Already created\n") ;
			}
		}

		if (cc == '+')
		{
			if (msgSize < MAX_MSG)
			{
				msgSize++ ;
			}
			aaPrintf ("Master message size: %d\n", msgSize) ;
		}
		if (cc == '-')
		{
			if (msgSize > 1u)
			{
				msgSize-- ;
			}
			aaPrintf ("Master message size: %d\n", msgSize) ;
		}

		if (cc == 's')	// Slave message size
		{
			aaPrintf ("Slave message size (1..9): ") ;
			cc = aaGetChar () ;
			aaPutChar (cc) ;
			if (cc >= '1'  &&  cc <= '9')
			{
				slvMsgSize = (uint32_t) (cc - '0') ;
			}
			aaPrintf ("\nSlave message size: %d\n", slvMsgSize) ;
		}

		if (cc == 'B')	// Count of Master blocks
		{
			masterBlock = (masterBlock == 1u) ? 2u : 1u ;
			aaPrintf ("Master block blocks: %d\n", masterBlock) ;
		}

		if (cc == 'b')	// Count of Slave blocks
		{
			slvBlock = (slvBlock == 1u) ? 2u : 1u ;
			aaPrintf ("Slave blocks: %d\n", slvBlock) ;
		}

		if (cc == 'e')	// Master ReSTART on/off
		{
			masterRestart ^= 1u ;
			aaPrintf ("Master ReSTART: %d\n", masterRestart) ;
		}

		if (cc == 'g')
		{
			// Get: Master RX, slave TX
if (DBG_I2C != 0) swoSend16 ('\r\n', 0) ;
			i2cMasterRxSlaveTx (msgSize) ;
		}

		if (cc == 'p')
		{
			// Put: Master TX, slave RX
if (DBG_I2C != 0) swoSend16 ('\r\n', 0) ;
			i2cMasterTxSlaveRx (msgSize) ;
		}

		if (cc == '?')	// Print test configuration
		{
			aaPrintf ("\nMaster message size: %d\n", msgSize) ;
			aaPrintf ("Master ReSTART:      %d\n", masterRestart) ;
			aaPrintf ("Master blocks:       %d\n", masterBlock) ;
			aaPrintf ("Slave message size:  %d\n", slvMsgSize) ;
			aaPrintf ("Slave blocks:        %d\n", slvBlock) ;
		}

		if (cc == 'm')
		{
			ee24cTest () ;	// EEPROM 24Cxx test
		}
	}
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	EE24CXX test

#include	"aautils.h"
#include	"ee24Cxx.h"

/*	How to connect the EEPROM 24Cxx for the test:

	NUCLEO-STM32F429ZI
                      -------
	     GND A0 ------|1   8|------ VCC CN8-7
	     GND A1 ------|2   7|------ WP  NC
	     GND A2 ------|3   6|------ SCL PB10 CN7-32
	 CN8-11 GND ------|4   5|------ SDA PB11 CN7-34
	                  -------
*/

#define	EE24CADDDR	0xA0		// I2C address of the EEPROM (left shifted)

// Data buffer to write / read
static	uint8_t	eeBuffer [256] ;

//----------------------------------------------------------------------
//	Check the content of a read buffer

static	i2cStatus_t	eeCheck (uint8_t * pBuffer, uint32_t size)
{
	uint32_t	ii ;

	for (ii = 0u ; ii < size ; ii++)
	{
		if (pBuffer [ii] != ii)
		{
			return I2C_EFAIL ;
		}
	}
	return (I2C_EOK) ;
}

//----------------------------------------------------------------------

static	void	ee24cTest (void)
{
	ee24cDesc_t		eeDesc ;
	i2cStatus_t		sts ;
	uint32_t		size, offset, ii ;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Initialize the I2C interface used by the EEPROM
	sts = i2cConfigure (i2cGetDesc (MST_DESCID), I2CSPEED, TASK_SIGNAL, & eeDesc.i2cHdl) ;
	if (sts != I2C_ENONE)
	{
		aaPrintf ("i2cConfigure error %d %s\n", sts, i2cErrorText [sts]) ;
		AA_ASSERT (0) ;
		return ;
	}

	// Initialize the EEPROM descriptor
	sts = ee24cInit (& eeDesc, EE24C02, EE24CADDDR) ;
	if (sts != I2C_EOK)
	{
		aaPrintf ("ee24cInit error: %d %s\n", sts, i2cErrorText [sts]) ;
		return ;
	}

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Write 4 bytes at the beginning of page 0
	aaMEMSET (eeBuffer, 0, sizeof (eeBuffer)) ;
	for (ii = 0u ; ii < 4u ; ii++)
	{
		eeBuffer [ii] = (uint8_t) ii ;
	}

	size   = 4u ;
	offset = 0u ;
	sts = ee24cWrite (& eeDesc, offset, eeBuffer, size) ;
	if (sts == I2C_EOK)
	{
		aaPrintf ("Write page 0, %u bytes OK sts = %d %s\n", size, sts, i2cErrorText [sts]) ;
		aaDump (eeBuffer, size) ;
	}
	else
	{
		aaPrintf ("Write page 0 error sts = %d %s\n", sts, i2cErrorText [sts]) ;
		return ;
	}

	// Read page 0
	aaMEMSET (eeBuffer, 0, sizeof (eeBuffer)) ;
	size   = EE24C_PAGESIZE (eeDesc.eeType) ;
	offset = 0u ;
	sts = ee24cRead (& eeDesc, offset, eeBuffer, size) ;
	if (sts == I2C_EOK)
	{
		aaPrintf ("Read page 0, %u bytes OK sts = %d %s\n", size, sts, i2cErrorText [sts]) ;
		aaDump (eeBuffer, size) ;
		if (I2C_EFAIL == eeCheck (eeBuffer, 4))
		{
			aaPrintf ("Read content error\n") ;
			return ;
		}
	}
	else
	{
		aaPrintf ("Read page 0 error sts = %d %s\n", sts, i2cErrorText [sts]) ;
		return ;
	}

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Write more than 1 page, from middle of page 4
	aaMEMSET (eeBuffer, 0, sizeof (eeBuffer)) ;
	size   = 2 * EE24C_PAGESIZE (eeDesc.eeType) ;
	offset = (EE24C_PAGESIZE (eeDesc.eeType) * 4) + (EE24C_PAGESIZE (eeDesc.eeType) / 2) ;
	for (ii = 0u ; ii < 2u * EE24C_PAGESIZE (eeDesc.eeType) ; ii++)
	{
		eeBuffer [ii] = (uint8_t) ii ;
	}
	sts = ee24cWrite (& eeDesc, offset, eeBuffer, size) ;
	if (sts == I2C_EOK)
	{
		aaPrintf ("\nWrite page 4, %u bytes OK sts %d = %s\n", size, sts, i2cErrorText [sts]) ;
		aaDump (eeBuffer, size) ;
	}
	else
	{
		aaPrintf ("\nWrite page 4 error sts = %d %s\n", sts, i2cErrorText [sts]) ;
		return ;
	}

	// Read data from page 4
	aaMEMSET (eeBuffer, 0, sizeof (eeBuffer)) ;
	sts = ee24cRead (& eeDesc, offset, eeBuffer, size) ;
	if (sts == I2C_EOK)
	{
		aaPrintf ("Read page 4, %u bytes OK sts = %d %s\n", size, sts, i2cErrorText [sts]) ;
		aaDump (eeBuffer, size) ;
		if (I2C_EFAIL == eeCheck (eeBuffer, size))
		{
			aaPrintf ("Read content error\n") ;
			return ;
		}
	}
	else
	{
		aaPrintf ("Read page 4 error sts = %d %s\n", sts, i2cErrorText [sts]) ;
		return ;
	}

	aaPrintf ("EEPROM test OK\n") ;
}

//----------------------------------------------------------------------
#if (DBG_I2C != 0)
	#pragma GCC diagnostic pop
#endif
