/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	testspi.c	Test for SPI basic STM32F1/F4/L4/G0, polling and DMA

	When		Who	What
	12/04/18	ac	Creation
	27/02/20	ac	Port to STM32H7xx
	31/03/20	ac	Port to STM32L4+
	13/04/21	ac	Port to STM32G0/L4 with slave mode
	30/04/21	ac	Updated for F4 with slave mode

----------------------------------------------------------------------
*/

/*
	BEWARE: NSS management
	This code use the manual NSS management:
		For master SPI mode the user code uses SPI_NSS_USER configuration flag,
		and the spiSlaveSelect() function.
		In the SPI descriptor the NSS pin is defined as AA_GPIO_MODE_OUTPUT

		For slave SPI mode the user code uses SPI_NSS_USER configuration flag,
		In the SPI descriptor the NSS pin is defined as AA_GPIO_MODE_INPUT
		The NSS input use EXTI to detect the end of the message (NSS rising edge)

	To use automatic NSS management:
		Use the SPI_NSS_AUTO configuration flag, and don't use the spiSlaveSelect() function.
		In the SPI descriptor the NSS pin is defined as AA_GPIO_MODE_ALT_PP, with the proper AF
		PUT A PULL UP ON THE NSS PIN !!!  This is mandatory.
*/

#include	"aa.h"
#include	"aaprintf.h"
#include	"aalogmes.h"

#include	"test.h"
#include	"spibasic.h"
#include	"extibasic.h"
#include	"aautils.h"		// For memory dump

// Used for timing measurement with an oscilloscope.
//#define dbgPin0(v)		bspOutput (BSP_OUTPUT0, v) ;	// Output pulses
#define dbgPin0(v)											// No output pulses


#define		SPI_DESC		10u			// Master SPI descriptor ID in spiXXxx.h
#define		SPI_BAUDRATE	1000000u	// Hint for speed
#define		SPI_MAX			20u			// The number of items in the buffers

// For test of data size 8 bits
static		uint8_t		inBuffer8  [SPI_MAX] ;
static		uint8_t		outBuffer8 [SPI_MAX] ;

// For test of data size 16 bits
static		uint16_t	inBuffer16  [SPI_MAX] ;
static		uint16_t	outBuffer16 [SPI_MAX] ;

// The tests to execute
#define	T_POLL_TXONLY	0x01u
#define	T_POLL_TXRX		0x02u
#define	T_DMA_TXONLY	0x04u
#define	T_DMA_TXRX		0x08u

// Test configuration
static	uint32_t	configLast ;		// Flags of last spiMasterConfigure() call
static	spiHandle_t	hdl ;				// The master SPI handle returned by spiMasterConfigure()

//----------------------------------------------------------------------

static	void	test25xx (void) ;

//----------------------------------------------------------------------

static	uint32_t	compareBuffer (void * pIn1, void * pIn2, uint32_t byteCount)
{
	uint8_t		* p1 = (uint8_t *) pIn1 ;
	uint8_t		* p2 = (uint8_t *) pIn2 ;
	uint32_t	ii ;

	ii = byteCount ;
	while (ii--)
	{
		if (* p1++ != * p2++)
		{
			return 0u ;
		}
	}
	return 1u ;
}

//----------------------------------------------------------------------
//	Configure SPI only if not already configured
//	This allows to configure only once for many exchanges
//	Use SPI_NSS_USER, so the SPI descriptor should describe NSS pin accordingly

static	spiStatus_t	configureMaster (const spiDesc_t * pSpiDesc, uint32_t flags, spiHandle_t * pHdl)
{
	spiStatus_t			sts = SPI_ENONE ;

	if (flags != configLast)
	{
		aaPrintf ("Configure SPI: %08X\n", flags) ;
		configLast = flags ;
		sts = spiMasterConfigure (pSpiDesc, SPI_BAUDRATE, SPI_NSS_USER | SPI_MODE_0 | flags, pHdl) ;
//		sts = spiMasterConfigure (pSpiDesc, SPI_BAUDRATE, SPI_NSS_AUTO | SPI_MODE_0 | flags, pHdl) ;
		if (sts != SPI_ENONE)
		{
			aaPrintf ("spiMasterConfigure error: %d\n", sts) ;
			configLast = 1u ;	// Not configured
		}
	}
	return sts ;
}

//----------------------------------------------------------------------
//	SPI tests for 8 bits data words

static	uint32_t	spiTest8 (uint32_t tests)
{
	const spiDesc_t 	* pSpiDesc ;
	spiStatus_t			sts ;
	uint32_t			ii ;

	//	----------------------------------------------------------------
	aaPrintf ("\nSPI test, data size 8 ------------------\n") ;

dbgPin0 (0u) ;
	// Find the SPI descriptor with index 10
	if (spiGetDesc (SPI_DESC, & pSpiDesc) == SPI_EFAIL)
	{
		aaPrintf ("spiGetDesc error\n") ;
		return 0u ;
	}

	for (ii = 0u ; ii < SPI_MAX ; ii++)
	{
		outBuffer8 [ii] = (uint8_t) (ii + 1u) ;
	}

	if ((tests & T_POLL_TXONLY) != 0u)
	{
		// Tx only exchange, polling 	-----------------------------------------
		aaPrintf ("Tx only polling test:\n") ;

		// Initialize SPI master
		sts = configureMaster (pSpiDesc, SPI_DIR_HALF_TX, & hdl) ;
		spiSlaveSelect (hdl, 0u) ;

		aaMEMSET (inBuffer8, 0u, sizeof (inBuffer8)) ;

dbgPin0 (1u) ;
		spiSlaveSelect (hdl, 1u) ;
		sts = spiMasterExchange (hdl, outBuffer8, NULL, SPI_MAX) ;
		spiSlaveSelect (hdl, 0u) ;
dbgPin0 (0u) ;

		aaDump (outBuffer8, SPI_MAX) ;
		if (sts == SPI_ENONE)
		{
			aaPrintf ("Tx only polling spiMasterExchange OK\n") ;
		}
		else
		{
			aaPrintf ("Tx only polling spiMasterExchange error: %d\n", sts) ;
			return 0u ;
		}
		aaTaskDelay (100u) ;
	}

	if ((tests & T_DMA_TXONLY) != 0u)
	{
		// Tx only exchange, DMA 	-----------------------------------------
		aaPrintf ("Tx only DMA test:\n") ;

		// Initialize SPI master
		sts = configureMaster (pSpiDesc, SPI_DIR_HALF_TX, & hdl) ;
		spiSlaveSelect (hdl, 0u) ;

		aaMEMSET (inBuffer8, 0u, sizeof (inBuffer8)) ;

dbgPin0 (1u) ;
		spiSlaveSelect (hdl, 1u) ;
		sts = spiMasterExchangeDma (hdl, outBuffer8, NULL, SPI_MAX) ;
		spiSlaveSelect (hdl, 0u) ;
dbgPin0 (0u) ;

		aaDump (outBuffer8, SPI_MAX) ;
		if (sts == SPI_ENONE)
		{
			aaPrintf ("Tx only DMA spiDmaExchange OK\n") ;
		}
		else
		{
			aaPrintf ("Tx only DMA spiDmaExchange error: %d\n", sts) ;
			return 0u ;
		}
		aaTaskDelay (100u) ;
	}

	if ((tests & T_POLL_TXRX) != 0u)
	{
		// Full duplex exchange	polling-----------------------------------------
		aaPrintf ("\nTx/Rx polling test:\n") ;

		// Initialize SPI master
		sts = configureMaster (pSpiDesc, 0u, & hdl) ;
		spiSlaveSelect (hdl, 0u) ;

		aaMEMSET (inBuffer8, 0u, sizeof (inBuffer8)) ;

dbgPin0 (1u) ;
		spiSlaveSelect (hdl, 1u) ;
		sts = spiMasterExchange (hdl, outBuffer8, inBuffer8, SPI_MAX) ;
		spiSlaveSelect (hdl, 0u) ;
dbgPin0 (0u) ;

		if (sts == SPI_ENONE)
		{
			aaPrintf ("Tx/Rx polling exchange OK:\n") ;
			aaDump ((uint8_t *) outBuffer8, SPI_MAX) ;
			aaPrintf ("\n") ;
			aaDump ((uint8_t *) inBuffer8,  SPI_MAX) ;
			if (compareBuffer (outBuffer8, inBuffer8, SPI_MAX) == 0u)
			{
				aaPrintf ("Compare ERROR\n") ;
				return 0u ;
			}
			else
			{
				aaPrintf ("Compare OK\n") ;
			}
		}
		else
		{
			aaPrintf ("Tx/Rx polling exchange error: %d\n", sts) ;
			return 0u ;
		}
		aaTaskDelay (100u) ;
	}

	if ((tests & T_DMA_TXRX) != 0u)
	{
		// Full duplex exchange	DMA -----------------------------------------
		aaPrintf ("\nTx/Rx DMA test:\n") ;

		// Initialize SPI master
		sts = configureMaster (pSpiDesc, 0u, & hdl) ;
		spiSlaveSelect (hdl, 0u) ;

		aaMEMSET (inBuffer8, 0xFF, sizeof (inBuffer8)) ;

dbgPin0 (1u) ;
		spiSlaveSelect (hdl, 1u) ;
		sts = spiMasterExchangeDma (hdl, outBuffer8, inBuffer8, SPI_MAX) ;
		spiSlaveSelect (hdl, 0u) ;
dbgPin0 (0u) ;

		if (sts == SPI_ENONE)
		{
			aaPrintf ("Tx/Rx DMA exchange OK:\n") ;
			aaDump ((uint8_t *) outBuffer8, SPI_MAX) ;
			aaPrintf ("\n") ;
			aaDump ((uint8_t *) inBuffer8,  SPI_MAX) ;
			if (compareBuffer (outBuffer8, inBuffer8, SPI_MAX) == 0u)
			{
				aaPrintf ("Compare ERROR\n") ;
				return 0u ;
			}
			else
			{
				aaPrintf ("Compare OK\n") ;
			}
		}
		else
		{
			aaPrintf ("Tx/Rx DMA exchange error: %d\n", sts) ;
			return 0u ;
		}
		aaTaskDelay (100u) ;
	}

	return 1u ;	// Test OK
}

//----------------------------------------------------------------------
//	SPI tests for 16 bits data words

static	uint32_t	spiTest16 (uint32_t tests)
{
	const spiDesc_t 	* pSpiDesc ;
	spiStatus_t			sts ;

	aaPrintf ("\nSPI test, data size 16 -----------------\n") ;

	//	----------------------------------------------------------------
dbgPin0 (0u) ;
	// Find the SPI descriptor with the specified index
	if (spiGetDesc (SPI_DESC, & pSpiDesc) == SPI_EFAIL)
	{
		aaPrintf ("spiGetDesc error\n") ;
		return 0u ;
	}

	// Initialize buffers
	{
		uint32_t	ii ;

		for (ii = 0u ; ii < SPI_MAX ; ii++)
		{
			outBuffer16 [ii] = (uint16_t) (((ii*2+1) << 8u) | (ii*2u)) ;
		}
	}

	if ((tests & T_POLL_TXONLY) != 0u)
	{
		// Tx only exchange, polling 	-----------------------------------------
		aaPrintf ("Tx only polling test:\n") ;

		// Initialize SPI master
		sts = configureMaster (pSpiDesc, SPI_BITS_16 | SPI_DIR_HALF_TX, & hdl) ;
		spiSlaveSelect (hdl, 0u) ;

		aaMEMSET (inBuffer16, 0u, sizeof (inBuffer16)) ;

dbgPin0 (1u) ;
		spiSlaveSelect (hdl, 1u) ;
		sts = spiMasterExchange (hdl, outBuffer16, NULL, SPI_MAX) ;
		spiSlaveSelect (hdl, 0u) ;
dbgPin0 (0u) ;

		aaDump ((uint8_t *) outBuffer16, SPI_MAX * sizeof (* outBuffer16)) ;
		if (sts == SPI_ENONE)
		{
			aaPrintf ("Tx only polling spiMasterExchange OK\n") ;
		}
		else
		{
			aaPrintf ("Tx only polling spiMasterExchange error: %d\n", sts) ;
			return 0u ;
		}
	}

	if ((tests & T_DMA_TXONLY) != 0u)
	{
		// Tx only exchange, DMA 	-----------------------------------------
		aaPrintf ("Tx only DMA test:\n") ;

		// Initialize SPI master
		sts = configureMaster (pSpiDesc, SPI_BITS_16 | SPI_DIR_HALF_TX, & hdl) ;
		spiSlaveSelect (hdl, 0u) ;

		aaMEMSET (inBuffer16, 0u, sizeof (inBuffer16)) ;

dbgPin0 (1u) ;
		spiSlaveSelect (hdl, 1u) ;
		sts = spiMasterExchangeDma (hdl, outBuffer16, NULL, SPI_MAX) ;
		spiSlaveSelect (hdl, 0u) ;
dbgPin0 (0u) ;

		aaDump ((uint8_t *) outBuffer16, SPI_MAX * sizeof (* outBuffer16)) ;
		if (sts == SPI_ENONE)
		{
			aaPrintf ("Tx only DMA spiDmaExchange OK\n") ;
		}
		else
		{
			aaPrintf ("Tx only DMA spiDmaExchange error: %d\n", sts) ;
			return 0u ;
		}
		aaTaskDelay (100u) ;
	}

	if ((tests & T_POLL_TXRX) != 0u)
	{
		// Full duplex exchange	polling ----------------------------------
		aaPrintf ("\nTx/Rx polling test:\n") ;

		// Initialize SPI master
		sts = configureMaster (pSpiDesc, SPI_BITS_16, & hdl) ;
		spiSlaveSelect (hdl, 0u) ;

		aaMEMSET (inBuffer16, 0u, sizeof (inBuffer16)) ;

dbgPin0 (1u) ;
		spiSlaveSelect (hdl, 1u) ;
		sts = spiMasterExchange (hdl, outBuffer16, inBuffer16, SPI_MAX) ;
		spiSlaveSelect (hdl, 0u) ;
dbgPin0 (0u) ;

		if (sts == SPI_ENONE)
		{
			aaPrintf ("Tx/Rx polling exchange OK:\n") ;
			aaDump ((uint8_t *) outBuffer16, SPI_MAX * sizeof (* outBuffer16)) ;
			aaPrintf ("\n") ;
			aaDump ((uint8_t *) inBuffer16,  SPI_MAX * sizeof (* inBuffer16)) ;
			if (compareBuffer (outBuffer16, inBuffer16, SPI_MAX * sizeof (* outBuffer16)) == 0u)
			{
				aaPrintf ("Compare ERROR\n") ;
				return 0u ;
			}
			else
			{
				aaPrintf ("Compare OK\n") ;
			}
		}
		else
		{
			aaPrintf ("Tx/Rx polling exchange error: %d\n", sts) ;
			return 0u ;
		}
		aaTaskDelay (100u) ;
	}

	if ((tests & T_DMA_TXRX) != 0u)
	{
		// Full duplex exchange	DMA -----------------------------------------
		aaPrintf ("\nTx/Rx DMA test:\n") ;

		// Initialize SPI master
		sts = configureMaster (pSpiDesc, SPI_BITS_16, & hdl) ;
		spiSlaveSelect (hdl, 0u) ;

		aaMEMSET (inBuffer16, 0u, sizeof (inBuffer16)) ;

dbgPin0 (1u) ;
		spiSlaveSelect (hdl, 1u) ;
		sts = spiMasterExchangeDma (hdl, outBuffer16, inBuffer16, SPI_MAX) ;
		spiSlaveSelect (hdl, 0u) ;
dbgPin0 (0u) ;

		if (sts == SPI_ENONE)
		{
			aaPrintf ("Tx/Rx DMA exchange OK:\n") ;
			aaDump ((uint8_t *) outBuffer16, SPI_MAX * sizeof (* outBuffer16)) ;
			aaPrintf ("\n") ;
			aaDump ((uint8_t *) inBuffer16,  SPI_MAX * sizeof (* inBuffer16)) ;
			if (compareBuffer (outBuffer16, inBuffer16, SPI_MAX * sizeof (* outBuffer16)) == 0u)
			{
				aaPrintf ("Compare ERROR\n") ;
				return 0u ;
			}
			else
			{
				aaPrintf ("Compare OK\n") ;
			}
		}
		else
		{
			aaPrintf ("Tx/Rx DMA exchange error: %d\n", sts) ;
			return 0u ;
		}
		aaTaskDelay (100u) ;
	}

	return 1u ;		// Tests OK
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	Test of slave API. Use 2 SPI interfaces on the same board

//	Material links to be made. Best to have pull-up on NSS, and pull-down on SCK (if SPI mode is 0 or 1)
//	Master SCK  to slave SCK
//	Master MOSI to slave MOSI
//	Master MISO to slave MISO
//	Master NSS  to slave NSS

//	The menu sequences for slave SPI are:
//	s b			Init slave for 8 bits data, slave RX only
//	1 b			Master config TX only and exchange for 8 bits data
//	c			Check slave received data

//	s d			Init slave for 8 bits data, slave TX+RX
//	1 d			Master config TX+RX and exchange for 8 bits data
//	c			Check slave received data

//	S b			Init slave for 16 bits data, slave RX only
//	2 b			Master config TX only and exchange for 16 bits data
//	c			Check slave received data

//	S d			Init slave for 16 bits data, slave TX+RX
//	2 d			Master config TX+RX and exchange for 16 bits data
//	c			Check slave received data

#define				SPI_SLAVE_DESC		20u			// Slave SPI descriptor ID in spiG0xx.h

const spiDesc_t 	* pSpiSlaveDesc ;
spiHandle_t			hdlSlave ;
uint32_t			slvState ;

// For test of data size 8 and 16 bits
static	uint16_t	inSlvBuffer16 [SPI_MAX*3] ;
static	uint8_t		inSlvBuffer8  [SPI_MAX*3] ;

static	void		nssExtiInit		(void) ;

//----------------------------------------------------------------------
//	Initialize slave SPI for 16 bits data

static	uint32_t	spiSlaveInit16 (uint32_t flags)
{
	spiStatus_t		sts ;

	(void) tests ;

	// Initialize only once
	if (slvState != 16)
	{
		// Fully configure SPI only on configuration change
		if (spiGetDesc (SPI_SLAVE_DESC, & pSpiSlaveDesc) != SPI_ENONE)
		{
			aaPrintf ("spiGetDesc %u error\n", SPI_SLAVE_DESC) ;
			return 0 ;
		}

		sts = spiSlaveConfigure (pSpiSlaveDesc, SPI_BITS_16 | SPI_NSS_USER | SPI_MODE_0 | flags, & hdlSlave) ;
		if (sts != SPI_ENONE)
		{
			aaPrintf ("spiSlaveConfigure error: %d\n", sts) ;
		}
		slvState = 16 ;
	}
	else
	{
		// Configuration not changed:
		// Reset and configure again the SPI because there is no way to flush the TX FIFO... Sigh...
		spiConfigAgain (hdlSlave) ;
	}

	// Initialize the transmit buffers before executing spiSlaveExchangeDma:
	// because the TX DMA immediately fills the SPI FIFO
	{
		uint32_t	ii ;

		for (ii = 0u ; ii < SPI_MAX ; ii++)
		{
			outBuffer16 [ii] = (uint16_t) (((ii*2+1) << 8u) | (ii*2u)) ;
		}
	}

	aaMEMSET (inSlvBuffer16, 0, sizeof (inSlvBuffer16)) ;

	if (flags == SPI_DIR_RX_ONLY)
	{
		// Slave RX only
		sts = spiSlaveExchangeDma (hdlSlave, NULL, inSlvBuffer16, SPI_MAX+2) ;
	}
	else
	{
		// Slave full duplex
		sts = spiSlaveExchangeDma (hdlSlave, outBuffer16, inSlvBuffer16, SPI_MAX+2) ;
	}
	if (sts != SPI_ENONE)
	{
		aaPrintf ("spiSlaveExchangeDma error: %d\n", sts) ;
	}

	// Initialize interrupts on NSS edges
	nssExtiInit () ;

	return 1u ;		// OK
}

//----------------------------------------------------------------------
//	Initialize slave SPI for 8 bits data

static	uint32_t	spiSlaveInit8 (uint32_t flags)
{
	spiStatus_t		sts ;

	(void) tests ;

	// Initialize only once
	if (slvState != 8)
	{
		// Fully configure SPI only on configuration change
		if (spiGetDesc (SPI_SLAVE_DESC, & pSpiSlaveDesc) != SPI_ENONE)
		{
			aaPrintf ("spiGetDesc %u error\n", SPI_SLAVE_DESC) ;
			return 0 ;
		}

		sts = spiSlaveConfigure (pSpiSlaveDesc, SPI_BITS_8 | SPI_NSS_USER | SPI_MODE_0 | flags, & hdlSlave) ;
		if (sts != SPI_ENONE)
		{
			aaPrintf ("spiSlaveConfigure error: %d\n", sts) ;
		}
		slvState = 8 ;
	}
	else
	{
		// Configuration not changed:
		// Reset and configure again the SPI because there is no way to flush the TX FIFO... Sigh...
		spiConfigAgain (hdlSlave) ;
	}

	// Initialize the transmit buffers before executing spiSlaveExchangeDma:
	// because the TX DMA immediately fills the SPI FIFO
	{
		uint32_t	ii ;

		for (ii = 0u ; ii < SPI_MAX ; ii++)
		{
			outBuffer8 [ii] = (uint8_t) (ii + 1u) ;
		}
	}

	aaMEMSET (inSlvBuffer8, 0, sizeof (inSlvBuffer8)) ;

	if (flags == SPI_DIR_RX_ONLY)
	{
		// Slave RX only
		sts = spiSlaveExchangeDma (hdlSlave, NULL, inSlvBuffer8, SPI_MAX+2) ;
	}
	else
	{
		// Slave full duplex
		sts = spiSlaveExchangeDma (hdlSlave, outBuffer8, inSlvBuffer8, SPI_MAX+2) ;
	}
	if (sts != SPI_ENONE)
	{
		aaPrintf ("spiSlaveExchangeDma error: %d\n", sts) ;
	}

	// Initialize interrupts on NSS edges
	nssExtiInit () ;

	return 1u ;		// OK
}

//----------------------------------------------------------------------

static	void	spiSlaveCheck (void)
{
	(void) tests ;

	if (slvState == 8)
	{
		// Stop SPI
		spiSlaveAbort (hdlSlave) ;

		// Compare buffers
		aaPrintf ("Slave Rx DMA exchange:\n") ;
		aaDump ((uint8_t *) outBuffer8, SPI_MAX * sizeof (* outBuffer8)) ;
		aaPrintf ("\n") ;
		aaDump ((uint8_t *) inSlvBuffer8,  SPI_MAX * sizeof (* inBuffer8)) ;
		if (compareBuffer (outBuffer8, inSlvBuffer8, SPI_MAX * sizeof (* outBuffer8)) == 0u)
		{
			aaPrintf ("Compare ERROR\n") ;
		}
		else
		{
			aaPrintf ("Compare OK\n") ;
		}
	}

	if (slvState == 16)
	{
		// Stop SPI
		spiSlaveAbort (hdlSlave) ;

		// Compare buffers
		aaPrintf ("Slave Rx DMA exchange:\n") ;
		aaDump ((uint8_t *) outBuffer16, SPI_MAX * sizeof (* outBuffer16)) ;
		aaPrintf ("\n") ;
		aaDump ((uint8_t *) inSlvBuffer16,  SPI_MAX * sizeof (* inBuffer16)) ;
		if (compareBuffer (outBuffer16, inSlvBuffer16, SPI_MAX * sizeof (* outBuffer16)) == 0u)
		{
			aaPrintf ("Compare ERROR\n") ;
		}
		else
		{
			aaPrintf ("Compare OK\n") ;
		}
	}
}

//----------------------------------------------------------------------
// Slave NSS management: interrupt on falling and rising edge
// The Slave SPI NSS is PA15

// You must enable the required EXTI interrupt handler in extibasic.h
#define	SPI_NSS_EXTI_PORT	'A'
#define	SPI_NSS_EXTI_LINE	15


void	nssExtiInit (void)
{
	// Initialize the NSS input pin
	gpioPinDesc_t	pinDesc = {SPI_NSS_EXTI_PORT, SPI_NSS_EXTI_LINE, 0, AA_GPIO_MODE_INPUT_UP } ;

	extiInitGpio (& pinDesc, BSP_IRQPRIOMIN_PLUS(1), EXTI_FLAG_INTR | EXTI_FLAG_RISING) ;
}

// Remember that F4/L4/H7 does not provide the direction of the signal edge
// G0 manages both edges independently
void EXTI_CALLBACK(SPI_NSS_EXTI_LINE) (uint32_t flags)
{
	if ((flags & EXTI_FLAG_RISING) != 0)
	{
		aaLogMes ("R\n", 0, 0, 0, 0, 0) ;
	}
	if ((flags & EXTI_FLAG_FALLING) != 0)
	{
		aaLogMes ("F\n", 0, 0, 0, 0, 0) ;
	}
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*	Interactive SPI master test (connect MOSI/ to MISO)

 	Use master SPI1 on NUCLEO-F103RB/F446RB/G071RB (LQFP64)
		MOSI:	PA7	CN5-4
		MISO:	PA6	CN5-5
		SCK:	PA5	CN5-6
		NSS:	PA4 CN8-3 (option)

	Use SPI1 on NUCLEO-F429ZI/L4R5ZI/H743ZI (LQFP 144):
		MOSI:	PA7	CN7-14
		MISO:	PA6	CN7-12
		SCK:	PA5	CN7-10
		NSS:	PA4 CN7-17

	For Slave test: connect master SPI1 to slave SPI2 (Nucleo L4R5)
		MOSI:	PD4	CN9-8
		MISO:	PD3	CN9-10
		SCK:	PD1	CN9-27	internal pull down enabled
		NSS:	PD0 CN9-25	internal pull up enabled

	BlackPillF411:
		master SPI1					slave SPI2
		MOSI:	PA7	17				PB15	28
		MISO:	PA6	16				PB14	27
		SCK:	PA5	15				PB13	26	internal pull down enabled
		NSS:	PA4 14				PB12	25	internal pull up enabled
*/

void	spiTest (void)
{
	char		cc ;
	char		ccArg ;
	char		tests ;

	aaPrintf ("SPI test, h for test menu\n") ;
	aaPrintf ("To test  8 bits TX+RX with DMA:  s d 1 d\n") ;
	aaPrintf ("To test 16 bits TX+RX with DMA:  S d 2 d\n") ;

	// Display real baud rate
	{
		uint32_t 			br ;
		const spiDesc_t 	* pSpiDesc ;

		if (spiGetDesc (SPI_DESC, & pSpiDesc) != SPI_EFAIL)
		{
			br = spiSpeedToBr (pSpiDesc, SPI_BAUDRATE) ;
			br = spiBrToSpeed (pSpiDesc, br) ;
			aaPrintf ("Baudrate required %d.%06d, real:%d.%06d MHz\n",
					SPI_BAUDRATE / 1000000u, SPI_BAUDRATE % 1000000u,
					br / 1000000u, br % 1000000u) ;
		}
	}

	tests = 0u ;			// Avoid compiler warning
	configLast = 1u ;		// Force SPI configuration, different of any SPI_DIR_xx
	while (1)
	{
		cc = aaGetChar () ;
		aaPutChar (cc) ;

		if (cc == ' ')		// Line space on message log terminal
		{
			aaLogMes ("\n", 0u, 0u, 0u, 0u, 0u) ;
		}

		if (cc == 'h')
		{
			aaPrintf ("\n") ;
			aaPrintf ("1 v    Test  8 bits data size\n") ;
			aaPrintf ("2 v    Test 16 bits data size\n") ;
			aaPrintf ("  v    a  Polling TX only\n") ;
			aaPrintf ("  v    b  DMA     TX only\n") ;
			aaPrintf ("  v    c  Polling TX RX\n") ;
			aaPrintf ("  v    d  DMA     TX RX\n") ;
			aaPrintf ("m      EEPROM 25xx test\n") ;
			aaPrintf ("s v    8  bits slave init\n") ;
			aaPrintf ("  v    b  DMA     RX only\n") ;
			aaPrintf ("  v    d  DMA     TX RX\n") ;
			aaPrintf ("S v    16 bits slave init\n") ;
			aaPrintf ("  v    b  DMA     TX only\n") ;
			aaPrintf ("  v    d  DMA     TX RX\n") ;
			aaPrintf ("c      Slave  Check\n") ;
			aaPrintf ("q      Quit test\n") ;
			continue ;
		}

		if (cc == 's')		// Slave 8 bits data
		{
			// Get arg char
			aaPutChar (' ') ;
			ccArg = aaGetChar () ;
			aaPutChar (ccArg) ;
			aaPutChar ('\n') ;
			if (ccArg == 'b')
			{
				spiSlaveInit8 (SPI_DIR_RX_ONLY) ;
			}
			if (ccArg == 'd')
			{
				spiSlaveInit8 (SPI_DIR_FULL_DUPLEX) ;
			}
			continue ;
		}

		if (cc == 'S')		// Slave 16 bits data
		{
			// Get arg char
			aaPutChar (' ') ;
			ccArg = aaGetChar () ;
			aaPutChar (ccArg) ;
			aaPutChar ('\n') ;
			if (ccArg == 'b')
			{
				spiSlaveInit16 (SPI_DIR_RX_ONLY) ;
			}
			if (ccArg == 'd')
			{
				spiSlaveInit16 (SPI_DIR_FULL_DUPLEX) ;
			}
			continue ;
		}
		if (cc == 'c')
		{
			aaPutChar ('\n') ;
			spiSlaveCheck () ;
			continue ;
		}

		if (cc == '1' || cc == '2')
		{
			// Get arg char
			aaPutChar (' ') ;
			ccArg = aaGetChar () ;
			aaPutChar (ccArg) ;
			if (ccArg == 'a')
			{
				tests = T_POLL_TXONLY ;
			}
			else if (ccArg == 'b')
			{
				tests = T_DMA_TXONLY ;
			}
			else if (ccArg == 'c')
			{
				tests = T_POLL_TXRX ;
			}
			else if (ccArg == 'd')
			{
				tests = T_DMA_TXRX ;
			}
			else
			{
				aaPutChar ('\n') ;
				continue ;		// Bad argument
			}
		}
		aaPutChar ('\n') ;

		if (cc == 'q')
		{
			if (slvState != 0)
			{
				// Disable NSS EXTI interrupt
				extiResetLine (SPI_NSS_EXTI_LINE) ;
			}
			break ;	// End of test
		}

		if (cc == '1')
		{
			spiTest8 (tests) ;
		}

		if (cc == '2')
		{
			spiTest16 (tests) ;
		}

		if (cc == 'm')
		{
			test25xx () ;		// EEPROM test
		}
	}
	aaPrintf ("SPI test ended\n") ;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	Test EEPROM 25xx128

#include	"spi25xx.h"
#include	"gpiobasic.h"

/*	How to connect the EEPROM for the test (NUCLEO-STM32F429ZI ?):
  	 	 	 	 	 	  --------
	PA4/CN7-17  NSS ------|1    8|------ VCC      CN10-1
	PA6/CN7-12 MISO ------|2    7|------ H/   VCC CN10-1
	VCC CN10-1   W/ ------|3    6|------ CLK  PA5/CN7-10
	    CN10-5  GND ------|4    5|------ MOSI PB5/CN7-13
	                      --------
*/

//	The selection of the EEPROM as SPI slave (Chip Select) can be made by two means:
//
//	1) Use the pin described in the NSS field of the SPI descriptor with SPI_NSS_USER.
//	In this case the pSlaveSelect field in spi25xxDesc_t should be NULL,
//	and the NSS pin descriptor should be AA_GPIO_MODE_OUTPUT (with AF 0)
//
//	2) The user manages himself a GPIO pin. This allows to manage multiple slave devices on the same SPI port.
//	In this case the user should enable the GPIO pin as output and set the output to 1.
//	Then define a callback function and use the name of this function as pSlaveSelect field in spi25xxDesc_t

#define		EE_WITH_SLAVESELECT	0		// If 1 use application specific SPI slave select GPIO pin
										// If 0 use the NSS pin in the SPI descriptor

#define		EE_SPI_ID			10u		// Id of SPI descriptor to use
#define		EE_ADDR				96u		// EEPROM address of the data
#define		EE_SIZE				256u	// Size of the data to write/read

static	spi25xxDesc_t	ee25 ;
static	uint8_t			eeBuffer [EE_SIZE] ;


#if (EE_WITH_SLAVESELECT == 1)

	// With application specific SPI slave select GPIO pin
	gpioPinDesc_t	selPin = {	'A',	4u,	AA_GPIO_AF_0,	AA_GPIO_MODE_OUTPUT_PP | AA_GPIO_SPEED_LOW } ;

	void cb25SlaveSelect (spiHandle_t spiHdl, uint32_t bSelect) ;

	void cb25SlaveSelect (spiHandle_t spiHdl, uint32_t bSelect)
	{
		//	SPI slave select is active low: so bSelect 1 => pin output 0
		gpioPinSet (& selPin, (bSelect & 1u) ^ 1u) ;
		(void) spiHdl ;
	}
	#define	EE_CB	cb25SlaveSelect		// Use callback function to select SPI slave

#else

	// Use SPI descriptor slave select pin
	#define	EE_CB	NULL				// Don't use callback function to select SPI slave

#endif

static	void	test25xx (void)
{
	uint32_t	ii ;
	uint8_t		* p8 ;

	#if (EE_WITH_SLAVESELECT == 1)
	{
		// User slave select
		gpioConfigurePin (& selPin) ;
		gpioPinSet (& selPin, 1u) ;
		ee25.pSlaveSelect = cb25SlaveSelect ;
	}
	#else
	{
		// The library use the SPI descriptor to select the slave
		ee25.pSlaveSelect = NULL ;
	}
	#endif

	// Get SPI descriptor from ID
	if (spiGetDesc (EE_SPI_ID, & ee25.pSpiDesc) == SPI_EFAIL)
	{
		aaPrintf ("spiGetDesc error\n") ;
		return ;
	}

	// Initialize SPI for EEPROM
	if (SPI_ENONE != spi25xxOpen (& ee25, SPI25XX128, 6000000u))
	{
		aaPrintf ("spi25Open error\n") ;
		return ;
	}

	// Initialize data buffer
	p8 = eeBuffer ;
	for (ii = 0u ; ii < EE_SIZE ; ii++)
	{
		* p8++ = (uint8_t) ii ;
	}

	// Write data
	if (SPI_ENONE != spi25xxWrite (& ee25, EE_ADDR, eeBuffer, EE_SIZE))
	{
		aaPrintf ("spi25xxWrite error\n") ;
		spi25xxClose (& ee25) ;
		return ;
	}
	aaPrintf ("Write OK\n") ;

	// Reset data buffer
	aaMEMSET (eeBuffer,  0x00u, EE_SIZE) ;

	// Read EEPROM data
	if (SPI_ENONE != spi25xxRead (& ee25, EE_ADDR, eeBuffer, EE_SIZE))
	{
		aaPrintf ("spi25xxRead error\n") ;
		spi25xxClose (& ee25) ;
		return ;
	}
	spi25xxClose (& ee25) ;

	// Check data
	p8 = eeBuffer ;
	for (ii = 0u ; ii < EE_SIZE ; ii++)
	{
		if (* p8 != ii)
		{
			aaPrintf ("Check error addr: %04X  data: %02X  expected: %02X\n",  EE_ADDR+ii, * p8, ii) ;
			break ;
		}
		p8++ ;
	}
	if (ii == EE_SIZE)
	{
		aaPrintf ("Read OK\n") ;
	}
}

//----------------------------------------------------------------------
